/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/
 
function tr(s) { return s; }



/*
	loader widget definition starts here
*/

function QtAppLoader(parent, runOutsideLively)
{
    QWidget.call(this, parent);

    // Default app loader size
    this.defaultWidth = 600;
    this.defaultHeight = 40;

	this.styleSheet = "QLineEdit { font-size: 16px; font-weight: bold; border: 1px solid white }";

	this.acceptDrops = true;

	this.url = new QUrl();

	this.manager = new QNetworkAccessManager();
	this.manager["finished(QNetworkReply*)"].connect(this, this.handleNetworkReply);

	this.lineEdit = new QLineEdit(this);
	this.lineEdit["returnPressed"].connect(this, function() {
		this.url = new QUrl(this.lineEdit.text, QUrl.TolerantMode);
		this.manager.get(new QNetworkRequest(this.url));
	});
	this.loadBtn = new QPushButton("Load", this);
	this.loadBtn["clicked"].connect(this, function() {
		this.url = new QUrl(this.lineEdit.text, QUrl.TolerantMode);
		this.manager.get(new QNetworkRequest(this.url));
	});
	
	/* Check if this application is executed as standalone app */
	if(runOutsideLively) {
		this.runOutsideLively = true;
	}
	else {
		this.runOutsideLively = false;
	}
	
	/* Get all functions of object called Global inside Lively */
	if(this.runOutsideLively == false) {
		this.globalFunctionNames = this.getObjectFunctions(Global);
		this.suggestionList = this.globalFunctionNames;
	}
	else {
		this.suggestionList = new Array();
	}
	
	/* One may want to use this.runOutsideLively as a condition 
	   because the graphics cet a bit messed up
	   inside Lively environment when the completer pops up */
	if(true) {
		/* Go trough all the files inside current directory */
		this.dirIterator = new QDirIterator(".", "*", QDir.Files, QDirIterator.NoIteratorFlags);
		while (this.dirIterator.hasNext()) {
			/* Put filenames to QCompleter suggestion list */
			this.suggestionList.push(this.dirIterator.fileName());
			this.dirIterator.next();
		}
		
		/* Create QCompleter that suggests alternatives to the user */
		this.completer = new QCompleter(this.suggestionList, this);
		this.completer.caseSensitivity = Qt.CaseInsensitive;
		this.completer.completionMode = QCompleter.InlineCompletion; // ei vaikutusta!!
		this.completer["highlighted(QString)"].connect(this, 
			function(text) {
				this.lineEdit.text = text;
			}
		);
		this.lineEdit.setCompleter(this.completer);
	}
	
	this.layout = new QGridLayout(this);
	this.layout.addWidget(this.lineEdit, 0,0,0);
	this.layout.addWidget(this.loadBtn,0,1,0);

	this.layout.setContentsMargins(2,2,2,2);
	this.windowTitle = tr("Application Loader");
	this.resize(this.defaultWidth, this.defaultHeight);
}

QtAppLoader.prototype = new QWidget();

QtAppLoader.prototype.getObjectFunctions = function (obj) {
	var functionsArray = new Array();
	print("All functions of the object:");
	for(var name in obj) {
		if ((obj[name] instanceof Function)) {
			print(name);
			functionsArray.push(name);
		}
	}
	return functionsArray;
}

QtAppLoader.prototype.handleNetworkReply = function(reply) {
	var status = reply.attribute(QNetworkRequest.HttpStatusCodeAttribute);
	
	// If status is undefined, the request was most likely for a local file.
	// If it is defined, a web file was requested.
	// In the latter case we can check if the HTTP response was not 200.
	if(status && status != 200) {
		if(this.runOutsideLively != true) {
			// Create WebVidget and load html page into it
			var wdgt = new QWebView();
			wdgt.load(new QUrl(this.url));
			wdgt.settings().setAttribute(QWebSettings.PluginsEnabled, true);
			var newObject = new QtProxyMorph(wdgt, qrect(0, 0, 600, 400));
			newObject.setPos(QtEvents.lastMouseDownPos);
			that = QtWorldMorph.current().addMorph(newObject);
		} else {
			this.errorMsgBox("Oops! HTTP Error: " + status);
		}
		return;
	}
	
	var path = new QFileInfo(this.url.path());
	var fileName = path.completeBaseName();	
	
	// At first we try to create an object that is already defined on Lively
	if(this.runOutsideLively == false && 
		this.globalFunctionNames.indexOf(fileName) != -1
		&& !status) {
		try{
			print("found a global object");
			this.createObjectInsideProxyMorph(fileName);
			return;
		}
		catch(err) {
			this.errorMsgBox(err + " (" + fileName + " in Global object)");
		}
	}
	
	// The object was not already defined, so we evaluate what we have possibly got
	// and try to create object that has the same name as the file
	var data = reply.readAll();
	try {
		print(data.toString());
		var newStuff = eval(data.toString());
		
		if(this.runOutsideLively != true) {
			eval('var wdgt = new ' + fileName +'();');
			var newObject = new QtProxyMorph(wdgt);
			newObject.setPos(QtEvents.lastMouseDownPos);
			that = QtWorldMorph.current().addMorph(newObject);
		}
	}
	catch (err) {
		if(this.runOutsideLively != true
			&& status) {
			// Create WebVidget and load html page into it
			var wdgt = new QWebView();
			wdgt.load(new QUrl(this.url));
			wdgt.settings().setAttribute(QWebSettings.PluginsEnabled, true);
			var newObject = new QtProxyMorph(wdgt, qrect(0, 0, 600, 400));
			newObject.setPos(QtEvents.lastMouseDownPos);
			that = QtWorldMorph.current().addMorph(newObject);
		}
		else {
			this.errorMsgBox(err + " (" + fileName + " in loaded data) \n" +
				"Try these: \n" +
				"- Check that the local file name matches the class name\n" +
				"- Check that you wrote the URL with \"http://\"");
		}
	}
}

QtAppLoader.prototype.createObjectInsideProxyMorph = function (name) {
	eval('var wdgt = new ' + name +'();');
	var newObject = new QtProxyMorph(wdgt);
	newObject.setPos(QtEvents.lastMouseDownPos);
	that = QtWorldMorph.current().addMorph(newObject);
};

QtAppLoader.prototype.errorMsgBox = function (msg) {
		print(msg);
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + msg;
		messageBox.show();
};

QtAppLoader.prototype.dragMoveEvent = function (event) {
	if (event.mimeData().hasUrls()) {
		//event.setDropAction(this.dropEvent);
		event.accept(this.rect);
		print("dropImg(dropMoveEvent): acceptAction");
	}
	print("dropImg: dropMoveEvent");
};

QtAppLoader.prototype.dragEnterEvent = function (event) {
	if (event.mimeData().hasUrls()) {
		//event.setDropAction(this.dropEvent);
		event.acceptProposedAction();
		print("dropImg: acceptAction");
	}
	print("dropImg: dragEnterEvent");
};

QtAppLoader.prototype.dropEvent = function (event) {
	var urls = event.mimeData().urls();
	
	print("mimedata, text: " + event.mimeData().text());
	
	print("dropImg: urls: " + urls + " urls.length: " + urls.length);
	if(urls.length) {
		print("dropImg: there was urls");
		try {
			this.url = new QUrl(urls, QUrl.TolerantMode);
			this.lineEdit.text = this.url;
			this.setWindowTitle(urls);
		}
		catch(err) {
			return;
		}

		this.manager.get(new QNetworkRequest(this.url));

		event.acceptProposedAction();
	}
};

QtAppLoader.prototype.mousePressEvent = function (event) {
};


QtAppLoader.prototype.mousePressEvent = function (event) {
	if (event.button() === Qt.LeftButton) {
        var drag = new QDrag(this);
        var mimeData = new QMimeData();
		mimeData.setData("text/uri-list", new QByteArray(this.url.toString()));
		print("MousePressEvent: this.url: " + this.url);
        drag.setMimeData(mimeData);
        drag.exec();
	}
};


QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   var codec = QTextCodec.codecForName(new QByteArray("ISO-8859-1"));
   ts.setCodec(codec);
   return ts.readAll();
};

/* To use the application in standalone mode, remove the comment around this code  

var wdgt = new QtAppLoader(0, true);
wdgt.show();

QCoreApplication.exec();
*/