/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

// TODO: Add some credit to original creator of this game!

CubeCrashGrid.prototype = new QWidget();

CubeCrashGrid.prototype.CUBE_SIZE = 24;
CubeCrashGrid.prototype.NO_CUBE = 999;
CubeCrashGrid.prototype.DARKER = 6;
//Original colors:
/*
CubeCrashGrid.prototype.COLOR_TABLE = { 999: new QColor(0, 0, 0),
                                        0: new QColor(255, 0, 0),
                                        1: new QColor(0, 255, 0),
                                        2: new QColor(0, 0, 255),
                                        3: new QColor(255, 255, 0),
                                        4: new QColor(255, 0, 255),
                                        5: new QColor(0, 255, 255) };
*/
// Low contrast green colors:
CubeCrashGrid.prototype.COLOR_TABLE = { 999: new QColor(0, 0, 0),
                                        0: new QColor(192, 128, 64),
                                        1: new QColor(64, 255, 64),
                                        2: new QColor(64, 128, 192),
                                        3: new QColor(192, 255, 64),
                                        4: new QColor(192, 128, 192),
                                        5: new QColor(64, 255, 192),
                                        6: new QColor(96, 64, 32),
                                        7: new QColor(32, 128, 32),
                                        8: new QColor(32, 64, 96),
                                        9: new QColor(96, 128, 32),
                                        10: new QColor(96, 64, 96),
                                        11: new QColor(32, 128, 96) };
// Low contrast red colors:
/*
CubeCrashGrid.prototype.COLOR_TABLE = { 999: new QColor(0, 0, 0),
                                        0: new QColor(255, 64, 64),
                                        1: new QColor(128, 192, 64),
                                        2: new QColor(128, 64, 192),
                                        3: new QColor(255, 192, 64),
                                        4: new QColor(255, 64, 192),
                                        5: new QColor(128, 192, 192) };
*/
CubeCrashGrid.prototype.TURN_DELAY = 1000 / 25;
CubeCrashGrid.prototype.TURN_SPEED = Math.PI * 0.75; // Radians per second


function CubeCrashGrid(parent, owner, cubes_width, cubes_height, colors)
{

    QWidget.call(this, parent);

    this.owner = owner;
    this.colors = colors;

    this.cubes_width = cubes_width;
    this.cubes_height = cubes_height;

    this.timer = new QTimer(this);
    this.timer.timeout.connect(this, this.run);

    this.mouseTracking = true;

    this.activated_cube_angle = 0;

    this.restart(colors);

}

CubeCrashGrid.prototype.getSizeHint = function()
{
    return new QSize(this.cubes_width * this.CUBE_SIZE,
                     this.cubes_height * this.CUBE_SIZE);
}

CubeCrashGrid.prototype.getMinimumSizeHint = function()
{
    return new QSize(this.cubes_width * this.CUBE_SIZE,
                     this.cubes_height * this.CUBE_SIZE);
}

CubeCrashGrid.prototype.getColors = function()
{
    return this.colors;
}

CubeCrashGrid.prototype.restart = function(colors)
{
    this.colors = colors;
    this.cubes = [];
    for (var cubes_id = 0;
         cubes_id < this.cubes_width * this.cubes_height;
         cubes_id ++) {
        this.cubes[cubes_id] = { color: Math.floor(Math.random() * colors),
                                 active: false };
    }
    this.shrapnels = [];
    this.update();
}

CubeCrashGrid.prototype.run = function()
{
    if (!this.underMouse()) {
        // Clear active cubes.
        for (var clear_offset = 0;
             clear_offset < this.cubes_width * this.cubes_height;
             clear_offset ++) {
            this.cubes[clear_offset].active = false;
        }
    } else {
        this.activated_cube_angle += this.TURN_SPEED / 1000.0 * this.TURN_DELAY;
    }
    if (!this.underMouse() && this.shrapnels.length == 0) {
        this.timer.stop();
        this.update();
        return;
    }
    for (var shrapnels_id = 0; shrapnels_id < this.shrapnels.length; shrapnels_id ++) {
        var s = this.shrapnels[shrapnels_id];
        s.angle += this.TURN_SPEED / 1000.0 * this.TURN_DELAY;
        s.x += s.x_vel * this.TURN_DELAY;
        s.y += s.y_vel * this.TURN_DELAY;
        s.y_vel += 0.001 * this.TURN_DELAY;
        // Check if shrapnel can be destroyed
        if (s.y > this.cubes_height * this.CUBE_SIZE * 1.5) {
            this.shrapnels[shrapnels_id] = this.shrapnels[this.shrapnels.length-1];
            this.shrapnels[this.shrapnels.length-1] = null;
            this.shrapnels.length --;
            shrapnels_id --;
        }
    }
    this.update();
}


CubeCrashGrid.prototype.paintEvent = function(event)
{
    var painter = new QPainter();

    painter.begin(this);
    painter.setRenderHint(QPainter.Antialiasing);
    painter.setClipRect(0, 0, this.cubes_width * this.CUBE_SIZE, this.cubes_height * this.CUBE_SIZE);

    var brush = new QBrush(new QColor(0, 0, 0));
    brush.setColor(new QColor(0, 0, 0));

    painter.fillRect(0, 0, this.width, this.height, brush);

    // Path and gradient for activated cubes
    var path = new QPainterPath();
    path.moveTo( 0.707106781 * Math.sin(this.activated_cube_angle) * this.CUBE_SIZE, -0.707106781 * Math.cos(this.activated_cube_angle) * this.CUBE_SIZE);
    path.lineTo( 0.707106781 * Math.cos(this.activated_cube_angle) * this.CUBE_SIZE,  0.707106781 * Math.sin(this.activated_cube_angle) * this.CUBE_SIZE);
    path.lineTo(-0.707106781 * Math.sin(this.activated_cube_angle) * this.CUBE_SIZE,  0.707106781 * Math.cos(this.activated_cube_angle) * this.CUBE_SIZE);
    path.lineTo(-0.707106781 * Math.cos(this.activated_cube_angle) * this.CUBE_SIZE, -0.707106781 * Math.sin(this.activated_cube_angle) * this.CUBE_SIZE);
    path.closeSubpath();
    var agradient = new QRadialGradient(new QPointF(0, 0), this.CUBE_SIZE*0.75);

    // Draw cubes
    var cubes_offset = 0;
    for (var cubes_y = 0; cubes_y < this.cubes_height; cubes_y ++) {
        for (var cubes_x = 0; cubes_x < this.cubes_width; cubes_x ++) {
            var cube = this.cubes[cubes_offset];
            if (!cube.active) {
                if (cube.color != this.NO_CUBE) {
                    var c_x = cubes_x*this.CUBE_SIZE;
                    var c_y = cubes_y*this.CUBE_SIZE;
                    var gradient = new QRadialGradient(new QPointF(c_x+this.CUBE_SIZE*0.5, c_y+this.CUBE_SIZE*0.5), this.CUBE_SIZE*0.75);
                    gradient.setColorAt(0.0, this.COLOR_TABLE[cube.color]);
                    gradient.setColorAt(1.0, this.COLOR_TABLE[cube.color + this.DARKER]);
                    var gbrush = new QBrush(gradient);
                    painter.fillRect(cubes_x*this.CUBE_SIZE, cubes_y*this.CUBE_SIZE,
                                     this.CUBE_SIZE-1, this.CUBE_SIZE-1, gbrush);
                }
            } else {
                var transl_x = (cubes_x + 0.5) * this.CUBE_SIZE;
                var transl_y = (cubes_y + 0.5) * this.CUBE_SIZE;
                painter.translate(new QPoint(transl_x, transl_y));
                agradient.setColorAt(0.0, this.COLOR_TABLE[cube.color]);
                agradient.setColorAt(1.0, this.COLOR_TABLE[cube.color + this.DARKER]);
                painter.fillPath(path, new QBrush(agradient));
                painter.translate(new QPoint(-transl_x, -transl_y));
            }
            cubes_offset ++;
        }
    }

    // Draw shrapnels
    for (var shrapnels_id = 0; shrapnels_id < this.shrapnels.length; shrapnels_id ++) {
        var s = this.shrapnels[shrapnels_id];
        var transf_rot = new QTransform(1, 0, 0, 1, s.x, s.y);
        transf_rot.rotateRadians(-this.activated_cube_angle + s.angle)
        var old_transf = painter.transform();
        painter.setTransform(transf_rot, true);
        agradient.setColorAt(0.0, this.COLOR_TABLE[s.color]);
        agradient.setColorAt(1.0, this.COLOR_TABLE[s.color + this.DARKER]);
        painter.fillPath(path, new QBrush(agradient));
        painter.setTransform(old_transf, false);
    }

    painter.end();
}

CubeCrashGrid.prototype.mouseMoveEvent = function(event)
{
    var cube_x = Math.floor(event.x() / this.CUBE_SIZE);
    var cube_y = Math.floor(event.y() / this.CUBE_SIZE);
    var cube_offset = cube_y * this.cubes_width + cube_x;
    // If cube is already active, then do nothing.
    if (cube_x >= 0 && cube_x < this.cubes_width &&
        cube_y >= 0 && cube_y < this.cubes_height &&
        this.cubes[cube_offset].active) {
        return;
    }
    // Clear old active cubes.
    for (var clear_offset = 0;
         clear_offset < this.cubes_width * this.cubes_height;
         clear_offset ++) {
        this.cubes[clear_offset].active = false;
    }
    // If cursor is not over cube map, then do nothing.
    if (cube_x < 0 || cube_x >= this.cubes_width &&
        cube_y < 0 || cube_y >= this.cubes_height) {
        if (this.shrapnels.length == 0) {
            this.timer.stop();
            this.update();
        }
        return;
    }
    // If cursor is over NO_CUBE, then do nothing either
    if (this.cubes[cube_offset].color == this.NO_CUBE) {
        return;
    }
    // Find new active cubes
    this.cubes[cube_offset].active = true;
    var act_finder = [];
    act_finder.push(cube_offset);
    var color = this.cubes[cube_offset].color;
    while (act_finder.length > 0) {
        var offset = act_finder.shift();
        var offset_x = offset % this.cubes_width;
        var offset_y = Math.floor(offset / this.cubes_width);
        var offset_ngb;
        // Left cube
        if (offset_x > 0) {
            offset_ngb = offset - 1;
            var cube = this.cubes[offset_ngb];
            if (cube.color == color && !cube.active) {
                this.cubes[offset_ngb].active = true;
                act_finder.push(offset_ngb);
            }
        }
        // Right cube
        if (offset_x < this.cubes_width-1) {
            offset_ngb = offset + 1;
            var cube = this.cubes[offset_ngb];
            if (cube.color == color && !cube.active) {
                cube.active = true;
                act_finder.push(offset_ngb);
            }
        }
        // Upper cube
        if (offset_y > 0) {
            offset_ngb = offset - this.cubes_width;
            var cube = this.cubes[offset_ngb];
            if (cube.color == color && !cube.active) {
                cube.active = true;
                act_finder.push(offset_ngb);
            }
        }
        // Lower cube
        if (offset_y < this.cubes_height-1) {
            offset_ngb = offset + this.cubes_width;
            var cube = this.cubes[offset_ngb];
            if (cube.color == color && !cube.active) {
                cube.active = true;
                act_finder.push(offset_ngb);
            }
        }
    }

    // Reset angle of activated cubes
    this.activated_cube_angle = Math.PI * 0.25;

    this.timer.start(this.TURN_DELAY);
}

CubeCrashGrid.prototype.mousePressEvent = function(event)
{
    var cubes_crashed = 0;

    // First do dropping of cubes
    for (var offset_x = 0; offset_x < this.cubes_width; offset_x ++) {
        var offset = (this.cubes_height-1) * this.cubes_width + offset_x;
        var drop = 0;
        while (offset >= 0 && this.cubes[offset].color != this.NO_CUBE) {
            var cube = this.cubes[offset];
            if (cube.active) {
                // Create shrapnels
                this.shrapnels.push({ x: (offset_x + 0.5) * this.CUBE_SIZE,
                                      y: ((Math.floor(offset / this.cubes_width) + 0.5)) * this.CUBE_SIZE,
                                      x_vel: Math.random() * 0.6 - 0.3,
                                      y_vel: Math.random() * 0.6 - 0.3,
                                      angle: this.activated_cube_angle,
                                      color: cube.color });
                // Update cube
                cube.color = this.NO_CUBE
                cube.active = false;
                drop ++;
                cubes_crashed ++;
            } else if (drop > 0) {
                var target_cube = this.cubes[offset + drop*this.cubes_width];
                target_cube.color = cube.color;
                cube.color = this.NO_CUBE;
            }
            offset -= this.cubes_width;
        }
    }

    // Do sliding
    var slide = 0;
    for (var offset_x = 0; offset_x < this.cubes_width; offset_x ++) {
        var offset = (this.cubes_height-1) * this.cubes_width + offset_x;
        var cubes_found = false;
        while (offset >= 0) {
            if (this.cubes[offset].color != this.NO_CUBE) {
                cubes_found = true;
                break;
            }
            offset -= this.cubes_width;
        }
        if (!cubes_found) {
            slide ++;
        } else if (slide > 0) {
            offset = (this.cubes_height-1) * this.cubes_width + offset_x;
            while (offset >= 0) {
                var cube = this.cubes[offset];
                var target_cube = this.cubes[offset - slide];
                target_cube.color = cube.color;
                cube.color = this.NO_CUBE;
                offset -= this.cubes_width;
            }
        }
    }

    // Calculate score and inform main widget
    var score_add = Math.floor(Math.pow(cubes_crashed, Math.PI) * 10);
    this.owner.addScore(score_add);

    // Call mouse move event, so cubes under mouse can be activated again.
    this.mouseMoveEvent(event);

}



QtCubeCrashWithScores.prototype = new QWidget();

QtCubeCrashWithScores.prototype.CUBES_WIDTH = 10;
QtCubeCrashWithScores.prototype.CUBES_HEIGHT = 10;
QtCubeCrashWithScores.prototype.MIN_COLORS = 3;

function QtCubeCrashWithScores(parent)
{

    QWidget.call(this, parent);

    this.score = 0;
    
    // Obtain the highscores and add them to an array
    
    
    this.hiscores = [0, 0, 0, 0];
    this.colors = this.MIN_COLORS;

    this.grid = new CubeCrashGrid(this, this, this.CUBES_WIDTH, this.CUBES_HEIGHT, this.colors);

    this.scoreLine = new QLineEdit(this);
    this.scoreLine.readOnly = true;
    this.scoreLine.text = "0";

    this.hiscoreLine = new QLineEdit(this);
    this.hiscoreLine.readOnly = true;
    // Try the obtain the highest score from the db and add it to game 
    this.getHighScore(this.setGameHigh);

    this.colorsSelector = new QComboBox();
    this.colorsSelector.addItem("Three colors");
    this.colorsSelector.addItem("Four colors");
    this.colorsSelector.addItem("Five colors");
    this.colorsSelector.addItem("Six colors");
    this.colorsSelector["activated(int)"].connect(this, this.colorsSelected);

    this.restartButton = new QPushButton("Restart", this);
    this.restartButton["clicked"].connect(this, this.restart);

    var scoreLayout = new QFormLayout();
    scoreLayout.addRow("Score:", this.scoreLine);
    scoreLayout.addRow("High score:", this.hiscoreLine);

    var buttonLayout = new QHBoxLayout();
    buttonLayout.addStretch();
    buttonLayout.addWidget(this.colorsSelector, 0, 0);
    buttonLayout.addWidget(this.restartButton, 0, 0);
    buttonLayout.addStretch();

    var layout = new QVBoxLayout();
    layout.addWidget(this.grid, 0, 0);
    layout.addLayout(scoreLayout);
    layout.addLayout(buttonLayout);
    layout.setContentsMargins(0, 0, 0, 0);
    this.setLayout(layout);

}

QtCubeCrashWithScores.prototype.addScore = function(score_add)
{
	this.score += score_add;
	this.scoreLine.text = this.score;
	var colors = this.grid.getColors();
	if (this.score > this.hiscores[colors - this.MIN_COLORS]) {
	    this.hiscores[colors - this.MIN_COLORS] = this.score;
	    if (colors == this.colors) {	    
			this.updateHigh(this.score);
	        this.hiscoreLine.text = this.hiscores[colors - this.MIN_COLORS];
	    }
	}
}


QtCubeCrashWithScores.prototype.colorsSelected = function(index)
{
    this.colors = this.MIN_COLORS + index;
    // Get the high score from the DB and set it
    this.getHighScore(this.setGameHigh);
}

QtCubeCrashWithScores.prototype.restart = function()
{
    // Get the high score from the DB and set it 
    this.getHighScore(this.setGameHigh);
    this.score = 0;
    this.scoreLine.text = "0";
    this.grid.restart(this.colors);
}


// Obtains the highscore from the db if possible
// The execution proceeds to the callback function when the request is finished
QtCubeCrashWithScores.prototype.getHighScore = function(callback) 
{
	// Form the URL to fetch the high score for a certain level (number of colors)
	var urlStr = 'http://lively.cs.tut.fi:8010/CCScore/?colors='+this.colors;
	// QNetworkAccessManager for network requests
	var accessManager = new QNetworkAccessManager(this);
	// When the request has finished, proceed to the callback function
	accessManager["finished(QNetworkReply*)"].connect(this, callback);
	// Send an HTTP/GET request
	accessManager.get(new QNetworkRequest((new QUrl(urlStr))));
}


// Sets the high score in the game
QtCubeCrashWithScores.prototype.setGameHigh = function(reply) {
	var scoresJSONString = reply.readAll().toString();
	var scoresJSONObject = eval('(' + scoresJSONString + ')'); 
	// If there is an error in the JSON or there are no scores in the db, 
	// zero is used as the default high score
	if(scoresJSONObject.error || scoresJSONObject.length == 0) {
		this.hiscoreLine.text = '0';
	}
	else {
		this.hiscoreLine.text = scoresJSONObject[0].playerScore;		
	}
}

QtCubeCrashWithScores.prototype.scoreSaved = function(reply) {
	var data = reply.readAll().toString();
	print(data);
}


// Updates a high score in the database by using HTTP/PUT
// PUT /CCScore/1
// {"id":"1","playerScore":newHigh}
QtCubeCrashWithScores.prototype.updateHigh = function(playerScore) {
	var id = this.colors - 2;
	var baseStr = 'http://lively.cs.tut.fi:8010/CCScore/' + id;
	
	var jsonStr = '{"id":"'+id +'","colors":' +this.colors +',"playerScore":' +playerScore +'}';
	var jsonArr = new QByteArray(jsonStr);
	var req = new QNetworkRequest(new QUrl(baseStr));
	
	// QNetworkAccessManager for network requests
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.scoreSaved);
	// Send the HTTP/PUT request to update the data in the db
	accessManager.put(req, jsonArr);
}



// Returns the contents of QByteArray as a string
QByteArray.prototype.toString = function() {
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};



/* To use the application in standalone mode, remove the comment around this code
var cubeCrashGame = new QtCubeCrashWithScores();

cubeCrashGame.show();
QCoreApplication.exec();
 */

