/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/


// ----------------------------------------
// AI controllers
// ----------------------------------------

function DiceWarsAIBasic()
{
}

DiceWarsAIBasic.prototype.getCommand = function(turn, zones)
{
    var best = -1;
    var best_targets = 9999;
    var best_units = 0;
    var best_target;
    for (var zones_id = 0; zones_id < zones.length; zones_id ++) {
        var att = zones[zones_id];
        if (att.player != turn) {
            continue;
        }

        if (best_units > att.units) {
            continue;
        }

        var targets = 0;
        var best_target2 = -1;
        var best_target2_units = 0;
        for (var ngbs_id = 0; ngbs_id < att.ngbs.length; ngbs_id ++) {
            var ngb_id = att.ngbs[ngbs_id];
            var ngb = zones[ngb_id];
            if (ngb.player != turn &&
                ((ngb.units == QtDiceWars.prototype.MAX_UNITS_PER_ZONE &&
                  att.units == QtDiceWars.prototype.MAX_UNITS_PER_ZONE) ||
                 ngb.units < att.units)) {
                targets ++;
                if (ngb.units > best_target2_units) {
                    best_target2 = ngb_id;
                    best_target2_units = ngb.units;
                }
            }
        }
        if (best_target2 < 0) {
            continue;
        }

        if (targets < best_targets) {
            best = zones_id;
            best_targets = targets;
            best_units = att.units;
            best_target = best_target2;
        }

    }
    // Check if no good attackers could be found
    if (best < 0) {
        return [];
    }
    return [ best, best_target ];
}


// ----------------------------------------
// Display area widget
// ----------------------------------------

DiceWarsPlayarea.prototype = new QWidget();

DiceWarsPlayarea.prototype.COLORS = [ QColor.fromHsvF(0.625, 0.75, 0.75, 1.0),
                                      QColor.fromHsvF(0.000, 0.75, 0.75, 1.0),
                                      QColor.fromHsvF(0.250, 0.75, 0.75, 1.0),
                                      QColor.fromHsvF(0.500, 0.75, 0.75, 1.0),
                                      QColor.fromHsvF(0.375, 0.75, 0.75, 1.0),
                                      QColor.fromHsvF(0.750, 0.75, 0.75, 1.0),
                                      QColor.fromHsvF(0.125, 0.75, 0.75, 1.0),
                                      QColor.fromHsvF(0.875, 0.75, 0.75, 1.0) ];
DiceWarsPlayarea.prototype.COLORS_DARK = [ QColor.fromHsvF(0.625, 0.75, 0.5, 1.0),
                                           QColor.fromHsvF(0.000, 0.75, 0.5, 1.0),
                                           QColor.fromHsvF(0.250, 0.75, 0.5, 1.0),
                                           QColor.fromHsvF(0.500, 0.75, 0.5, 1.0),
                                           QColor.fromHsvF(0.375, 0.75, 0.5, 1.0),
                                           QColor.fromHsvF(0.750, 0.75, 0.5, 1.0),
                                           QColor.fromHsvF(0.125, 0.75, 0.5, 1.0),
                                           QColor.fromHsvF(0.875, 0.75, 0.5, 1.0) ];
DiceWarsPlayarea.prototype.COLORS_HILIGHT = [ QColor.fromHsvF(0.625, 0.75, 1.0, 1.0),
                                              QColor.fromHsvF(0.000, 0.75, 1.0, 1.0),
                                              QColor.fromHsvF(0.250, 0.75, 1.0, 1.0),
                                              QColor.fromHsvF(0.500, 0.75, 1.0, 1.0),
                                              QColor.fromHsvF(0.375, 0.75, 1.0, 1.0),
                                              QColor.fromHsvF(0.750, 0.75, 1.0, 1.0),
                                              QColor.fromHsvF(0.125, 0.75, 1.0, 1.0),
                                              QColor.fromHsvF(0.875, 0.75, 1.0, 1.0) ];

function DiceWarsPlayarea(parent, owner)
{
    QWidget.call(this, parent);
    this.owner = owner;
    this.mouseTracking = true;
    this.zones = [];
    this.zones_fixed = [];
    this.zones_bounds = new QRectF(0, 0, 0, 0);

    this.detail = true;
    this.animations = false;

    this.finished = false;

}

DiceWarsPlayarea.prototype.setGameFinished = function(won)
{
    this.zones_mouseover = -1;
    this.finished = true;
    this.won = won;
    this.update();
}

DiceWarsPlayarea.prototype.setZones = function(zones)
{
    this.zones = zones;
    while (this.zones_fixed.length < this.zones.length) {
        this.zones_fixed.push(null);
    }
    // Calculate bounding of all zones
    var min_x = zones[0].shape[0].x();
    var max_x = zones[0].shape[0].x();
    var min_y = zones[0].shape[0].y();
    var max_y = zones[0].shape[0].y();
    for (var zones_id = 0; zones_id < zones.length; zones_id ++) {
        var zone = zones[zones_id];
        for (var shape_id = 0; shape_id < zone.shape.length; shape_id ++) {
            var p = zone.shape[shape_id];
            if (p.x() < min_x) {
                min_x = p.x();
            }
            if (p.x() > max_x) {
                max_x = p.x();
            }
            if (p.y() < min_y) {
                min_y = p.y();
            }
            if (p.y() > max_y) {
                max_y = p.y();
            }
        }
    }
    this.zones_bounds = new QRectF(min_x, min_y, max_x - min_x, max_y - min_y);

    this.recalculateFixedZones();

    this.update();
}

DiceWarsPlayarea.prototype.clearMouseover = function()
{
    this.zones_mouseover = -1;
}

DiceWarsPlayarea.prototype.recalculateFixedZones = function()
{
    var mul;
    var add_x;
    var add_y;
    if (this.zones_bounds.width() / this.zones_bounds.height() > this.width / this.height) {
        mul = this.width / this.zones_bounds.width();
        add_x = -this.zones_bounds.left()*mul;
        add_y = (this.height - (this.zones_bounds.height()*mul)) / 2.0 - this.zones_bounds.top()*mul
    } else {
        mul = this.height / this.zones_bounds.height();
        add_y = -this.zones_bounds.top()*mul;
        add_x = (this.width - (this.zones_bounds.width()*mul)) / 2.0 - this.zones_bounds.left()*mul
    }
    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];
        var points_fixed = []
        for (var shape_offset = 0;
             shape_offset < zone.shape.length;
             shape_offset ++) {
            var p = zone.shape[shape_offset];
            points_fixed.push(new QPointF(add_x + mul*p.x(), add_y + mul*p.y()));
        }
        var poly = new QPolygonF(points_fixed);
        this.zones_fixed[zones_id] = poly;
    }
}

DiceWarsPlayarea.prototype.getMinimumSizeHint = function()
{
    return new QSize(320, 240);
}

DiceWarsPlayarea.prototype.getSizeHint = function()
{
    return new QSize(512, 384);
}

DiceWarsPlayarea.prototype.resizeEvent = function(event)
{
    this.recalculateFixedZones();
}

DiceWarsPlayarea.prototype.paintEvent = function(event)
{
    var painter = new QPainter();

    painter.begin(this);
    painter.setRenderHint(QPainter.Antialiasing, this.detail);
    painter.setRenderHint(QPainter.SmoothPixmapTransform, this.detail);
    painter.setClipRect(0, 0, this.width, this.height);

    var scale;
    if (this.zones_bounds.width() / this.zones_bounds.height() > this.width / this.height) {
        scale = this.width / 320;
    } else {
        scale = this.height / this.zones_bounds.height() * this.zones_bounds.width() / 320;
    }

    // Draw zones and calculate positions of zones
    var zones_pos = [];
    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];
        var zone_fixed = this.zones_fixed[zones_id];
// TODO: The following causes errors sometimes. It seems, that zone_fixed is not an object!
        zones_pos.push({ pos: zone_fixed.boundingRect().center(), zone: zones_id });
        var brush;
        if (zone.hilight || this.zones_mouseover == zones_id) {
            painter.setPen(new QPen(this.COLORS[zone.player], 1.6*scale, Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin));
            brush = new QBrush(this.COLORS_HILIGHT[zone.player]);
        } else {
            painter.setPen(new QPen(this.COLORS_DARK[zone.player], 1.6*scale, Qt.SolidLine, Qt.RoundCap, Qt.MiterJoin));
            brush = new QBrush(this.COLORS[zone.player]);
        }
        painter.setBrush(brush);
        painter.drawPolygon(zone_fixed);
    }

    // Sort unit positions so they can be drawn in proper order
    zones_pos.sort(function(pos1, pos2) { return pos1.pos.y() - pos2.pos.y(); });

    // Draw units of zones
// TODO: Use vector graphics if possible!
    painter.scale(scale/2.0, scale/2.0);
    var scale_inv = 1.0 / scale * 2.0;
    for (var zones_pos_id = 0; zones_pos_id < zones_pos.length; zones_pos_id ++) {
        var pos = zones_pos[zones_pos_id];
        var zone = this.zones[pos.zone];
        var pixmap = this.pixmap_dices[zone.player];
        if (pixmap != null) {
            var stack1 = Math.min(zone.units, 4);
            var stack2 = Math.min(zone.units-4, 4);
            for (var stack_id = 0; stack_id < stack1; stack_id ++) {
                var p = new QPointF(pos.pos.x()*scale_inv - 5.0, pos.pos.y()*scale_inv - 12.5 - 12*stack_id);
                painter.drawPixmap(p, pixmap);
            }
            for (var stack_id = 0; stack_id < stack2; stack_id ++) {
                var p = new QPointF(pos.pos.x()*scale_inv - 15.0, pos.pos.y()*scale_inv - 7.5 - 12*stack_id);
                painter.drawPixmap(p, pixmap);
            }
        }
    }

    painter.scale(scale_inv, scale_inv);

    if (this.finished) {
        var window_color = this.palette.window().color();
        window_color.setAlpha(192);
        var alphabrush = new QBrush(window_color);
        painter.fillRect(0, 0, this.width, this.height, alphabrush);
        if (this.won) {
            painter.setPen(new QColor(32, 255, 32));
            var font_scale = 4;
            painter.setTransform(QTransform.fromScale(font_scale, font_scale), true);
            painter.drawText(0, 0, this.width/font_scale, this.height/font_scale, Qt.AlignCenter, "Victory!", null);
            painter.setTransform(QTransform.fromScale(1/font_scale, 1/font_scale), true);
        } else {
            painter.setPen(new QColor(255, 0, 0));
            var font_scale = 4;
            painter.setTransform(QTransform.fromScale(font_scale, font_scale), true);
            painter.drawText(0, 0, this.width/font_scale, this.height/font_scale, Qt.AlignCenter, "You have been defeated!", null);
            painter.setTransform(QTransform.fromScale(1/font_scale, 1/font_scale), true);
        }
    }

    painter.end();
}

DiceWarsPlayarea.prototype.mouseMoveEvent = function(event)
{

    var old_mouseover = this.zones_mouseover;
    this.zones_mouseover = -1;

    // Go zones through and check if mouse is over a clickable zone.
    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];
        var fzone = this.zones_fixed[zones_id];
        if (zone.clickable && fzone.containsPoint(new QPointF(event.x(), event.y()), Qt.OddEvenFill)) {
            this.zones_mouseover = zones_id;
            break;
        }
    }

    if (this.zones_mouseover != old_mouseover) {
        this.update();
    }

}

DiceWarsPlayarea.prototype.mousePressEvent = function(event)
{
    if (this.zones_mouseover >= 0) {
        this.owner.zoneClicked(this.zones_mouseover);
    }
}


// ----------------------------------------
// Main application widget
// ----------------------------------------

QtDiceWars.prototype = new QWidget();

QtDiceWars.prototype.MAX_UNITS_PER_ZONE = 8;
QtDiceWars.prototype.FRAME_LENGTH = 1000 / 15;
QtDiceWars.prototype.PAUSE_FRAMES = 15;

function QtDiceWars(parent)
{
    QWidget.call(this, parent);

    this.game_running = false;

    this.landpixmaps = [];
    this.pixmap_dices = [];
    for (var pixmap_id = 0; pixmap_id < 8; pixmap_id ++) {
        this.pixmap_dices.push(null);
    }

    var files_to_load = [ "http://lively.cs.tut.fi/images/dicewars/Dice0.png",
                          "http://lively.cs.tut.fi/images/dicewars/Dice1.png",
                          "http://lively.cs.tut.fi/images/dicewars/Dice2.png",
                          "http://lively.cs.tut.fi/images/dicewars/Dice3.png",
                          "http://lively.cs.tut.fi/images/dicewars/Dice4.png",
                          "http://lively.cs.tut.fi/images/dicewars/Dice5.png",
                          "http://lively.cs.tut.fi/images/dicewars/Dice6.png",
                          "http://lively.cs.tut.fi/images/dicewars/Dice7.png" ];
    var fileloader = new Fileloader();
    fileloader.loadFiles(files_to_load, this, this.fileLoaded);

    this.mainlayout = new QVBoxLayout();
    this.mainlayout.setContentsMargins(0, 0, 0, 0);
    this.layoutholder = null;
    this.setLayout(this.mainlayout);

    this.timer = new QTimer(this);
    this.timer.timeout.connect(this, this.timerEvent);

    // Poor seeding tweak
    var seed = Math.round((new Date()).getTime() / 10000) % 1000;
    for (var seeding = 0; seeding < seed; seeding ++) {
        Math.random();
    }

    this.initStartmenu();

}

QtDiceWars.prototype.fileLoaded = function(caller, filename, data, filegroup)
{
    var new_pixmap = new QPixmap();
    new_pixmap.loadFromData(data, 0, Qt.AutoColor);
    for (var pixmap_id = 0; pixmap_id < 8; pixmap_id ++) {
        if (filename == "http://lively.cs.tut.fi/images/dicewars/Dice" + pixmap_id + ".png") {
            caller.pixmap_dices[pixmap_id] = new_pixmap
            if (pixmap_id < caller.landpixmaps.length) {
                caller.landpixmaps[pixmap_id].pixmap = new_pixmap;
            }
        }
    }
    if (caller.playarea) {
        caller.player.pixmap_dices = caller.pixmap_dices;
    }
    caller.update();
}

QtDiceWars.prototype.initStartmenu = function()
{
    if (this.layoutholder) {
        this.mainlayout.removeWidget(this.layoutholder);
        this.layoutholder.setParent(null);
    }

    var startbutton = new QPushButton("Start game", this);
    startbutton["clicked"].connect(this, this.startGamePressed);

    this.playersSlider = new QSlider(Qt.Horizontal, this);
    this.playersSlider.setRange(2, 8);
    this.playersSlider["sliderMoved(int)"].connect(this, this.playersSliderChanged);
    this.playersSlider["valueChanged(int)"].connect(this, this.playersSliderChanged);

    this.playersLabel = new QLabel("2", this);

    this.animationCheckbox = new QCheckBox("", this);
    this.animationCheckbox.checked = true;

    this.hidetailCheckBox = new QCheckBox("", this);
    this.hidetailCheckBox.checked = true;

    var playerslayout = new QHBoxLayout();
    playerslayout.addWidget(this.playersSlider, 0, 0);
    playerslayout.addWidget(this.playersLabel, 0, 0);

    var formlayout = new QFormLayout();
    formlayout.addRow("Animations:", this.animationCheckbox);
    formlayout.addRow("High detail:", this.hidetailCheckBox);
    formlayout.addRow("Players:", playerslayout);

    var menulayout2 = new QVBoxLayout();
    menulayout2.addStretch();
    menulayout2.addLayout(formlayout);
    menulayout2.addWidget(startbutton, 0, 0);
    menulayout2.addStretch();

    var menulayout = new QHBoxLayout();
    menulayout.addStretch();
    menulayout.addLayout(menulayout2);
    menulayout.addStretch();

    this.layoutholder = new LayoutHolder(this, menulayout);
    this.mainlayout.addWidget(this.layoutholder, 0, 0);
}

QtDiceWars.prototype.startGamePressed = function()
{

    var players_count = this.playersSlider.value;

    this.mainlayout.removeWidget(this.layoutholder);
    this.layoutholder.setParent(null);

    this.playarea = new DiceWarsPlayarea(this, this);
    this.playarea.animations = this.animationCheckbox.checked;
    this.playarea.detail = this.hidetailCheckBox.checked;
    this.playarea.pixmap_dices = this.pixmap_dices;
    this.animations = this.animationCheckbox.checked;

    this.endturnbutton = new QPushButton("End turn", this);
    this.endturnbutton["clicked"].connect(this, this.endturnPressed);
    this.endturnbutton.enabled = false;

    this.mainmenubutton = new QPushButton("Main menu", this);
    this.mainmenubutton["clicked"].connect(this, this.mainmenuPressed);

    var attackerlabel = new QLabel("Attacker:", this);
    var defenderlabel = new QLabel("Defender:", this);
    this.attackerline = new QLabel(this);
    this.defenderline = new QLabel(this);

    this.landpixmaps = [];
    this.landlabels = [];
    var landlayout = new QHBoxLayout();
    for (var players_id = 0; players_id < players_count; players_id ++) {
        var landpixmap = new QLabel(this);
        landpixmap.pixmap = this.pixmap_dices[players_id];
        var landlabel = new QLabel("0", this);
        this.landpixmaps.push(landpixmap);
        this.landlabels.push(landlabel)
        landlayout.addWidget(landpixmap, 0, 0);
        landlayout.addWidget(landlabel, 0, 0);
        if (players_id != players_count-1) {
            landlayout.addStretch();
        }
    }

    var hlayout = new QHBoxLayout();
    hlayout.addWidget(this.mainmenubutton, 0, 0);
    hlayout.addStretch();
    hlayout.addWidget(attackerlabel, 0, 0);
    hlayout.addWidget(this.attackerline, 0, 0);
    hlayout.addStretch();
    hlayout.addWidget(defenderlabel, 0, 0);
    hlayout.addWidget(this.defenderline, 0, 0);
    hlayout.addStretch();
    hlayout.addWidget(this.endturnbutton, 0, 0);

    var layout = new QVBoxLayout();
    layout.addWidget(this.playarea, 1, 0);
    layout.addLayout(landlayout);
    layout.addLayout(hlayout);

    this.layoutholder = new LayoutHolder(this, layout);
    this.mainlayout.addWidget(this.layoutholder, 0, 0);

    this.startNewGame(players_count);
}

QtDiceWars.prototype.playersSliderChanged = function(value)
{
    this.playersLabel.text = value;
}

QtDiceWars.prototype.generateNewZones = function(players, player_in_turn)
{
    var HEX_WIDTH = 0.89;
    var ASPECT_RATIO = 0.75;
    var zones = [];

    function getNeighbors(offset, width, height)
    {
        result = [];
        var x = offset % width;
        var y = Math.floor(offset / width);
        if (x > 0) {
            result.push(offset - 1);
        }
        if (y % 2 == 0) {
            if (y > 0) {
                if (x > 0) result.push(offset - width - 1);
                result.push(offset - width);
            }
        } else {
            result.push(offset - width);
            if (x < width-1) result.push(offset - width + 1);
        }
        if (x < width-1) {
            result.push(offset + 1);
        }
        if (y % 2 == 0) {
            if (y < height-1) {
                result.push(offset + width);
                if (x > 0) result.push(offset + width - 1);
            }
        } else {
            if (y < height-1) {
                if (x < width-1) result.push(offset + width + 1);
                result.push(offset + width);
            }
        }
        return result;
    }

    function getNeighbor(offset, edge, width, height)
    {
        var x = offset % width;
        var y = Math.floor(offset / width);
        switch (edge) {
        case 0:
            x --;
            break;
        case 1:
            if (y % 2 == 0) x --;
            y --;
            break;
        case 2:
            if (y % 2 == 1) x ++;
            y --;
            break;
        case 3:
            x ++;
            break;
        case 4:
            if (y % 2 == 1) x ++;
            y ++;
            break;
        case 5:
            if (y % 2 == 0) x --;
            y ++;
            break;
        }
        if (x < 0 || x >= width ||
            y < 0 || y >= height) {
            return -1;
        }
        return y*width + x;
    }

    var HGRID_WIDTH = 24;
    var HGRID_HEIGHT = 24;

    var total_land;
    var filled_land;
    var hgrid;
    do {
        total_land = 0;
        hgrid = [];
        while (hgrid.length < HGRID_WIDTH*HGRID_HEIGHT) {
            hgrid.push(-1);
        }
        var rzones = [];
        var rzones_size = Math.round(HGRID_WIDTH*HGRID_HEIGHT/13.5);
        while (rzones.length < rzones_size) {
            var x;
            var y;
            do {
                x = Math.floor(Math.random()*HGRID_WIDTH);
                y = Math.floor(Math.random()*HGRID_HEIGHT);
            } while (hgrid[y*HGRID_HEIGHT + x] >= 0);
            hgrid[y*HGRID_HEIGHT + x] = rzones.length;
            total_land ++;
            rzones.push([y*HGRID_HEIGHT + x]);
        }
        for (var inc = 1; inc < 10; inc ++) {
            for (var rzones_id = 0; rzones_id < rzones.length; rzones_id ++) {
                var rzones2 = rzones[rzones_id];
                var ngbs = [];
                var ngbs_selfngbs = 0;
                for (var rzones2_id = 0; rzones2_id < rzones2.length; rzones2_id ++) {
                    rzone = rzones2[rzones2_id];
                    ns = getNeighbors(rzone, HGRID_WIDTH, HGRID_HEIGHT);
                    for (var ns_id = 0; ns_id < ns.length; ns_id ++) {
                        var n = ns[ns_id];
                        if (hgrid[n] < 0) {
                            var nns = getNeighbors(n, HGRID_WIDTH, HGRID_HEIGHT);
                            var selfngbs = 0;
                            for (var nns_id = 0; nns_id < nns.length; nns_id ++) {
                                var nn = nns[nns_id];
                                if (hgrid[nn] == rzones_id) {
                                    selfngbs ++;
                                }
                            }
                            if (selfngbs > ngbs_selfngbs) {
                                ngbs = [n];
                                ngbs_selfngbs = selfngbs;
                            } else if (selfngbs > ngbs_selfngbs) {
                                var already_added = false;
                                for (var ngbs_id = 0; ngbs_id < ngbs.end(); ngbs_id ++) {
                                    if (ngbs[ngbs_id] == n) {
                                        already_added = true;
                                        break;
                                    }
                                }
                                if (!already_added) {
                                    ngbs.push(n);
                                }
                            }
                        }
                    }
                }
                if (ngbs.length > 0) {
                    var ngbs_id = Math.floor(Math.random() * ngbs.length);
                    var ngb = ngbs[ngbs_id];
                    hgrid[ngb] = rzones_id;
                    total_land ++;
                    rzones2.push(ngb);
                }
            }
        }
        filled_land = [];
        var new_land = [];
        for (var land_finder = 0;
             land_finder < HGRID_WIDTH*HGRID_HEIGHT;
             land_finder ++) {
            if (hgrid[land_finder] >= 0) {
                new_land.push(land_finder);
                filled_land.push(land_finder);
                break;
            }
        }
        while (new_land.length > 0) {
            var land = new_land.pop();
            ngbs = getNeighbors(land, HGRID_WIDTH, HGRID_HEIGHT);
            for (var ngbs_id = 0; ngbs_id < ngbs.length; ngbs_id ++) {
                var ngb = ngbs[ngbs_id];
                if (hgrid[ngb] >= 0) {
                    var already_found = false;
                    for (var filled_land_id = 0;
                         filled_land_id < filled_land.length;
                         filled_land_id ++) {
                        if (filled_land[filled_land_id] == ngb) {
                            already_found = true;
                            break;
                        }
                    }
                    if (!already_found) {
                        new_land.push(ngb);
                        filled_land.push(ngb);
                    }
                }
            }
        }
    } while (filled_land.length < total_land);
    for (var rzones_id = 0; rzones_id < rzones.length; rzones_id ++) {
        var rzones2 = rzones[rzones_id];
        var edgehex = -1;
        var edgehex_edge;
        for (var rzones2_id = 0; rzones2_id < rzones2.length; rzones2_id ++) {
            var rzone = rzones2[rzones2_id];
            var ngbs = getNeighbors(rzone, HGRID_WIDTH, HGRID_HEIGHT);
            if (ngbs.length < 6) {
                edgehex = rzone;
                var x = rzone % HGRID_WIDTH;
                var y = Math.floor(rzone / HGRID_WIDTH);
                if (x == 0) {
                    edgehex_edge = 0;
                } else if (x == HGRID_WIDTH-1) {
                    edgehex_edge = 3;
                } else if (y == 0) {
                    edgehex_edge = 1;
                } else {
                    edgehex_edge = 4;
                }
                break;
            } else {
                for (var ngbs_id = 0; ngbs_id < 6; ngbs_id ++) {
                    var ngb = ngbs[ngbs_id];
                    if (hgrid[ngb] != rzones_id) {
                        edgehex = rzone;
                        edgehex_edge = ngbs_id;
                        break;
                    }
                }
                if (edgehex >= 0) {
                    break;
                }
            }
        }
        var points = [];
        var ngbs = [];
        var begin_hex = edgehex;
        var begin_edge = edgehex_edge;
        var skip_next_point = false;
        do {
            var hx = edgehex % HGRID_WIDTH;
            var hy = Math.floor(edgehex / HGRID_WIDTH);
            var dx = 0.866025404*(2*hx + (hy % 2));
            var dy = 1.5*hy;
            var ex = HEX_WIDTH * -Math.cos((edgehex_edge*60-30)/180*Math.PI);
            var ey = HEX_WIDTH * -Math.sin((edgehex_edge*60-30)/180*Math.PI);
            if (!skip_next_point) {
                points.push(new QPointF(dx + ex, (dy + ey) * ASPECT_RATIO));
            } else {
                skip_next_point = false;
            }
            edgehex_edge = (edgehex_edge + 1) % 6;
            var checkhex = getNeighbor(edgehex, edgehex_edge, HGRID_WIDTH, HGRID_HEIGHT);
            if (checkhex >= 0) {
                if (hgrid[checkhex] == rzones_id) {
                    var ex = HEX_WIDTH * -Math.cos((edgehex_edge*60-30)/180*Math.PI);
                    var ey = HEX_WIDTH * -Math.sin((edgehex_edge*60-30)/180*Math.PI);
                    ex += (1 - HEX_WIDTH) * -Math.cos((edgehex_edge*60+30)/180*Math.PI);
                    ey += (1 - HEX_WIDTH) * -Math.sin((edgehex_edge*60+30)/180*Math.PI);
                    points.push(new QPointF(dx + ex, (dy + ey) * ASPECT_RATIO));
                    skip_next_point = true;
                    edgehex_edge = (edgehex_edge + 4) % 6;
                    edgehex = checkhex;
                } else if (hgrid[checkhex] >= 0) {
                    var already_found = false;
                    for (var ngbs_id = 0; ngbs_id < ngbs.length; ngbs_id ++) {
                        if (ngbs[ngbs_id] == hgrid[checkhex]) {
                            already_found = true;
                            break;
                        }
                    }
                    if (!already_found) {
                        ngbs.push(hgrid[checkhex]);
                    }
                }
            }
        } while (edgehex != begin_hex || edgehex_edge != begin_edge);
        var zone = { player: -1, units: 1, ngbs: ngbs, shape: points };
        zones.push(zone);
    }

    // Place players to zones
    var unready_left = zones.length;
    var next_player = Math.floor(Math.random()*players);
    var units_per_player = Math.round(zones.length*3/players);
    units_left = [];
    while (units_left.length < players) {
        units_left.push(units_per_player);
    }
    // Give some extra units to those players that will wait their turn for
    // longest.
    for (var extra = 1; extra < players; extra ++) {
        player_in_turn ++;
        if (player_in_turn >= players) {
            player_in_turn = 0;
        }
        units_left[player_in_turn] += Math.round(units_per_player*0.08*extra);
    }
    while (unready_left > 0) {
        var zones_id = Math.floor(Math.random()*zones.length);
        while (zones[zones_id].player != -1) {
            zones_id ++;
            if (zones_id == zones.length) zones_id = 0;
        }
        zones[zones_id].player = next_player;
        units_left[next_player] --;
        next_player ++;
        if (next_player >= players) {
            next_player = 0;
        }
        unready_left --;
    }
    // Add more units to zones
    next_player = 0;
    while (next_player < players) {
        while (units_left[next_player] > 0) {
            var zones_id = Math.floor(Math.random()*zones.length);
            while (zones[zones_id].player != next_player &&
                   zones[zones_id].units >= this.MAX_UNITS_PER_ZONE) {
                zones_id ++;
                if (zones_id == zones.length) zones_id = 0;
            }
            zones[zones_id].units ++;
            units_left[next_player] --;
        }
        next_player ++;
    }

    return zones;
}

QtDiceWars.prototype.serializeZones = function()
{
    var result = "[";

    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];
        result += "{";
        result += "\"units\":" + zone.units + ",";
        result += "\"player\":" + zone.player + ",";
        result += "\"shape\":[";
        for (var points_id = 0; points_id < zone.shape.length; points_id ++) {
            var point = zone.shape[points_id];
            result += "{\"x\":" + point.x() + ",\"y\":" + point.y() + "}";
            if (points_id != zone.shape.length - 1) {
                result += ",";
            }
        }
        result += "],";
        result += "\"neighbors\":[";
        for (var ngbs_id = 0; ngbs_id < zone.ngbs.length; ngbs_id ++) {
            var ngb = zone.ngbs[ngbs_id];
            result += ngb;
            if (ngbs_id != zone.ngbs.length - 1) {
                result += ",";
            }
        }
        result += "]";
        result += "}";
        if (zones_id != this.zones.length - 1) {
            result += ",";
        }
    }

    result += "]";
    return result;
}

QtDiceWars.prototype.deserializeZones = function(sdata)
{
    var zones_raw = eval(sdata);
    zones = [];
    for (var zones_raw_id = 0; zones_raw_id < zones_raw.length; zones_raw_id ++) {
        var zone_raw = zones_raw[zones_raw_id];
        var shape = [];
        for (var shape_id = 0; shape_id < zone_raw.shape.length; shape_id ++) {
            var point = zone_raw.shape[shape_id];
            shape.push(new QPointF(point.x, point.y));
        }
        zones.push({ player: zone_raw.player,
                     units: zone_raw.units,
                     ngbs: zone_raw.neighbors,
                     shape: shape });
    }
    return zones;
}

QtDiceWars.prototype.endturnPressed = function()
{
    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];
        zone.hilight = false;
        zone.clickable = false;
    }

    // Get all zones that can receive more units.
    this.unit_giving_targets = [];
    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];
        if (zone.player == this.turn && zone.units < this.MAX_UNITS_PER_ZONE) {
            this.unit_giving_targets.push(zones_id);
        }
    }

    this.new_units_to_give = this.unitedzones[this.turn];

    this.endturnbutton.enabled = false;

    this.timer.start(this.FRAME_LENGTH);

    this.update();
}

QtDiceWars.prototype.mainmenuPressed = function()
{
    this.initStartmenu();
    this.game_running = false;
}

QtDiceWars.prototype.timerEvent = function()
{
    if (!this.game_running) {
        return;
    }
    // Check if pause is requested
    if (this.pause > 0) {
        this.pause --;
        return;
    }
    // Check if turn is being ended
    if (this.new_units_to_give > 0) {

        do {

            if (this.unit_giving_targets.length > 0) {

                var id = Math.floor(Math.random()*this.unit_giving_targets.length)
                var give_id = this.unit_giving_targets[id];
                this.zones[give_id].units ++;

                if (this.zones[give_id].units >= this.MAX_UNITS_PER_ZONE) {
                    this.unit_giving_targets[id] = this.unit_giving_targets[this.unit_giving_targets.length-1];
                    this.unit_giving_targets.length --;
                }

            }
// TODO: Some reserve if all zones are full?

            this.new_units_to_give --;

        } while (!this.animations && this.new_units_to_give > 0);

        if (this.new_units_to_give == 0) {
            do {
                this.turn ++;
                if (this.turn >= this.players) {
                    this.turn = 0;
                }
            } while (this.unitedzones[this.turn] <= 0);
            this.timer.stop();
            this.initTurn();
        }

    }
    // Attacking is happening
    else {
        if (this.attacker_units_left > 0) {
            this.attacker_units_left --;
            this.attacker_power += 1+Math.floor(Math.random()*6);
            this.attackerline.text = this.attacker_power;
            if (this.attacker_units_left <= 0) {
                this.pause = this.PAUSE_FRAMES;
            }
        } else if (this.defender_units_left > 0) {
            this.defender_units_left --;
            this.defender_power += 1+Math.floor(Math.random()*6);
            this.defenderline.text = this.defender_power;
            if (this.defender_units_left <= 0) {
                this.pause = this.PAUSE_FRAMES;
            }
        } else {
            var azone = this.zones[this.attacker];
            var dzone = this.zones[this.defender];
            if (this.attacker_power > this.defender_power) {
                dzone.units = azone.units - 1;
                dzone.player = azone.player;
            }
            azone.units = 1;
            this.timer.stop();
            for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
                var zone = this.zones[zones_id];
                zone.hilight = false;
            }
            this.attackerline.text = "";
            this.defenderline.text = "";
            this.initTurn();
        }
    }

    this.playarea.update();
}

QtDiceWars.prototype.startNewGame = function(players)
{
    this.players = players;
    this.game_running = true;
    this.turn = Math.floor(Math.random()*players);
    this.zones = this.generateNewZones(players, this.turn);
    this.playarea.setZones(this.zones);
    this.pause = 0;
    // Create AI players
    this.controllers = [ null ];
    while (this.controllers.length < players) {
        this.controllers.push(new DiceWarsAIBasic());
    }
    this.initTurn();
}

QtDiceWars.prototype.initTurn = function()
{
    // Check if some player has won the whole game
    this.calculateBiggestUnitedZones();
    var living_players = [];
    for (var players_id = 0; players_id < this.players; players_id ++) {
        if (this.unitedzones[players_id] > 0) {
            living_players.push(players_id);
        }
        this.landlabels[players_id].text = this.unitedzones[players_id];
    }
    if (this.unitedzones[0] == 0) {
        this.playarea.setGameFinished(false);
        this.endturnbutton.enabled = false;
        return;
    } else if (living_players.length == 1) {
        this.playarea.setGameFinished(true);
        this.endturnbutton.enabled = false;
        return;
    }

    // Check if controller is player
    if (this.controllers[this.turn] == null) {
        this.markPossibleAttackersClickable();
        this.playarea.update();
        this.endturnbutton.enabled = true;
    } else {
        var c = this.controllers[this.turn];
        var commands = c.getCommand(this.turn, this.zones);
        if (commands.length == 0) {
            this.endturnPressed();
        } else {
            if (this.animations) {
                this.attacker = commands[0];
                this.defender = commands[1];
                this.attacker_power = 0;
                this.defender_power = 0;
                this.attacker_units_left = this.zones[this.attacker].units;
                this.defender_units_left = this.zones[this.defender].units;
                this.endturnbutton.enabled = false;
                this.timer.start(this.FRAME_LENGTH);
                this.attackerline.text = "";
                this.defenderline.text = "";
                this.zones[this.attacker].hilight = true;
                this.zones[this.defender].hilight = true;
                this.update();
            } else {
                this.instantAttack(commands[0], commands[1]);
            }
        }
    }
}

QtDiceWars.prototype.zoneClicked = function(clicked_id)
{
    this.playarea.clearMouseover();

    var clicked = this.zones[clicked_id];
    // If own zone was clicked and it's already hilighted, then it means that
    // attack has been cancelled.
    if (clicked.player == this.turn && clicked.hilight) {
        this.markPossibleAttackersClickable();
    }
    // If own zone was clicked, then it means that attackers have been chosen.
    else if (clicked.player == this.turn) {
        // Mark all zones not clickable
        for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
            var zone = this.zones[zones_id];
            zone.clickable = false;
        }
        // Mark enemy neighbors clickable
        for (var ngbs_id = 0; ngbs_id < clicked.ngbs.length; ngbs_id ++) {
            var ngb = this.zones[clicked.ngbs[ngbs_id]];
            if (ngb.player != this.turn) {
                ngb.clickable = true;
            }
        }
        // Mark attacker hilighted and clickable (so that attacking can be cancelled)
        clicked.hilight = true;
        clicked.clickable = true;
        this.attacker = clicked_id;
    }
    // Target was clicked
    else {
        if (this.animations) {
            this.defender = clicked_id;
            this.attacker_power = 0;
            this.defender_power = 0;
            this.attacker_units_left = this.zones[this.attacker].units;
            this.defender_units_left = clicked.units;
            this.endturnbutton.enabled = false;
            this.timer.start(this.FRAME_LENGTH);
            this.attackerline.text = "";
            this.defenderline.text = "";
            clicked.hilight = true;
            for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
                var zone = this.zones[zones_id];
                zone.clickable = false;
            }
            this.update();
        } else {
            this.instantAttack(this.attacker, clicked_id);
        }
    }
}

QtDiceWars.prototype.instantAttack = function(attacker, defender)
{
    this.attacker = attacker;
    this.defender = defender;
    this.attacker_units_left = 0;
    this.defender_units_left = 0;
    this.attacker_power = 0;
    this.defender_power = 0;
    for (var dice_id = 0; dice_id < this.zones[this.attacker].units; dice_id ++) {
        this.attacker_power += 1+Math.floor(Math.random()*6);
    }
    for (var dice_id = 0; dice_id < this.zones[this.defender].units; dice_id ++) {
        this.defender_power += 1+Math.floor(Math.random()*6);
    }
    this.endturnbutton.enabled = false;
    this.attackerline.text = this.attacker_power;
    this.defenderline.text = this.defender_power;
    this.zones[this.attacker].hilight = true;
    this.zones[this.defender].hilight = true;
    this.update();
    this.pause = this.PAUSE_FRAMES;
    this.timer.start(this.FRAME_LENGTH);
}

QtDiceWars.prototype.markPossibleAttackersClickable = function()
{
    // Go through all zones and mark them either clickable or not
    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];
        zone.hilight = false;
        // Skip zones that are not owned by player in turn
        if (zone.player != this.turn) {
            zone.clickable = false;
            continue;
        }
        // Skip zones that do not have enough units
        if (zone.units <= 1) {
            zone.clickable = false;
            continue;
        }
        // Check if this zone has enemy neighbors
        var enemies_found = false;
        for (var ngbs_id = 0; ngbs_id < zone.ngbs.length; ngbs_id ++) {
            if (this.zones[zone.ngbs[ngbs_id]].player != this.turn) {
                enemies_found = true;
                break;
            }
        }
        zone.clickable = enemies_found;
    }
}

QtDiceWars.prototype.calculateBiggestUnitedZones = function()
{
    this.unitedzones = [];
    while (this.unitedzones.length < this.players) {
        this.unitedzones.push(0);
    }

    this.unit_giving_targets = [];
    for (var zones_id = 0; zones_id < this.zones.length; zones_id ++) {
        var zone = this.zones[zones_id];

        var player = zone.player;

        var uzones = [zones_id];
        var uzones_new = [zones_id];
        while (uzones_new.length > 0) {
            var uzones_id = uzones_new.pop();
            var uzone = this.zones[uzones_id];
            for (var ngbs_id = 0; ngbs_id < uzone.ngbs.length; ngbs_id ++) {
                var ngb = uzone.ngbs[ngbs_id];
                if (this.zones[ngb].player != player) {
                    continue;
                }
                var ngb_already_exists = false;
                for (var uzones_find = 0; uzones_find < uzones.length; uzones_find ++) {
                    if (uzones[uzones_find] == ngb) {
                        ngb_already_exists = true;
                        break;
                    }
                }
                if (!ngb_already_exists) {
                    uzones.push(ngb);
                    uzones_new.push(ngb);
                }
            }
        }
        if (uzones.length > this.unitedzones[player]) {
            this.unitedzones[player] = uzones.length;
        }

    }

}

/* To use the application in standalone mode, remove the comment around this code
var dicewarsgame = new QtDiceWars();

dicewarsgame.show();
QCoreApplication.exec();
*/
