/*========================================================================
 * Copyright (c) 2009 Sun Microsystems Laboratories
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

/*
	QtDockCalendar requires SLL support from Qt installation.
	This application will propably work well in N810
*/


function tr(s) { return s; }


function CalendarAuthenticationWidget(parent)
{
    QWidget.call(this);
    
    this.parent = parent;
    
    this.authStr = "";
    
    this.email = new QLineEdit(this);
	this.email.text = 'webmodemo@gmail.com';
	this.passwd = new QLineEdit(this);
	this.passwd.echoMode = QLineEdit.Password;
	this.passwd.text = 'livelywebmo';
	
	this.authBtn = new QPushButton("Authenticate", this);
	this.authBtn.clicked.connect(this, this.getAuthorization);
	
	this.emailLbl = new QLabel("Email:",this);
	this.passwdLbl = new QLabel("Password:", this);
	this.statusLbl = new QLabel("", this);
	
	this.authLayout = new QGridLayout(this);
	this.authLayout.addWidget(this.emailLbl,0,0,0);
	this.authLayout.addWidget(this.email,0,1,0);
	this.authLayout.addWidget(this.passwdLbl,1,0,0);
	this.authLayout.addWidget(this.passwd,1,1,0);
	this.authLayout.addWidget(this.statusLbl,2,0,0);
	this.authLayout.addWidget(this.authBtn,2,1,Qt.AlignRight);
	
	this.windowTitle = "Authenticate...";
}

CalendarAuthenticationWidget.prototype = new QWidget();

CalendarAuthenticationWidget.prototype.getAuthorization = function() {
	var email = this.email.text;
	email = email.replace('@', '%40');
	var ba = new QByteArray('Email=' + email + '&Passwd=' + this.passwd.text + '&source=Lively-CalWdgt-1&service=cl\0');
	
	manager = new QNetworkAccessManager(this);
	try {
		request = new QNetworkRequest(new QUrl('https://www.google.com/accounts/ClientLogin'));
		request.setHeader(QNetworkRequest.ContentTypeHeader, "application/x-www-form-urlencoded");
		manager.post(request, ba);
	}
	catch(err) {
		print(err);
		this.statusLbl.text = "<b>No OpenSSL!</b>";
	}
	manager.finished.connect(this, function(reply) {
		var replyString = reply.readAll().toString();
		var index = replyString.indexOf('Auth');
		if(replyString.indexOf('Error') !== -1 || index === -1) {
			this.statusLbl.text = "<b>Failed.</b>";
			this.authStr = "";
		}
		else {
			this.authStr = replyString.substring(index + 5,replyString.length-1);
			this.close();
		}
	});	
};

CalendarAuthenticationWidget.prototype.closeEvent = function (event) {
	this.parent.authStr = this.authStr;
	this.parent.calendar.selectionChanged();
	print(this.parent.authStr);
};

function QtDockCalendar(parent)
{
    QMainWindow.call(this);
	
	/* Edit this.timezone to match your calendar timezone 
	 * otherwise events will be misaligned. */
	this.timezone = 3; // Finland (daylight savings)
	
    this.windowTitle = tr("QtDockCalendar - Google Calendar Widget with Docking");
	
	
	this.authStr = "";
	//var mw = new QMainWindow(this);
	
	this.setupUI();
	this.setupMenu();
	this.setupActions();
	
    //this.resize(800,480);
}

QtDockCalendar.prototype = new QMainWindow();

QtDockCalendar.prototype.setupUI = function() {
	this.dockOptions = QMainWindow.AllowTabbedDocks | QMainWindow.AnimatedDocks | !QMainWindow.AllowNestedDocks;
	
	this.browser = new QTextBrowser(this);
	this.welcomeHtml = '<h3><center>QtCalendar</center></h3>'+
		'<p>This simple Google Calendar Widget utilizes HTTPS protocol for authorization. Qt with OpenSSL is required for this application to work.</p>' +
		'<center><b>Please authenticate first</b>' +
		'<p><small>Use menu: File -> Authenticate...</small></p></center>';
	this.browser.setHtml(this.welcomeHtml);
	
	this.setCentralWidget(this.browser);
	
	this.calendar = new QCalendarWidget(this);
	this.calendar.showToday();
	var calendarDW = new QDockWidget("QtDockCalendar", this);
	calendarDW.setWidget(this.calendar);

	calendarDW.features = !QDockWidget.DockWidgetClosable | QDockWidget.DockWidgetMovable | QDockWidget.DockWidgetFloatable;
//	calendarDW.floating = true;
	calendarDW.allowedAreas = Qt.RightDockWidgetArea;
	
	this.addEventW = new QWidget(this);
	
	this.titleLbl = new QLabel("Title:", this.addEventW);
	this.title = new QLineEdit(this.addEventW);
	this.startTimeLbl = new QLabel("Time:",this.addEventW);
	this.endTimeLbl = new QLabel("-",this.addEventW);
	this.startTime = new QTimeEdit(this.addEventW);
	this.startTime.displayFormat = "hh:mm";
	this.startTime.time = new QTime(12,00);
	this.endTime = new QTimeEdit(this.addEventW);
	this.endTime.displayFormat = "hh:mm";
	this.endTime.time = new QTime(13,00);
	this.locationLbl = new QLabel("Location:",this.addEventW);
	this.location = new QLineEdit(this.addEventW);
	this.eventDescriptionLbl = new QLabel("Description:",this.addEventW);
	this.eventDescription = new QTextEdit(this.addEventW);
this.eventDescription.minimumHeight = 30;	
this.cancelBtn = new QPushButton("Clear", this.addEventW);
	this.cancelBtn.clicked.connect(this, function() {
			this.clearAddEventDialog();
		});
	this.addEventBtn = new QPushButton("Add Event", this.addEventW);
	this.addEventBtn.clicked.connect(this, this.addEvent);
	
	this.addEventLayout = new QGridLayout(this.addEventW);
	this.addEventLayout.addWidget(this.titleLbl,0,0,0);
	this.addEventLayout.addWidget(this.title,0,1,1,3,0);
	this.addEventLayout.addWidget(this.startTimeLbl,1,0,0);
	this.addEventLayout.addWidget(this.startTime,1,1,0);
	this.addEventLayout.addWidget(this.endTimeLbl,1,2,Qt.AlignCenter);
	this.addEventLayout.addWidget(this.endTime,1,3,0);
	this.addEventLayout.addWidget(this.locationLbl,2,0,0);
	this.addEventLayout.addWidget(this.location,2,1,1,-1,0);
	this.addEventLayout.addWidget(this.eventDescriptionLbl,3,0,1,-1,0);
	this.addEventLayout.addWidget(this.eventDescription,4,0,1,-1,0);
	this.addEventLayout.addWidget(this.cancelBtn,5,0,0);
	this.addEventLayout.addWidget(this.addEventBtn,5,1,1,-1,0);
	this.addEventLayout.setContentsMargins(2,2,2,2);
	
	addEventDW = new QDockWidget("Add New Event", this);
	addEventDW.setWidget(this.addEventW);
	addEventDW.allowedAreas = Qt.RightDockWidgetArea | Qt.BottomDockWidgetArea;
	addEventDW.features = !QDockWidget.DockWidgetClosable | QDockWidget.DockWidgetMovable | QDockWidget.DockWidgetFloatable;
	
	this.addDockWidget(Qt.RightDockWidgetArea, addEventDW);
	this.addDockWidget(Qt.RightDockWidgetArea, calendarDW);
};

QtDockCalendar.prototype.setupMenu = function() {	
	this.fileMenu = new QMenu(this);
	this.fileMenu = this.menuBar().addMenu(tr("File"));
	
	var authAction = this.fileMenu.addAction("Authenticate...");
	authAction.triggered.connect(this, this.getAuthorization);
	
	this.fileMenu.addSeparator();
	
	var quitAction = this.fileMenu.addAction("Quit");
	quitAction.shortcut = "CTRL+Q";
	quitAction.triggered.connect(this, this.close);
	
	/*this.viewMenu = new QMenu(this);
	this.viewMenu = this.menuBar().addMenu(tr("&View"));
	* */
};

QtDockCalendar.prototype.setupActions = function() {	
	this.calendar.selectionChanged.connect(this, function() {
		if(this.authStr !== "") {
			var date = this.calendar.selectedDate;
			var temp = Date.parse(date);
			this.date = new Date();
			this.date.setTime(temp);
			this.dateStr = this.date.getFullYear().toString();
			if(this.date.getMonth().toString().length === 1) {
				this.dateStr += '-0' + (this.date.getMonth()+1);
			}
			else {
				this.dateStr += '-' + (this.date.getMonth()+1);
			}
			if(this.date.getDate().toString().length === 1) {
				this.dateStr += '-0' + this.date.getDate();
			}
			else {
				this.dateStr += '-' + this.date.getDate();
			}
			print(this.dateStr);
			this.browser.setHtml("");
			this.requestCalendarData();
		}
		else{
			this.browser.setHtml(this.welcomeHtml);
		}
	});
	
	/* Bind handler for keyPressEvent */
	QtDockCalendar.prototype.keyPressEvent = keyDown;

};

/* Key event handler */
keyDown = function(event) {
	var key = event.key();

    if (key === Qt.Key_F6) {
		this.toggleWindowState();
	}
};

QtDockCalendar.prototype.toggleWindowState = function() {
	var state = this.windowState() ^ Qt.WindowFullScreen;
	/* State is 4 when fullscreen, 0 in normal state */
	if(state !== 4) {
		this.setWindowState(Qt.WindowNoState | !Qt.WindowFullScreen);
	}
	else {
		this.setWindowState(Qt.WindowFullScreen);
	}
};

QtDockCalendar.prototype.clearAddEventDialog = function() {
		this.title.text = "";
		this.startTime.time = new QTime(12,00);
		this.endTime.time = new QTime(13,00);
		this.location.text = "";
		this.eventDescription.plainText = "";
};

QtDockCalendar.prototype.getAuthorization = function() {
	var authWdgt = new CalendarAuthenticationWidget(this);
	authWdgt.show();
};

QtDockCalendar.prototype.addEvent = function() {
	if(this.authStr === "") {
		return;
	}
		
	//print('START: ' + this.startTime.time.toString());
	var startTime = new QTime(this.startTime.time.hour()-this.timezone,this.startTime.time.minute());
	var endTime = new QTime(this.endTime.time.hour()-this.timezone,this.endTime.time.minute());
	print('starttime '+startTime.toString());
	print('endtime '+endTime.toString());
	var ba = new QByteArray("<entry xmlns=\'http://www.w3.org/2005/Atom\'   xmlns:gd=\'http://schemas.google.com/g/2005\'>   <category scheme=\'http://schemas.google.com/g/2005#kind\'    term=\'http://schemas.google.com/g/2005#event\'></category>  <title type=\'text\'>"+ this.title.text +'</title>  <content type=\'text\'>'+ this.eventDescription.plainText +'</content>  <gd:transparency    value=\'http://schemas.google.com/g/2005#event.opaque\'>  </gd:transparency>  <gd:eventStatus    value=\'http://schemas.google.com/g/2005#event.confirmed\'>  </gd:eventStatus>  <gd:where valueString=\'' + this.location.text + '\'></gd:where>  <gd:when startTime=\''+this.dateStr+'T'+startTime.toString()+'.000Z\'    endTime=\''+this.dateStr+'T'+ endTime.toString() +'.000Z\'></gd:when></entry>\0');
	
	manager = new QNetworkAccessManager(this);
	if(this.addEventUrl === undefined) {
		request = new QNetworkRequest(new QUrl('http://www.google.com/calendar/feeds/default/private/full'));
	}
	else {
		request = new QNetworkRequest(new QUrl(this.addEventUrl));
		//request.setRawHeader(new QByteArray('Set-Cookie'), new QByteArray(this.setCookieHeader));
		if(this.addEventCookieHdr) {
			request.setRawHeader(new QByteArray('Set-Cookie'), new QByteArray(this.addEventCookieHdr));
		}
	}

	request.setRawHeader(new QByteArray('Authorization'),new QByteArray('GoogleLogin auth=' + this.authStr));	
	request.setHeader(QNetworkRequest.ContentTypeHeader, "application/atom+xml");

	manager.post(request, ba);
	manager.finished.connect(this, this.addEventReply);
};

QtDockCalendar.prototype.addEventReply = function(reply) {
	print(reply.attribute(QNetworkRequest.HttpStatusCodeAttribute));
	if(reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) === 302) {
		this.addEventCookieHdr = reply.rawHeader(new QByteArray('Set-Cookie'));
		var result = reply.readAll().toString();
		var index = result.indexOf('A HREF="');
		var index2 = result.indexOf('">here</A>');
		if(index !== -1 && index2 !== -1) {
			this.addEventUrl = result.substring(index + 8, index2);
		}
		this.addEvent();
	}
	else if (reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) === 201) {
		this.browser.setHtml('<b>New event created</b>');
		this.calendar.selectionChanged();
		this.clearAddEventDialog();
	}
	else {
		this.browser.setHtml(reply.readAll().toString());
	}
};

QtDockCalendar.prototype.requestCalendarData = function() {
	this.cursor = Qt.WaitCursor;
	var manager = new QNetworkAccessManager();
	var request = undefined;
	if(this.cookie) {
		request = new QNetworkRequest(new QUrl('http://www.google.com/calendar/feeds/default/private/full?start-min=' + this.dateStr + 'T00:00:00' + '&start-max='+ this.dateStr +'T23:59:59' + '&gsessionid=' + this.cookie));
		}
	else {
		request = new QNetworkRequest(new QUrl('http://www.google.com/calendar/feeds/default/private/full?start-min=' + this.dateStr + 'T00:00:00' + '&start-max='+ this.dateStr +'T23:59:59'));
	}
	request.setRawHeader(new QByteArray('Authorization'),new QByteArray('GoogleLogin auth=' + this.authStr));
	if(this.setCookieHeader !== undefined) {
		request.setRawHeader(new QByteArray('Set-Cookie'), new QByteArray(this.setCookieHeader));
	}
	manager.get(request);
	manager.finished.connect(this, this.entryRequestReply);
	this.cursor = Qt.ArrowCursor;
};

QtDockCalendar.prototype.entryRequestReply = function(reply) {
	// Got 302 Moved Temporarily?
	print(reply.attribute(QNetworkRequest.HttpStatusCodeAttribute));
	if(reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) === 302) {
		this.setCookieHeader = reply.rawHeader(new QByteArray('Set-Cookie'));
		var result = this.setCookieHeader.toString();
		var index = this.setCookieHeader.indexOf('S=calendar=');
		var index2 = this.setCookieHeader.indexOf(';Expires');
		if(index !== -1 && index2 !== -1) {
			this.cookie = result.substring(index + 11, index2);
		}
		this.requestCalendarData();
	}
	else if (reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) === 200) {
		var xml = new QXmlStreamReader();
		xml.addData(reply.readAll());
		
		var entrys = [];
		
		var authorStr = "";
		var titleString = "";
		var whereString = "undefined";
		var currentTopLevelTag = undefined;
		var description = "";
		var attributes = undefined;
		
		while (!xml.atEnd()) {
			xml.readNext();
        
			// We are at the start of an element
			if(xml.isStartElement()) {
				var currentTag = xml.name().toString();
				
				if(xml.name() === "entry") {
					currentTopLevelTag = "entry";
				}
				
				if(xml.name() === "author") {
					currentTopLevelTag = "author";
				}

				if(xml.name() === "when") {
					attributes = xml.attributes();
					
					var atomdate = attributes.at(0).value();
					var itemPubDateDay = atomdate.substr(0, 10);
					var itemPubDateTime = atomdate.substr(11, 5 );
					
					atomdate = attributes.at(1).value();
					var itemPubEndTime = atomdate.substr(11, 5 );
				}
				
				if(xml.name() === "where") {
					attributes = xml.attributes();
					
					whereString = attributes.at(0).value();
				}
			} 
			// We are at the end of an element
			else if (xml.isEndElement()) {
				if(xml.name() === "author") {
					currentTopLevelTag = "entry";
				}
				if(xml.name() === "entry") {
					var entry = { 
						title: titleString,
						date: itemPubDateDay,
						startTime: itemPubDateTime,
						endTime: itemPubEndTime,
						author: authorStr,
						location: whereString,
						description: description
					};
					
					entrys.push(entry);
				}
			} 
			// Collecting items we would probably want to save
			else if (xml.isCharacters() && !xml.isWhitespace()) {
				if(currentTopLevelTag === "entry" && currentTag === "title") {
					titleString = "";
					titleString = xml.text().toString();
				}
				if(currentTopLevelTag === "author" && currentTag === "name") {
					authorStr = "";
					authorStr = xml.text().toString();
				}
				if(currentTag === "content") {
					description = "";
					description = xml.text().toString();
				}
			}
		}
		if (xml.hasError() && (xml.error() !== QXmlStreamReader.PrematureEndOfDocumentError)) {
			print("XML ERROR:", xml.lineNumber() + ":", xml.errorString());
		}
		var htmlString = this.createHtml(entrys);
		if(htmlString) {
			this.browser.setHtml(htmlString);
		}
	}
	else {
		this.browser.setHtml(reply.readAll().toString());
	}
};


function entrySort(a, b) {
	if(a.startTime < b.startTime) {
		return -1;
	}
	else if( a.startTime > b.startTime) {
		return 1;
	}
	else {
		return 0;
	}
}

QtDockCalendar.prototype.createHtml = function(entrys) {
	var count = entrys.length;
	
	var htmlString = '<html><head>' +
		'<style type="text/css">' +
		'strong { }' +
		'p {margin: 5px 0px 5px 30px;}' +
		'small {color: gray; font-size: 70%; line-height: 80%}'+
		'</style></head><body>';
	
	entrys.sort(entrySort);
	
	htmlString += 'Events on <i>' + (this.date.getMonth() + 1) + '/' + this.date.getDate() + '/' +this.date.getFullYear() + '</i>:<br>';
	
	htmlString += '<table>';
	for(var i = 0; i < entrys.length; i = i + 1) {
		htmlString += '<tr><td>';
		var entry = entrys[i];
		if(entry.title.length > 0) {
			htmlString += '<strong>' + entry.title + ' </strong><br>';
		}
		if(entry.startTime.length > 0){
			htmlString += '<small> From: <i>'	+ entry.startTime + '</i> To: <i>' + entry.endTime +'</i></small>';
		}
		if(entry.description.length > 0){
			htmlString += '<p class="content">' + entry.description + '</p>';
		}
		if(entry.location.length > 0){
			htmlString += '<small> Location: ' + entry.location + '</small><br>';
		}
		if(entry.author.length > 0){
			htmlString += '<small> Author: ' + entry.author + '</small><hr color="black">';
		}
		htmlString += '\n\n';
		htmlString += '</tr></td>';
	}
	htmlString += '</table>';
	
	htmlString += '<small>';
	if(count === 1) {
		htmlString += 'One event found.';
	}
	else if(count > 1) {
		htmlString += count + ' events found.';
	}
	else{
		htmlString += 'No events found.';
	}
	htmlString += '</small></body></html>';
	
	return htmlString;
};

function entrySort(a, b) {
	if(a.startTime < b.startTime) {
		return -1;
	}
	else if( a.startTime > b.startTime) {
		return 1;
	}
	else {
		return 0;
	}
}

QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};



function CalendarWrapper(parent)
{
    QWidget.call(this,parent);
    this.wdgt = new QtDockCalendar(this);
	
	this.layout = new QVBoxLayout(this);
	this.layout.addWidget(this.wdgt,0,0);
	this.layout.setContentsMargins(0,0,0,0);
}

CalendarWrapper.prototype = new QWidget();

/* To use the application in standalone mode, remove the comment around this code 
QCoreApplication.applicationName = tr("QtCalendar");
var wdgt = new QtDockCalendar();
wdgt.show();

QCoreApplication.exec();*/
