/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

// QtFlickr.js
// Animated Flickr Photo Viewer uses current Twitter trends as tags.

function tr(s) { return s; }

function QtFlickrTwitterTrends(parent)
{
    QWidget.call(this, parent);

	this.flickrUrl = "http://api.flickr.com/services/feeds/photos_public.gne?format=rss2";
	this.imageUrls = [];
	
	this.currentTagLabel = new QLabel("", this);

	this.imageLabel = new QLabel(this);
	this.imageLabel.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored);
    this.imageLabel.alignment = Qt.AlignCenter;

	this.imagePixmap = new QPixmap();
	this.nextPixmap = new QPixmap();

	this.changeTagsTimer = new QTimer(this);
	this.changeTagsTimer.timeout.connect(this, this.changeTagsTimerTimeout);
	this.changeTagsTimer.start(30000); // 30 seconds

	this.fetchImageTimer = new QTimer(this);
	this.fetchImageTimer.timeout.connect(this, this.fetchImageTimerTimeout);
	
	this.rotTimer = new QTimer(this);
	this.rotTimer.timeout.connect(this, this.rotTimerTimeout);
	
	this.angle = 90;

	var hBoxLayout = new QHBoxLayout();
	hBoxLayout.addWidget(new QLabel(tr("Tags:"),this),0,0);
	hBoxLayout.addWidget(this.currentTagLabel,1,0);
	
	this.layout = new QVBoxLayout(this);
	this.layout.addLayout(hBoxLayout);
	this.layout.addWidget(this.imageLabel,1,0);

    this.windowTitle = tr("QtTwitterTrendPhotos: Twitter Trends Photo Viewer");
	this.resize(300,300);
	
	this.getTwitterTrends();
}

QtFlickrTwitterTrends.prototype = new QWidget();

QtFlickrTwitterTrends.prototype.changeTagsTimerTimeout = function() {
	this.getTwitterTrends();
};

QtFlickrTwitterTrends.prototype.fetchImageTimerTimeout = function() {
	this.showRandomImage();
};

QtFlickrTwitterTrends.prototype.rotTimerTimeout = function() {
	var speed = 0;
	
	this.angle = this.angle + 10;
	
	if(this.angle > 450) {
		this.angle = 90;
	}
	else if((this.angle === 90) || (this.angle === 270)) {
		this.imagePixmap = new QPixmap(this.nextPixmap);
	}
	
	var trans = new QTransform();
	trans.rotate(this.angle,Qt.YAxis);
	trans.rotate(this.angle,Qt.ZAxis);

	this.imageLabel.setPixmap(this.imagePixmap.transformed(trans,Qt.SmoothTransformation));
};


QtFlickrTwitterTrends.prototype.showRandomImage = function() {
	if(this.imageUrls.length !== 0) {
		this.fetchImageTimer.stop();
		var accessManager = new QNetworkAccessManager(this);
		accessManager["finished(QNetworkReply*)"].connect(this, this.imageReplyFinished);
		var urlString = this.imageUrls[Math.floor((this.imageUrls.length-2)*Math.random())];
		accessManager.get(new QNetworkRequest((new QUrl(urlString))));
	}
};

QtFlickrTwitterTrends.prototype.imageReplyFinished = function(reply) {
	var data = reply.readAll();
	this.nextPixmap.loadFromData(data, 0, Qt.AutoColor);
	if(!this.nextPixmap.isNull()) {
		this.nextPixmap = this.nextPixmap.scaled(this.imageLabel.width*0.8,this.imageLabel.height*0.8,Qt.KeepAspectRatio,Qt.SmoothTransformation);
		this.rotTimer.start(50);
	}
	else {
		/* imagePixmap was null, image data was not available (bad url)*/
		this.imageLabel.text = '<center>No image available</center>';
	}
	/* fetch next image after 3 seconds */
	this.fetchImageTimer.start(3000);
};

QtFlickrTwitterTrends.prototype.getTwitterTrends = function() {
	var url = 'http://search.twitter.com/trends.json';
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, twitterReplyFinished);
	accessManager.get(new QNetworkRequest(new QUrl(url)));
};

twitterReplyFinished = function(reply) {
	var trendJSONString = reply.readAll().toString();
	var trendJSONObject = eval('(' + trendJSONString + ')');
	var tags = [];
	for(i=0; i < trendJSONObject.trends.length; i=i+1) {
		tags.push(trendJSONObject.trends[i].name);
	}
	this.loadFeed(tags);
};


QtFlickrTwitterTrends.prototype.loadFeed = function(tags) {
	this.imageUrls = [];
	var currentTag = tags[Math.floor(Math.random()*tags.length)];
	var url = this.flickrUrl + "&tags=" + currentTag;
	this.currentTagLabel.text = currentTag;
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, flickrReplyFinished);
	accessManager.get(new QNetworkRequest(new QUrl(url)));
};

flickrReplyFinished = function(reply) {
	var xml = new QXmlStreamReader();
	xml.addData(reply.readAll());
	
	while (!xml.atEnd()) {
        xml.readNext();
        if(xml.isStartElement()) {
            if(xml.name() === "enclosure") {
            	this.imageUrls.push(xml.attributes().value("url").toString());
			}
		} 
    }
	
	/* fetch next image after 3 seconds */
	this.fetchImageTimer.start(3000);
};

QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};

/*jslint evil: true */

/* To use the application in standalone mode, remove the comment around this code
var wdgt = new QtFlickrTwitterTrends();
wdgt.show();

QCoreApplication.exec();  */