/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

// QtGeoTweet
// Obtains a set of current Tweets from Twitter and attempts to show them on 
// a Google map. 
// Development and improvement ideas: 
// * search of tweets geographically, i.e. the user should be able to define 
//   an area on the map and the search would be made according to it. 
// * use GPS in mobile devices to set the location
// * select a location by clicking the map or by searching textually
// * fetch and present other relevant data
// * make the objects to work in a more sensible way when resizing etc.
// * add the possibility to follow twitter updates on a certain location
// * show the user image as marker/in a marker 
// * Show current Tweet trends in an area

function tr(s) { return s; }

function QtGeoTweet(parent) {
	QWidget.call(this, parent);

	// QWebView handles showing a Google map
	this.view = new QWebView();
	this.view.load(new QUrl('http://www.cs.tut.fi/~nyrhinen/index.html'));
	this.view["loadFinished(bool)"].connect(this, this.loadFinished);

	// For showing an image of the tweeter selected.
	this.userImageLbl = new QLabel(this);

	// Initialization of tweetTreeWidget
	this.tweetTreeWidget = new QTreeWidget(this);
	// An array for the labels
	var headerLabels = [];
	// Add the labels to the array
	headerLabels.push(tr("User"));
	headerLabels.push(tr("Tweet"));
	headerLabels.push(tr("Location"));
	// set the created labels as headers
    this.tweetTreeWidget.setHeaderLabels(headerLabels);
    // Make the header columns resizable by the user. 
    this.tweetTreeWidget.header().setResizeMode(QHeaderView.Interactive);
    // Make the column containing the tweet wider.
    this.tweetTreeWidget.header().resizeSection (1,350);
    // Connect an action to a click of an item (the view is updated when an item
    // is clicked in the tree. 
	this.tweetTreeWidget["itemClicked(QTreeWidgetItem*, int)"].connect(this, this.itemClicked);

	// a push button for updating the data and an update action connected to it
	this.updateTweetsButton = new QPushButton("Update");
	this.updateTweetsButton.clicked.connect(this, this.getTweets);

	// Add splitters and attach elements to them
	var hSplitter = new QSplitter(this);
	hSplitter.addWidget(this.tweetTreeWidget);
	hSplitter.addWidget(this.userImageLbl);
	var vSplitter = new QSplitter(Qt.Vertical, this);
	vSplitter.addWidget(this.view);
	vSplitter.addWidget(hSplitter);
	vSplitter.addWidget(this.updateTweetsButton);
	
	// The vertical main layout to which the widgets and other layouts are added
	this.actionLayout = new QVBoxLayout();
	this.actionLayout.addWidget(vSplitter,0,0);
	this.actionLayout.addWidget(this.updateTweetsButton, 0, 0);

	// Set the size, layout and window title	
	this.resize(640, 480);
    this.setLayout(this.actionLayout);
	this.setWindowTitle(tr("QtGeoTweet"));
	
	// Get tweets
	this.getTweets();	
}

// The main widget is of type QWidget
QtGeoTweet.prototype = new QWidget();

// Gets a list of current tweets available in JSON format. 
QtGeoTweet.prototype.getTweets = function() {
	var url = 'http://twitter.com/statuses/public_timeline.json';
	var accessManager = new QNetworkAccessManager();
	accessManager.get(new QNetworkRequest(new QUrl(url)));
	accessManager["finished(QNetworkReply*)"].connect(this, this.twitterReplyFinished);
};

// Processes the JSON file containing tweets and creates objects of the contents.
// The objects are then stored to a QTreeWidget
QtGeoTweet.prototype.twitterReplyFinished = function(reply) {
	var stream = new QTextStream(reply);
    var tweetJSONString = stream.readAll();
	var tweetJSONObject = eval('(' + tweetJSONString + ')');
	
	// Empty the tree widget containing the tweets
	this.tweetTreeWidget.clear();
	// TODO: Handle items with no location, null location or undefined location.
	for(i=0; i < tweetJSONObject.length; i++) {

		// Create a new QTreeWidgetItem and set its fields according to the 
		// JSON object. 
		var treeItem = new QTreeWidgetItem();
		treeItem.setText(0, tweetJSONObject[i].user.name);
		treeItem.setText(1, tweetJSONObject[i].text);
		treeItem.setText(2, tweetJSONObject[i].user.location);
		treeItem.setText(3, tweetJSONObject[i].user.profile_image_url);
		// Set a tooltip text to show the Tweet completely when hovering with 
		// the mouse. 
		treeItem.setToolTip(1, tweetJSONObject[i].text);

		// Add the item as a new top level item
		this.tweetTreeWidget.addTopLevelItem(treeItem);

		// TODO: Fine-tune the search for the location (use the data 
		// (direct coordinates) provided by the iPhone clients and others 
		// more effectively).
		var location = tweetJSONObject[i].user.location;

		// Search for a location
		this.searchTweetAddress(location);
	}
};

// Searches for an address based on the location
QtGeoTweet.prototype.searchTweetAddress = function(location, text) {
	this.geoCode(location, searchTweetAddressCallback);	
};

// Based on the request, a geocoded result is obtained
searchTweetAddressCallback = function(reply) {
	var replyString = reply.readAll().toString();
	// Array for the geocodes
	var geoCodeArray = [];
	geoCodeArray = replyString.split(",");
	var htmlResponse = geoCodeArray[0];
	var accuracy = geoCodeArray[1];
	var lat = geoCodeArray[2];
	var lng = geoCodeArray[3];
	if(htmlResponse == "200") {
		this.setCenter(lat, lng, parseInt(accuracy, 10) + 7);
	}
};

// Sends a request to Google to geocode a location
QtGeoTweet.prototype.geoCode = function(location, callback) {
	var APIKey = 'ABQIAAAAjlFvU2tzDcjGvu_UgwNBmRQK3nvu5HLNMiWiNO-NKEHBcpToshSiHtBg-gl-shA2gvPoH2Y-BrW-qQ';
	var requestStr = "http://maps.google.com/maps/geo?q=" +
		location + "&output=csv&key=" + APIKey;
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, callback);
	accessManager.get(new QNetworkRequest(new QUrl(requestStr)));
};

// Set map center point and zoom level
QtGeoTweet.prototype.setCenter = function(lat,lng,zoom) {
	var scriptString = 
	"map.setCenter(new GLatLng(" + lat + "," + lng + ")," + zoom + ");";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
	// TODO: Change the marker to show tweets etc.
	this.addMarker(lat, lng, 'marker');
};

QtGeoTweet.prototype.loadFinished = function(ok) {
	// If page loaded was the map page, do some initialization to the map
	if(this.view.page().mainFrame().url.toString() == 
			"http://www.cs.tut.fi/~nyrhinen/index.html") {
		var scriptString = 
	"var userClickedPoint = undefined;" +
	"var userClickedMarker = undefined;" +
	"var myClickListener = undefined;" +
	"var fromMarker = undefined;" +
	"var toMarker = undefined;" +
	"map.addControl(new GMapTypeControl());" +
	"map.addControl(new GLargeMapControl());" +
	"map.setCenter( new GLatLng(0,0),1 );" +
	"var markers = new Array();" +
	"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
    	"userClickedPoint = point;" +
    	"userClickedMarker = marker;" +
	"});" +
	"GEvent.addListener(map, \"moveend\", function() {" +
  		"userClickedPoint = undefined;" +
  		"userClickedMarker = undefined;" +
	"});";
		this.view.page().mainFrame().evaluateJavaScript(scriptString);
	}
};

// When an item is clicked in the tree, the location of the user is searched 
// for and an image of the user is shown. 
QtGeoTweet.prototype.itemClicked = function(item) {
	
	// Search for an address of item's location
	this.searchTweetAddress(item.text(2));
		
	// Instantiate a new QNetworkAccessManager and load the image of the 
	// user profile.
	var accessManager = new QNetworkAccessManager(this);
	var userLogo = new QPixmap();
	var reqReply = accessManager.get(new QNetworkRequest(new QUrl(item.text(3).toString())));
	function addImg() {
		var data = reqReply.readAll();
		userLogo.loadFromData(data, 0, Qt.AutoColor);
		userLogo = userLogo.scaled(75, 100, Qt.KeepAspectRatio, Qt.SmoothTransformation);
		this.userImageLbl.setPixmap(userLogo);
	}
	reqReply.finished.connect(this, addImg);
};

// Adds a marker to a Google map
QtGeoTweet.prototype.addMarker = function(latitude, longitude, description) {
	description = latitude + ',' +longitude;
	var scriptString = 
	"var newMarker = new GMarker(new GLatLng(" + latitude + 
						"," + longitude + "),{ title: \""+description+"\" });" +
	"newMarker.bindInfoWindowHtml(newMarker.getTitle().toString());" +
	"markers.push(newMarker);" +
	"map.addOverlay(markers[markers.length-1]);";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

// Returns the contents of QByteArray as a string
QByteArray.prototype.toString = function() {
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};

/* To use the application in standalone mode, remove the comment around this code
var wdgt = new QtGeoTweet();
wdgt.show();
QCoreApplication.exec();
*/




