/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

/*
 * This application is a GeoRSS News Reader. You can add GeoRSS feeds
 * to the application and then show feeds over Google Map. Application
 * uses Google Maps API to show geocoded RSS feeds. If your feed is not
 * GeoRSS feed, then application uses Geonames.org RSS-to-GeoRSS 
 * (http://www.geonames.org/rss-to-georss-converter.html) converter to
 * geocode the feed. Read more about GeoRSS from Wikipedia:
 * http://en.wikipedia.org/wiki/GeoRSS
*/ 
function tr(s) { return s; }

function QtMapNews3d(parent) {
	QWidget.call(this, parent);
	
	// API_KEY from Google. Everybody should get their own
	this.RSS_TO_GEO_RSS = "http://ws.geonames.org/rssToGeoRSS?feedUrl=";
	this.MAGIC_NUMBER = "25022009";
	this.selectedItem = undefined;
	this.lastLoadedUrl = undefined;
	
	// Buttons and connects
	this.homeButton = new QPushButton(tr(" Back to Map View "), this);
	this.homeButton.clicked.connect(this, this.backToHome);
	this.homeButton.hide();
	this.addButton = new QPushButton(tr("Add..."), this);	
	this.addButton.clicked.connect(this, this.addButtonClicked);
	this.removeButton = new QPushButton(tr("Remove"), this);
	this.removeButton.clicked.connect(this, this.removeButtonClicked);
	this.saveButton = new QPushButton(tr("Save..."), this);
	this.saveButton.clicked.connect(this, this.saveTree);
	this.loadButton = new QPushButton(tr("Load..."), this);
	this.loadButton.clicked.connect(this, this.loadTree);
	this.removeButton.enabled = false;
	
	// building button box layout
	var buttonBox = new QWidget();
	var buttonBoxLayout = new QVBoxLayout(buttonBox);
	buttonBoxLayout.addWidget(this.addButton,0,0);
	buttonBoxLayout.addWidget(this.removeButton,0,0);
	buttonBoxLayout.addWidget(this.saveButton,0,0);
	buttonBoxLayout.addWidget(this.loadButton,0,0);
	buttonBoxLayout.addStretch();

	this.view = new QWebView(this);
	this.view.settings().setAttribute(QWebSettings.PluginsEnabled, true);
	this.view.page().mainFrame().setScrollBarPolicy(Qt.Horizontal,Qt.ScrollBarAlwaysOff);
	this.view.page().mainFrame().setScrollBarPolicy(Qt.Vertical,Qt.ScrollBarAlwaysOff);
	
	this.progressBar = new QProgressBar(this);
	this.progressBar.hide();
	
	// rssTree includes feeds
	this.rssTree = new QTreeWidget(this);
	var headerLabels = [];
	headerLabels.push(tr("Name"));
	headerLabels.push(tr("URL"));
	//headerLabels.push(tr("Is GeoRSS"));
    this.rssTree.setHeaderLabels(headerLabels);
    this.rssTree.header().setResizeMode(QHeaderView.Interactive);
    this.rssTree.itemSelectionChanged.connect(
        this, this.itemSelectionChanged);
	this.rssTree["itemDoubleClicked(QTreeWidgetItem*, int)"].connect(
        this, this.itemDoubleClicked);
	
	hSplitter = new QSplitter(this);
	hSplitter.addWidget(this.rssTree);
	hSplitter.addWidget(buttonBox);
	
	splitter = new QSplitter(Qt.Vertical, this);
	splitter.addWidget(this.view);
	splitter.addWidget(hSplitter);
	
	hboxLayout = new QHBoxLayout();
	hboxLayout.addWidget(this.homeButton,0,0);
	hboxLayout.addStretch();
	hboxLayout.addWidget(this.progressBar,0,0);
	
	
	this.layout = new QVBoxLayout(this);
	this.layout.addLayout(hboxLayout);
	this.layout.addWidget(splitter,0,0);
	
	this.view["loadFinished(bool)"].connect(this, this.loadFinished);
	this.view.loadStarted.connect(this, this.loadStarted);
	this.view["loadProgress(int)"].connect(this, this.loadProgress);
	
	try {
		this.view.load(new QUrl("http://www.cs.tut.fi/~salmin39/map/map_3d.html"));
	} catch(err) {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + err;
		messageBox.show();
	}
	
	this.windowTitle = tr("QtMapNews3D");
	this.resize(800,480);
}

QtMapNews3d.prototype = new QWidget();

// Set map center point and zoom level
QtMapNews3d.prototype.setCenter = function(lat,lng,zoom) {
	var scriptString = 
	"map.setCenter(new GLatLng(" + lat + "," + lng + ")," + zoom + ");";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

QtMapNews3d.prototype.backToHome = function() {
	try {
		this.view.load(new QUrl("http://www.cs.tut.fi/~salmin39/map/map_3d.html"));
	} catch(err) {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + err;
		messageBox.show();
	}
};

QtMapNews3d.prototype.loadStarted = function() {
	this.homeButton.show();
	this.progressBar.show();
};

QtMapNews3d.prototype.loadProgress = function(progress) {
	this.progressBar.setValue(progress);
	if(progress === 100) {
		this.progressBar.hide();
	}
};

QtMapNews3d.prototype.loadFinished = function(ok) {
	// If page loaded was the map page, do some initialization to the map
	if(this.view.page().mainFrame().url.toString() === 
			"http://www.cs.tut.fi/~salmin39/map/map_3d.html") {
		var scriptString = '';
		this.view.page().mainFrame().evaluateJavaScript(scriptString);
		
		// Hide homeButton when in initial view
		this.homeButton.hide();
		this.progressBar.hide();
	}
};


QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};

// Add new feed to rssTree
QtMapNews3d.prototype.addButtonClicked = function() {
	
	var name = QInputDialog.getText(this, "Give a name for the new feed",
                                          "Name:", QLineEdit.Normal,
                                          "No name yet",0);
										  
	var url = QInputDialog.getText(this, "Give an URL for the new feed",
                                          "URL:", QLineEdit.Normal,
                                          "http://",0);

	var isGeoRss = "false";
	
	var msgBox = new QMessageBox();
	msgBox.text = "Is this feed a GeoRSS feed?";
	msgBox.informativeText = "Is this feed a GeoRSS feed?";
	var buttons = (QMessageBox.Yes | QMessageBox.No);
	msgBox.standardButtons = buttons;
	msgBox.icon = QMessageBox.Question;
	msgBox.setDefaultButton(QMessageBox.No);
	
	var ret = msgBox.exec();
	switch(ret) {
		case 16384:
			isGeoRSS = "true";
			break;
		case 65536:
			isGeoRSS = "false";
			break;
		default:
			isGeoRSS = "false";
			break;
	}
										  
	// Add new item to parent rssTreeWidget
	var item = new QTreeWidgetItem();
	item.setText(0, name);
	item.setText(1, url);
	item.setText(2, isGeoRSS);
	
	this.rssTree.addTopLevelItem(item);
};

QtMapNews3d.prototype.removeButtonClicked = function() {
	this.rssTree.takeTopLevelItem(
		this.rssTree.indexOfTopLevelItem(this.selectedItem[0]));
};

QtMapNews3d.prototype.setEnabled = function(enable) {
	if(enable === true) {
		this.setEnabled(true);
	}
	else {
		this.setDisabled(true);
	}
};

// When item is doubleClicked we use Google Maps API to show it on the
// map. If the feed is not GeoRSS feed we try to geocode it with
// Geonames.org RSS-to-GeoRSS converter.
QtMapNews3d.prototype.itemDoubleClicked = function(item) {
	var feedUrl = "";
	if(item.text(2) === "false") {
		feedUrl = this.RSS_TO_GEO_RSS + item.text(1);
	}
	else {
		feedUrl = item.text(1);
	}
	
	this.lastLoadedUrl = feedUrl;
	
	var scriptString = 
		'map.clearOverlays();'+
		'map.closeInfoWindow();'+
		'var rss = new FE.GeoRSS("'+feedUrl+'");'+
		'map.addOverlay(rss);'+
		'map.panTo(rss.getLocation());';

	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

QtMapNews3d.prototype.itemSelectionChanged = function()
{
	this.selectedItem = this.rssTree.selectedItems();
	if(this.selectedItem[0] !== null) {
		this.removeButton.enabled = true;
	} 
	else {
		this.removeButton.enabled = false;
	}
};

QtMapNews3d.prototype.showEvent = function (event) {
	this.loadTree("QtMapNews_AutoOpenFeeds.csv");
};

QtMapNews3d.prototype.closeEvent = function (event) {
	this.saveTree("QtMapNews_AutoOpenFeeds.csv");
};


QtMapNews3d.prototype.saveTree = function(fileToUse)
{	
	var filename = undefined;
	if(fileToUse) {
		filename = fileToUse;
	}
	else {
		filename = QFileDialog.getSaveFileName(this, "Save to CSV file", ".", "CSV Files (*.csv)");
	}
		
	if(filename !== "") {
		if(filename.substring(filename.length - 4, filename.length) !== ".csv") {
			filename = filename + ".csv";
		}
		var file = new QFile(filename);
		if(file.exists()) {
			file.remove();
		}
		file.open(QIODevice.ReadWrite);
		var stream = new QTextStream(file);
		stream.writeString(this.MAGIC_NUMBER + "\n");
		for(i = 0; i < this.rssTree.topLevelItemCount; i=i+1) {
			var treeItem = this.rssTree.topLevelItem(i);
			stream.writeString(treeItem.text(0) + "," +
								treeItem.text(1) + "," +
								treeItem.text(2) + "\n");
		}
		file.close();
	}
};

QtMapNews3d.prototype.loadTree = function(fileToUse)
{
	var filename = undefined;
	if(fileToUse) {
		filename = fileToUse;
	}
	else {
		filename = QFileDialog.getOpenFileName(this, "Load CSV file", ".", "CSV Files (*.csv)");
	}
		
	if(filename !== "") {
		var file = new QFile(filename);	
		if(file.exists()) {
			file.open(QIODevice.ReadOnly);
			var stream = new QTextStream(file);
			if(stream.readLine() !== this.MAGIC_NUMBER) {
				var messageBox = new QMessageBox(this);
				messageBox.text = "Error: File is not valid.";
				messageBox.show();
				file.close();
				return;
			}
			this.rssTree.clear();
			while(!stream.atEnd()) {
				var line = stream.readLine();
				var poiArray = [];
				poiArray = line.split(",");
				var poiItem = new QTreeWidgetItem();
				poiItem.setText(0, poiArray[0]);
				poiItem.setText(1, poiArray[1]);
				poiItem.setText(2, poiArray[2]);
				this.rssTree.addTopLevelItem(poiItem);
			}
			file.close();
		}
	}
};


/* To use the application in standalone mode, remove the comment around this code



var browser = new QtMapNews3d();
browser.show();

QCoreApplication.exec();

*/

