/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

function tr(s) { return s; }

/* poiDialog is used to add new marker on the map with name and user 
 * editable location. poiDialog could be used to change marker 
 * parameters. */
function poiDialog(parent, lat, lng, name) {
	QWidget.call(this);
	
	this.okButton = new QPushButton(tr("OK"), this);
	
	// Button connects
	this.okButton.clicked.connect(this, this.ok);
	
	this.nameLabel = new QLabel("Name:");
	this.latLabel = new QLabel("Latitude:");
	this.lngLabel = new QLabel("Longitude");
	
	this.parent = parent;
	
	this.nameLineEdit = new QLineEdit(this);
	this.nameLineEdit.text = name;
	this.latLineEdit = new QLineEdit(this);
	this.latLineEdit.text = lat;
	this.latLineEdit.setValidator(new QDoubleValidator(this));
	this.lngLineEdit = new QLineEdit(this);
	this.lngLineEdit.text = lng;
	this.lngLineEdit.setValidator(new QDoubleValidator(this));
	
	// Building layout
	var layout = new QVBoxLayout(this);

	layout.addWidget(this.nameLabel,0,0);
	layout.addWidget(this.nameLineEdit,0,0);
	layout.addWidget(this.latLabel,0,0);
	layout.addWidget(this.latLineEdit,0,0);
	layout.addWidget(this.lngLabel,0,0);
	layout.addWidget(this.lngLineEdit,0,0);
	layout.addWidget(this.okButton,0,0);
	
	this.windowTitle = tr("Add new POI");
}

poiDialog.prototype = new QWidget();

/* Function connected to OK button clicked */
poiDialog.prototype.ok = function() {
	// user must define lat and lng text
	if(this.latLineEdit.text === "" ||
	   this.lngLineEdit.text === "") {
		return;
	}
	// Copy user input to parent parameters
	this.parent.poiName = this.nameLineEdit.text;
	this.parent.poiLat = this.latLineEdit.text;
	this.parent.poiLng = this.lngLineEdit.text;
	this.hide();
};


function createIconReplyCallback(buttonName) {
	return new Function("reply",
		"var data = reply.readAll();"+
		"var pixmap = new QPixmap();"+
		"pixmap.loadFromData(data,0,Qt.AutoColor);"+
		"this."+buttonName+".icon = new QIcon(pixmap);"
	);
}

/* Main application widget */
function QtMapperPersevere(parent) {
	QWidget.call(this, parent);
	
	// Application is disabled until webview is fully loaded
	this.setDisabled(true);
	
	// Google Maps API key
	// Get your own API key from Google: http://code.google.com/intl/fi-FI/apis/maps/signup.html
	this.MY_API_KEY = "ABQIAAAAjfRjTDOGscFuDUwMxJYaQRQUpi_xkJDYa0W0Jatk4awxKuvp7BQBfOyxYXrIsE8mc8pdYrHlaNjZjQ";
	this.MAGIC_NUMBER = "24022009";
	
	// Some parameters to use in multiple functions
	this.poiLat = undefined;
	this.poiLng = undefined;
	this.poiName = undefined;
	this.poiId = undefined;
	this.point = undefined;
	
	// Item selected in poiTree
	this.selectedItem = undefined;
	
	
	this.treewidgetSem = new QSemaphore(1);
	
	
	// Many icons loaded from the server
	// Very ugly...
	var accessManager = new QNetworkAccessManager(this);
	accessManager.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/marker.png")));
    accessManager.finished.connect(this, createIconReplyCallback("addButton"));
	
	var accessManager1 = new QNetworkAccessManager(this);
	accessManager1.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/removeMarker.png")));
    accessManager1.finished.connect(this, createIconReplyCallback("removeButton"));
	
	var accessManager2 = new QNetworkAccessManager(this);
	accessManager2.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/directions.png")));
    accessManager2.finished.connect(this, createIconReplyCallback("directionsButton"));
	
	var accessManager3 = new QNetworkAccessManager(this);
	accessManager3.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/questionmark.png")));
    accessManager3.finished.connect(this, createIconReplyCallback("reverseGeocodeButton"));

	var accessManager4 = new QNetworkAccessManager(this);
	accessManager4.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/zoomin.png")));
    accessManager4.finished.connect(this, createIconReplyCallback("zoomInButton"));
	
	var accessManager5 = new QNetworkAccessManager(this);
	accessManager5.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/zoomout.png")));
    accessManager5.finished.connect(this, createIconReplyCallback("zoomOutButton"));
	
	var accessManager6 = new QNetworkAccessManager(this);
	accessManager6.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/panoramio.png")));
    accessManager6.finished.connect(this, createIconReplyCallback("showPanoramioImagesButton"));
	
	var accessManager7 = new QNetworkAccessManager(this);
	accessManager7.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/wikipedia-W.png")));
    accessManager7.finished.connect(this, createIconReplyCallback("showWikipediaButton"));
	
	
	// QWebView shows a web page with full screen Google Map
	this.view = new QWebView(this);
	
	// lineEdit and searchButton are in the upper row
	this.lineEdit = new QLineEdit(this);
	this.searchButton = new QPushButton(tr("Search"), this);
	
	// Buttons in the upper row
	this.addButton = new QPushButton(this);
	this.addButton.checkable = true;
	this.removeButton = new QPushButton(this);
	this.removeButton.checkable = true;
	this.directionsButton = new QPushButton(this);
	this.directionsButton.checkable = true;
	this.reverseGeocodeButton = new QPushButton(this);
	this.reverseGeocodeButton.checkable = true;
	this.zoomInButton = new QPushButton(this);
	this.zoomInButton.checkable = true;
	this.zoomOutButton = new QPushButton(this);
	this.zoomOutButton.checkable = true;
	
	// These buttons manipulate the poiTreeWidget
	this.addNewPoiButton = new QPushButton(tr("Add POI"), this);
	this.updatePoiButton = new QPushButton(tr("Update"), this);
	this.removePoiButton = new QPushButton(tr("Remove POI"), this);
	this.editPoiButton = new QPushButton(tr("Edit POI"), this);
//	this.saveButton = new QPushButton(tr("Save..."), this);
//	this.loadButton = new QPushButton(tr("Load..."), this);
	this.removePoiButton.enabled = false;
	this.editPoiButton.enabled = false;
	
	// These buttons show some Google Maps readily available overlays
	this.showPanoramioImagesButton = new QPushButton(tr(""),this);
	this.showPanoramioImagesButton.checkable = true;
	this.showWikipediaButton = new QPushButton(tr(""),this);
	this.showWikipediaButton.checkable = true;
	
	this.wikipediaComboBox = new QComboBox();
    this.wikipediaComboBox.addItem(tr("English"));
    this.wikipediaComboBox.addItem(tr("Finnish"));
    this.wikipediaComboBox.addItem(tr("German"));
    this.wikipediaComboBox.addItem(tr("Swedish"));
    this.wikipediaComboBox["activated(int)"].connect(this, this.wikipediaComboBoxChanged);
	
	// Use timer to check where user clicked in reverse geocoding
	this.userClickedTimer = new QTimer(this);
	this.userClickedTimer.timeout.connect(this, this.userClickedTimerTimeout);
	
	// Remember what we have clicked last
	this.lastButton = this.searchButton;

	// upper row button connections
	this.searchButton.clicked.connect(this, this.searchAddress);
	this.addButton.toggled.connect(this, this.toggleAddButton);
	this.removeButton.toggled.connect(this, this.toggleRemoveButton);
	this.directionsButton.toggled.connect(this, this.toggleDirectionsButton);
	this.reverseGeocodeButton.toggled.connect(this, this.toggleReverseGeocodeButton);
	this.zoomInButton.toggled.connect(this, this.toggleZoomInButton);
	this.zoomOutButton.toggled.connect(this, this.toggleZoomOutButton);
	this.lineEdit.returnPressed.connect(this, this.searchAddress);
	
	// let's do some initialization after view has loaded its web page
	this.view["loadFinished(bool)"].connect(this, this.loadFinished);
	
	// poiTree button connections
	this.addNewPoiButton.clicked.connect(this, this.addPoi);
	this.removePoiButton.clicked.connect(this, this.removePoi);
	this.editPoiButton.clicked.connect(this, this.editPoi);
	this.updatePoiButton.clicked.connect(this, this.updatePoiTree);
	//this.saveButton.clicked.connect(this, this.saveTree);
	//this.loadButton.clicked.connect(this, this.loadTree);
	
	// Google Overlay button connections
	this.showPanoramioImagesButton.toggled.connect(this, this.showPanoramio);
	this.showWikipediaButton.toggled.connect(this, this.showWikipedia);
	
	// Initialization of poiTreeWidget
	this.poiTree = new QTreeWidget(this);
	var headerLabels = [];
	headerLabels.push(tr("Point of interest"));
	headerLabels.push(tr("Latitude"));
	headerLabels.push(tr("Longitude"));
	headerLabels.push(tr("id"));
    this.poiTree.setHeaderLabels(headerLabels);
    this.poiTree.header().setResizeMode(QHeaderView.Stretch);
    this.poiTree["itemSelectionChanged ()"].connect(
        this, this.itemSelectionChanged);
	this.poiTree["itemDoubleClicked(QTreeWidgetItem*, int)"].connect(
        this, this.itemDoubleClicked);
	
	// Complex layout definitions
	this.layout = new QVBoxLayout(this);
	this.hboxLayout = new QHBoxLayout();
	this.splitter = new QSplitter(Qt.Vertical, this);
	this.vSplitter = new QSplitter(this);
	this.hViewSplitter = new QSplitter(this);
	
	var poiButtonBox = new QWidget();
	var poiButtonBoxLayout = new QVBoxLayout(poiButtonBox);
	poiButtonBoxLayout.addWidget(this.updatePoiButton,0,0);
	poiButtonBoxLayout.addWidget(this.addNewPoiButton,0,0);
	poiButtonBoxLayout.addWidget(this.editPoiButton,0,0);
	poiButtonBoxLayout.addWidget(this.removePoiButton,0,0);
	//poiButtonBoxLayout.addWidget(this.saveButton,0,0);
	//poiButtonBoxLayout.addWidget(this.loadButton,0,0);
	poiButtonBoxLayout.addStretch();
	
	var overlayButtonBox = new QWidget();
	var overlayButtonBoxLayout = new QVBoxLayout(overlayButtonBox);
	overlayButtonBoxLayout.addWidget(this.showPanoramioImagesButton,0,0);
	overlayButtonBoxLayout.addWidget(this.showWikipediaButton,0,0);
	overlayButtonBoxLayout.addWidget(this.wikipediaComboBox,0,0);
	overlayButtonBoxLayout.addStretch();
	
	this.hboxLayout.addWidget(new QLabel(tr("Location:")), 0, 0);
	this.hboxLayout.addWidget(this.lineEdit, 0, 0);
	this.hboxLayout.addWidget(this.searchButton, 0, 0);
	this.hboxLayout.addWidget(this.addButton, 0, 0);
	this.hboxLayout.addWidget(this.removeButton, 0, 0);
	this.hboxLayout.addWidget(this.directionsButton,0,0);
	this.hboxLayout.addWidget(this.reverseGeocodeButton,0,0);
	this.hboxLayout.addWidget(this.zoomInButton, 0, 0);
	this.hboxLayout.addWidget(this.zoomOutButton, 0, 0);
	
	this.hViewSplitter.addWidget(this.view);
	this.hViewSplitter.addWidget(overlayButtonBox);
	// Do not hide panels at the startup
	//this.hViewSplitter.setSizes(new Array(1,0));
	// this.hViewSplitter.opaqueResize = false; // Does not work properly on Windows
	
	this.vSplitter.addWidget(this.poiTree);
	this.vSplitter.addWidget(poiButtonBox);
	// this.vSplitter.opaqueResize = false; // Does not work properly on Windows
	
	this.splitter.addWidget(this.hViewSplitter);
	this.splitter.addWidget(this.vSplitter);
	// Do not hide panels at the startup
	//this.splitter.setSizes(new Array(1,0));
	// this.splitter.opaqueResize = false; // Does not work properly on Windows
	
	this.layout.addLayout(this.hboxLayout);
	this.layout.addWidget(this.splitter, 0, 0);
	
	// Try to load a web page with a Google Map
	try {
		this.view.load(new QUrl("http://www.cs.tut.fi/~salmin39/map/index.html"));
	} catch(err) {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + err;
		messageBox.show();
	}
	
	this.windowTitle = tr("QtMapperPersevere");
	this.resize(800,480);
}

QtMapperPersevere.prototype = new QWidget();

/* Set map center and zoom */
QtMapperPersevere.prototype.setCenter = function(lat,lng,zoom) {
	var scriptString = 
	"map.setCenter(new GLatLng(" + lat + "," + lng + ")," + zoom + ");";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

/* Insert new marker to the map with infoHtml. Marker title and infoHtml are the same */
QtMapperPersevere.prototype.insertMarker = function(lat,lng,name,id) {
/*	var scriptString = 
	"var newMarker = new GMarker(new GLatLng(" + lat + 
						"," + lng + "),{ title: \""+name+"\" });" +
	"newMarker.bindInfoWindowHtml(newMarker.getTitle().toString());" +
	"newMarker.latitude = " + lat + ";" +
	"newMarker.longitude = " + lng + ";" +
	"newMarker.persevereId = " + id + ";" +
	"newMarker.damnName = " + name + ";" +
	"markers.push(newMarker);" +
	"map.addOverlay(markers[markers.length-1]);";*/
	var scriptString = "createMarker("+lat+","+lng+",\""+name+"\","+id+");";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

/* Zoom in map */
QtMapperPersevere.prototype.zoomIn = function() {
	var scriptString = 
	"map.zoomIn();";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

/* Zoom out map */
QtMapperPersevere.prototype.zoomOut = function() {
	var scriptString = 
	"map.zoomOut();";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

/* Remove all markers */
QtMapperPersevere.prototype.removeAllMarkers = function() {
	var scriptString = 
	//"GLog.write(\"markers.length \" + markers.length);" +
	"while(markers.length > 0) {" +
		"map.removeOverlay(markers.pop());" +
		//"GLog.write(\"removed\" + i);" +
	"}";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};


/* When user click button, Show/hide panoramio overlay */
QtMapperPersevere.prototype.showPanoramio = function() {
	var scriptString = undefined;
	if(this.showPanoramioImagesButton.checked === true) {
		scriptString =
		"var myPanoramioLayer = new GLayer(\"com.panoramio.all\");" +
    	"map.addOverlay(myPanoramioLayer);";
	}
	else {
		scriptString = "map.removeOverlay(myPanoramioLayer);";
	}
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

/* When user clicks button show wikipedia overlay */
QtMapperPersevere.prototype.showWikipedia = function() {
	var scriptString = undefined;
	if(this.showWikipediaButton.checked === true) {
		scriptString = 
    	"map.addOverlay(myWikipediaLayer);";
	}
	else {
		scriptString = 
    	"map.removeOverlay(myWikipediaLayer);";
	}
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

/* User can change wikipedia overlay language 
 * if Wikipedia overlay is toggled "on" 
 * replace old overlay with new one */
QtMapperPersevere.prototype.wikipediaComboBoxChanged = function(index) {
	var wikipediaLanguage = undefined;
    switch (index) {
        // English
        case 0:
            wikipediaLanguage = "en";
            break;
        // Finnish
        case 1:
            wikipediaLanguage = "fi";
            break;
        // Deutch
        case 2:
            wikipediaLanguage = "de";
            break;
        // Sverige
        case 3:
            wikipediaLanguage = "sv";
            break;
	}
	var scriptString = undefined;
	scriptString = 
		"if(myWikipediaLayer !== undefined) {" +
			"map.removeOverlay(myWikipediaLayer);" +
			"myWikipediaLayer = null;" +
		"}" +
		"var myWikipediaLayer = undefined;" +
		"myWikipediaLayer = new GLayer(\"org.wikipedia." +
		wikipediaLanguage + "\");";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
	
	// If overlay should be visible, show it
	if(this.showWikipediaButton.checked === true) {
		scriptString = 
    	"map.addOverlay(myWikipediaLayer);";
	}
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

// When userClickedTimer timeouts do some reverse geocoding
QtMapperPersevere.prototype.userClickedTimerTimeout = function() {
	var tempPoint = this.getUserLastClickedPoint();
	if(tempPoint !== this.point) {
		this.reverseGeocode();
		this.point = tempPoint;
	}
};

/* Update poiTree when user clicks update button */
QtMapperPersevere.prototype.updatePoiTree = function() {
	var marker = undefined;
	var poiItem = undefined;
	
	// empty poiTree completely
	this.poiTree.clear();
	this.removeAllMarkers();

	// load existing stuff from persevere and add it to the tree
	this.loadTree();

	this.poiTree.setDisabled(true);
	while(!this.poiTree.enabled);
	
	// Go trough all the markers found inside webView component javascript engine
	for(var i=0;i<this.getMarkersLength();i=i+1) {
		// Get a marker from scriptEngine inside webView component
		marker = this.getMarkerWithIndex(i);
		
		//debugger;
		print(i);
		
		// Read marker coordinates and name to local variables
		this.poiLat = marker.latitude; //marker.ka.y; // 'ja' used to be 'ka'
		this.poiLng = marker.longitude; //marker.ka.x;
		this.poiId = marker.persevereId;
		this.poiName = marker.nameStr; //this.getMarkerTitleWithIndex(i);
		
		// if marker is already in the tree dont do anything
		
		if(this.poiTree.findItems(this.poiLat.toString(),Qt.MatchFixedString,1).length !== 0 &&
			this.poiTree.findItems(this.poiLng.toString(),Qt.MatchFixedString,2).length !== 0) {
			print(i + " was there");
			//debugger;
			continue;
		}
		else {
			print(i + " was not found, add it");
		
			//debugger;
			// marker was not in the poiTree let's add it
			poiItem = new QTreeWidgetItem();
			poiItem.setText(0, this.poiName);
			poiItem.setText(1, this.poiLat);
			poiItem.setText(2, this.poiLng);
			poiItem.setText(3, this.poiId);
			
			// add marker to treewidget
			this.poiTree.addTopLevelItem(poiItem);
			
			// add marker to the map
			this.insertMarker(this.poiName, this.poiLat, this.poiLng, "A new marker");
			
			// add marker to persevere
			this.saveTreeItem(this.poiTree.indexOfTopLevelItem(poiItem));
		}	
		
	}
};

/* After the web page has been fully loaded we can do some
 * initialization and other stuff. If this is done too soon it will
 * not work at all */
QtMapperPersevere.prototype.loadFinished = function(ok) {	
	var scriptString = 
	"var userClickedPoint = undefined;" +
	"var userClickedMarker = undefined;" +
	"var myClickListener = undefined;" +
	"var fromMarker = undefined;" +
	"var toMarker = undefined;" +
	"map.addControl(new GMapTypeControl());" +
	"map.setCenter( new GLatLng(0,0),1 );" +
	"var markers = new Array();" +
	//"var mm = new GMarkerManager(map, {borderPadding:1});" +
	"map.disableDoubleClickZoom();" +
	"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
    	"userClickedPoint = point;" +
    	"userClickedMarker = marker;" +
	"});" +
	"GEvent.addListener(map, \"moveend\", function() {" +
  		"userClickedPoint = undefined;" +
  		"userClickedMarker = undefined;" +
	"});" +
	"function createMarker(lat,lng,nameStr,id) {" +
		"var marker = new GMarker(new GLatLng(lat,lng));"+
		"marker.nameStr = nameStr;" +
		"marker.persevereId = id;" +
		"marker.latitude = lat;" +
		"marker.longitude = lng;" +
		//"map.openInfoWindow(map.getCenter(), nameStr);"+
		"markers.push(marker);" +
		"map.addOverlay(markers[markers.length-1]);"+
	"}";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
	
	// Enable whole application
	this.setEnabled(true);
	
	// Set wikipediaComboBox state. This runs some javascript code
	// inside the webView component. That's why this must be done
	// after web page has been fully loaded
	this.wikipediaComboBox.setCurrentIndex(0);
    this.wikipediaComboBox.activated(0);
	
	// Load a file with some POI information
	this.loadTree("QtMapper_AutoOpenPOIList.csv");
};

// Function to toggle Add button
QtMapperPersevere.prototype.toggleAddButton = function() {
	this.lastButton.checked = false;
	var scriptString = undefined;
	if(this.addButton.checked === true) {
		scriptString = 
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
    		/*"var length = markers.push(new GMarker(point));" +
			"map.addOverlay(markers[length-1]);" +*/
			"createMarker(point.lat(),point.lng(),\"No name yet\");" +
		"});";
		this.lastButton = this.addButton;
	}
	else {
		scriptString =
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
		"});";
		this.lastButton = this.searchButton;
	}
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

QtMapperPersevere.prototype.toggleRemoveButton = function() {	
	this.lastButton.checked = false;
	var scriptString = undefined;
	if(this.removeButton.checked === true) {
		scriptString = 
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
	    	"userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
    		"if(marker) {" +
    			"map.removeOverlay(marker);" +
    			"for(i=0; i<=markers.length-1; i=i+1) {" +
    				"if(markers[i].getPoint().lat() === marker.getPoint().lat() &&" +
    					"markers[i].getPoint().lng() === marker.getPoint().lng()) {" +
    					"markers.splice(i,1);" +
    				"}" +
				"}" +
    		"}" +
		"});";
		this.lastButton = this.removeButton;
	}
	else {
		scriptString =
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
		"});";
		this.lastButton = this.searchButton;
	}
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

QtMapperPersevere.prototype.toggleDirectionsButton = function() {
	this.lastButton.checked = false;
	var scriptString = undefined;
	if(this.directionsButton.checked === true) {
		scriptString = 
		"var directions = undefined;" +
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
		    "if(fromMarker === undefined && toMarker === undefined) {" +
		    	"fromMarker = new GMarker(point);" +
				"map.addOverlay(fromMarker);" + 
			"}" +
			"else if (toMarker === undefined){" +
				"toMarker = new GMarker(point);" +
				"var myWaypoints = [];" +
				"myWaypoints.push(fromMarker.getPoint().lat() + \", \" + fromMarker.getPoint().lng());" +
				"myWaypoints.push(toMarker.getPoint().lat() + \", \" + toMarker.getPoint().lng());" +
				"directions = new GDirections(map);" +
				"GEvent.addListener(directions, \"load\", onGDirectionsLoad);" +
				"directions.loadFromWaypoints(myWaypoints, {'getPolyline':true});"+
				"map.removeOverlay(fromMarker);" +
				"toMarker = undefined;" +
				"fromMarker = undefined;" +
			"}" +
			"else {" +
			"}"+
		"});" +
		"function onGDirectionsLoad(){ " +
			"var meters = directions.getDistance().meters / 1000;" +
			"var duration = directions.getDuration().html;" +
			"var myHtml = meters + \" km<br/> (Driving time \" + duration + \")\";" +
			"map.openInfoWindow(userClickedPoint, myHtml);" +
			"directions = undefined;" +
		"}";
		this.lastButton = this.directionsButton;
	}
	else if(this.directionsButton.checked === false)  {
		scriptString =
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
		"});";
		this.lastButton = this.searchButton;
	}

	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};


QtMapperPersevere.prototype.toggleReverseGeocodeButton = function() {
	this.lastButton.checked = false;
	if(this.reverseGeocodeButton.checked === true) {
		this.lastButton = this.reverseGeocodeButton;
		this.lineEdit.readOnly = true;
		this.searchButton.enabled = false;
		this.userClickedTimer.start(1000);
	}
	else if(this.reverseGeocodeButton.checked === false) {
		this.lastButton = this.searchButton;
		this.lineEdit.readOnly = false;
		this.searchButton.enabled = true;
		this.userClickedTimer.stop();
	}
};

QtMapperPersevere.prototype.toggleZoomInButton = function() {
	this.lastButton.checked = false;
	var scriptString = undefined;
	if(this.zoomInButton.checked === true) {
		scriptString = 
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
			"map.zoomIn();" + 
			"map.setCenter(point);" +
		"});";
		this.lastButton = this.zoomInButton;
	}
	else if(this.zoomInButton.checked === false)  {
		scriptString =
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
		"});";
		this.lastButton = this.searchButton;
	}

	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

QtMapperPersevere.prototype.toggleZoomOutButton = function() {
	this.lastButton.checked = false;
	var scriptString = undefined;
	if(this.zoomOutButton.checked === true) {
		scriptString = 
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
			"map.zoomOut();" + 
			"map.setCenter(point);" +
		"});";
		this.lastButton = this.zoomOutButton;
	}
	else if(this.zoomOutButton.checked === false)  {
		scriptString =
		"GEvent.removeListener(myClickListener);" +
		"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
		    "userClickedPoint = point;" +
    		"userClickedMarker = marker;" +
		"});";
		this.lastButton = this.searchButton;
	}
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

QtMapperPersevere.prototype.getMapCenter = function() {
	return this.view.page().mainFrame().evaluateJavaScript("map.getCenter()");
};

QtMapperPersevere.prototype.getUserLastClickedPoint = function() {
	return this.view.page().mainFrame().evaluateJavaScript("userClickedPoint;");
};

QtMapperPersevere.prototype.getUserLastClickedMarker = function() {
	return this.view.page().mainFrame().evaluateJavaScript("userClickedMarker;");
};

QtMapperPersevere.prototype.getMarkerWithIndex = function(index) {
	return this.view.page().mainFrame().evaluateJavaScript("markers[" + index + "];");
};

QtMapperPersevere.prototype.getMarkerTitleWithIndex = function(index) {
	return this.view.page().mainFrame().evaluateJavaScript("markers[" + index + "].getTitle().toString();");
};

QtMapperPersevere.prototype.getMarkersLength = function() {
	return this.view.page().mainFrame().evaluateJavaScript("markers.length;");
};

QtMapperPersevere.prototype.removeUserLastClickedMarker = function() {
	this.view.page().mainFrame().evaluateJavaScript("map.removeOverlay(userClickedMarker);");
	this.view.page().mainFrame().evaluateJavaScript("userLastClickedMarker = undefined;");
};

QtMapperPersevere.prototype.geoCode = function(locationString, callback) {
	requestStr = "http://maps.google.com/maps/geo?q=" +
		locationString + "&output=csv&key=" + this.MY_API_KEY;
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, callback);
	accessManager.get(new QNetworkRequest(new QUrl(requestStr)));
};

QtMapperPersevere.prototype.reverseGeocode = function() {
	var point = undefined;
	point = this.getUserLastClickedPoint();
	if(point === undefined) {
		point = this.getMapCenter();
	}
	this.geoCode(point.y.toString() +","+ point.x.toString(), reverseGeocodeCallback);
};

reverseGeocodeCallback = function(reply) {
	var replyString = reply.readAll().toString();
	var geoCodeArray = [];
	geoCodeArray = replyString.split(",");
	var htmlResponse = geoCodeArray[0];
	var accuracy = geoCodeArray[1];
	var address = geoCodeArray[2];
	for(i=3;i<geoCodeArray.length;i++) {
		address += geoCodeArray[i];
	}
	if(htmlResponse === "200" && this.lineEdit.text !== address.substring(1,address.length-1)) {
		this.lineEdit.text = address.substring(1,address.length-1);
	}
};

QtMapperPersevere.prototype.geocodeAddress = function() {
	this.geoCode(this.lineEdit.text, geocodeAddressCallback);
};

geocodeAddressCallback = function(reply) {
	var replyString = reply.readAll().toString();
	var geoCodeArray = [];
	geoCodeArray = replyString.split(",");
	var htmlResponse = geoCodeArray[0];
	var accuracy = geoCodeArray[1];
	var lat = geoCodeArray[2];
	var lng = geoCodeArray[3];
	if(htmlResponse === "200") {
		this.insertMarker(lat, lng,	this.lineEdit.text + "<br>Accuracy: " + accuracy);
		this.setCenter(lat, lng, parseInt(accuracy,10) + 6);
	}
};

QtMapperPersevere.prototype.searchAddress = function() {
	this.geoCode(this.lineEdit.text, searchAddressCallback);
};

searchAddressCallback = function(reply) {
	var replyString = reply.readAll().toString();
	var geoCodeArray = [];
	geoCodeArray = replyString.split(",");
	var htmlResponse = geoCodeArray[0];
	var accuracy = geoCodeArray[1];
	var lat = geoCodeArray[2];
	var lng = geoCodeArray[3];
	if(htmlResponse === "200") {
		this.setCenter(lat, lng, parseInt(accuracy,10) + 6);
	}
};

QtMapperPersevere.prototype.addPoi = function() {
	var point = undefined;
	point = this.getUserLastClickedPoint();
	if(point === undefined) {
		point = this.getMapCenter();
	}
	var poi = new poiDialog(this, point.y.toString(), point.x.toString(), this.lineEdit.text);
	poi.show();
	this.setDisabled(true);
	while(poi.visible === true);
	this.setEnabled(true);
	
	if(this.poiName !== undefined) {
		var poiItem = new QTreeWidgetItem();
		poiItem.setText(0, this.poiName);
		poiItem.setText(1, this.poiLat);
		poiItem.setText(2, this.poiLng);
		this.poiTree.addTopLevelItem(poiItem);
		// add marker to map
		this.insertMarker(poiItem.text(1),poiItem.text(2),poiItem.text(0),poiItem.text(3));
		
		if(poiItem.text(3).length === 0 || poiItem.text(3) === "undefined") {
			// add marker to persevere
			this.saveTreeItem(this.poiTree.indexOfTopLevelItem(poiItem));
		}
	}
};

QtMapperPersevere.prototype.editPoi = function() {
	var poi = new poiDialog(this, this.selectedItem[0].text(1), 
			this.selectedItem[0].text(2), this.selectedItem[0].text(0));
	poi.show();
	this.setDisabled(true);
	while(poi.visible === true);
	this.setEnabled(true);
	this.removePoi();
	if(this.poiName !== undefined) {
		var poiItem = new QTreeWidgetItem();
		poiItem.setText(0, this.poiName);
		poiItem.setText(1, this.poiLat);
		poiItem.setText(2, this.poiLng);
		poiItem.setText(3, this.poiId);
		this.poiTree.addTopLevelItem(poiItem);
		
		this.insertMarker(poiItem.text(1),poiItem.text(2),poiItem.text(0),poiItem.text(3));
	}
};

QtMapperPersevere.prototype.removePoi = function() {
	print("removePoi");
	var scriptString =
	    "for(i=0; i<=markers.length-1; i=i+1) {" +
    		"if(markers[i].getPoint().lat() === " + this.selectedItem[0].text(1) + "&&" +
    			"markers[i].getPoint().lng() === " + this.selectedItem[0].text(2) + "){" +
    			"map.removeOverlay(markers[i]);" +
    			"markers.splice(i,1);" +
    		"}" +
		"}";
	// remove item from persevere
	if(this.selectedItem[0].text(3).length != 0 || this.selectedItem[0].text(3) != "undefined") {
		this.removeTreeItem(this.selectedItem[0].text(3));
	}
	
	// remove item from treewidget
	this.poiTree.takeTopLevelItem(
		this.poiTree.indexOfTopLevelItem(this.selectedItem[0]));
	
	// remove item from the map
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

QtMapperPersevere.prototype.itemSelectionChanged = function()
{
	this.selectedItem = this.poiTree.selectedItems();
	if(this.selectedItem[0] !== null) {
		this.removePoiButton.enabled = true;
		this.editPoiButton.enabled = true;
	} else {
		this.removePoiButton.enabled = false;
		this.editPoiButton.enabled = false;
	}
};

QtMapperPersevere.prototype.itemDoubleClicked = function(item)
{
	this.setCenter(item.text(1),item.text(2),"12");
};


QtMapperPersevere.prototype.closeEvent = function (event) {
//	this.saveTree("QtMapper_AutoOpenPOIList.csv");
};

QtMapperPersevere.prototype.saveTree = function()
{
/*	for(var i = 0; i < this.poiTree.topLevelItemCount; i= i+1) {
		this.saveTreeItem(i);
	}
*/
};

QtMapperPersevere.prototype.removeTreeItem = function(item)
{
	var baseStr = 'http://lively.cs.tut.fi:8010/QtMapperPois/' + item;
	print(baseStr);

	var req = new QNetworkRequest(new QUrl(baseStr));
	
	// QNetworkAccessManager for network requests
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.treeItemRemoved);
	// Send the HTTP/PUT request to update the data in the db
	try {
		accessManager.deleteResource(req);
	}
	catch(err){
		// does not work if not running on Qt 4.6
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + err + "\nThis feature requires Qt 4.6";
		messageBox.show();
	}
};

QtMapperPersevere.prototype.treeItemRemoved = function(reply) {
	var data = reply.readAll().toString();
	var status = reply.attribute(QNetworkRequest.HttpStatusCodeAttribute);
	print("removeStatus: " + status);
	print(data);
};


QtMapperPersevere.prototype.editTreeItem = function(item)
{
	var treeItem = this.poiTree.topLevelItem(item);
	var baseStr = 'http://lively.cs.tut.fi:8010/QtMapperPois/' + treeItem.text(3);
	print(baseStr);
	var jsonStr = '{"name":"'+treeItem.text(0) +'","lat":' +treeItem.text(1) +',"lng":' +treeItem.text(2)+'}';
	print(jsonStr);
	var jsonArr = new QByteArray(jsonStr);
	var req = new QNetworkRequest(new QUrl(baseStr));
	
	// QNetworkAccessManager for network requests
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.treeItemEdited);
	// Send the HTTP/PUT request to update the data in the db
	accessManager.put(req, jsonArr);
};

QtMapperPersevere.prototype.treeItemEdited = function(reply) {
	var data = reply.readAll().toString();
	print(data);
};

QtMapperPersevere.prototype.saveTreeItem = function(item)
{
	var baseStr = 'http://lively.cs.tut.fi:8010/QtMapperPois/';
	print(baseStr);
	var treeItem = this.poiTree.topLevelItem(item);
	var jsonStr = '{"name":"'+treeItem.text(0) +'","lat":' +treeItem.text(1) +',"lng":' +treeItem.text(2)+'}';
	print(jsonStr);
	var jsonArr = new QByteArray(jsonStr);
	var req = new QNetworkRequest(new QUrl(baseStr));
	
	// QNetworkAccessManager for network requests
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.treeItemSaved);
	// Send the HTTP/PUT request to update the data in the db
	accessManager.post(req, jsonArr);

};

QtMapperPersevere.prototype.treeItemSaved = function(reply) {
	var data = reply.readAll().toString();
	print(data);
};

QtMapperPersevere.prototype.loadTree = function(fileToUse)
{
	// URL to fetch
	var urlStr = 'http://lively.cs.tut.fi:8010/QtMapperPois/'; /* ?id=' + '0';*/
	// QNetworkAccessManager for network requests
	var accessManager = new QNetworkAccessManager(this);
	// When the request has finished, proceed to the callback function
	accessManager["finished(QNetworkReply*)"].connect(this, this.treeLoaded);
	// Send an HTTP/GET request
	accessManager.get(new QNetworkRequest((new QUrl(urlStr))));
};

QtMapperPersevere.prototype.treeLoaded = function(reply) {
	if(reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) === 404) {
		var poiItem = new QTreeWidgetItem();
		poiItem.setText(0, "No items at DB");
		this.poiTree.addTopLevelItem(poiItem);
		this.poiTree.setEnabled(true);
		return;
	}
	var scoresJSONString = reply.readAll().toString();
	var scoresJSONObject = eval('(' + scoresJSONString + ')'); 
	print(scoresJSONString);
	// If there is an error in the JSON or there are no scores in the db
	if(scoresJSONObject.error || scoresJSONObject.length == 0) {
		var poiItem = new QTreeWidgetItem();
		poiItem.setText(0, "No items at DB");
		this.poiTree.addTopLevelItem(poiItem);
		this.poiTree.setEnabled(true);
		return;
	}
	else {
		this.treewidgetSem.acquire(1);
		for(var i = 0; i < scoresJSONObject.length; i = i +1) {
			var poiItem = new QTreeWidgetItem();
			poiItem.setText(0, scoresJSONObject[i].name);
			poiItem.setText(1, scoresJSONObject[i].lat);
			poiItem.setText(2, scoresJSONObject[i].lng);
			poiItem.setText(3, scoresJSONObject[i].id);
			this.poiTree.addTopLevelItem(poiItem);
			this.insertMarker(poiItem.text(1),poiItem.text(2),poiItem.text(0),poiItem.text(3));
		}
		this.treewidgetSem.release(1);
		this.poiTree.setEnabled(true);
	}
}

QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};


/* To use the application in standalone mode, remove the comment around this code*/

/*
var map = new QtMapperPersevere();
map.show();

QCoreApplication.exec();
*/


