/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

/*
	N810 Internal light/temperature sensor information
*/

function QtN810Info(parent)
{
    QWidget.call(this, parent);

	this.updateTimer = new QTimer(this);
	this.updateTimer.timeout.connect(this, this.updateData);
	this.updateTimer.start(5000);

	/* following lines define the user interface */	
	/*
	this.goBtn = new QPushButton("Reset Dials",this);
	this.goBtn.clicked.connect(this, function() {
		this.tempDial.maximum = 40;
		this.tempDial.minimum = 30; 
		this.tempDial.value = this.temp;
		this.luxDial.maximum = 300;
		this.luxDial.minimum = 0;
		this.luxDial.value = this.lux;
	});

	this.tempDial = new QDial(this);
	this.tempDial.maximum = 20;
	this.tempDial.minimum = 10;
	this.tempDial.notchesVisible = true;
	this.tempDial.value = 15;
	this.tempDial.enabled = false;

	this.tempLbl = new QLabel("Temperature is read in one second intervals", this);
	
	this.luxDial = new QDial(this);
	this.luxDial.maximum = 300;
	this.luxDial.minimum = 0;
	this.luxDial.notchesVisible = true;
	this.luxDial.value = 15;
	this.luxDial.enabled = false;
	
	this.luxLbl = new QLabel("", this);
	this.luxLbl.text = "Lux is read in one second intervals";
	
	this.layout = new QVBoxLayout(this);
	this.layout.addWidget(this.tempDial,0,Qt.AlignHCenter);
	this.layout.addWidget(this.tempLbl,0,Qt.AlignHCenter);
	this.layout.addWidget(this.luxDial,0,Qt.AlignHCenter);
	this.layout.addWidget(this.luxLbl,0,Qt.AlignHCenter);
	this.layout.addWidget(this.goBtn,0,0);
	*/
	
	this.temp = undefined;
	this.lux = undefined;
}

QtN810Info.prototype = new QWidget();

QtN810Info.prototype.updateData = function()
{
	var temperature = this.readTemp();
	temperature = temperature/1000;
	
	this.temp = temperature;
	
	/*this.tempLbl.text = "<h1>" + temperature + " &deg;C </h1>";
	
	if(this.tempDial.maximum < temperature)
		this.tempDial.maximum = temperature + 1;
	if(this.tempDial.minimum > temperature)
		this.tempDial.minimum = temperature - 1;
	
	this.tempDial.value = temperature;
	*/
	var lux = this.readLux();
	this.lux = lux;
	
	/*this.luxLbl.text = "<h1>" + lux + " Lux</h1>";
	
	if(this.luxDial.maximum < lux)
		this.luxDial.maximum = lux + 1;
	if(this.luxDial.minimum > lux)
		this.luxDial.minimum = lux - 1;
	
	this.luxDial.value = lux;
	*/
};

QtN810Info.prototype.readTemp = function()
{
	var filename = "/sys/bus/i2c/devices/1-0048/temp1_input";

	var file = new QFile(filename);	
	if(file.exists()) {
		file.open(QIODevice.ReadOnly);
		var stream = new QTextStream(file);
		//while(!stream.atEnd()) {
			var line = stream.readLine();
			this.temp = line;
			print("temp: " + this.temp);
		//}
		file.close();
	}
	else {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: Temperature can not be accessed.";
		messageBox.show();
		this.updateTimer.stop();
	}

	if(this.temp != undefined) {
		return this.temp;
	}
};

QtN810Info.prototype.readLux = function()
{
	var filename = "/sys/bus/i2c/devices/0-0029/lux";

	var file = new QFile(filename);	
	if(file.exists()) {
		file.open(QIODevice.ReadOnly);
		var stream = new QTextStream(file);
		//while(!stream.atEnd()) {
			var line = stream.readLine();
			this.lux = line;
			
			print("lux: " + this.lux);
		//}
		file.close();
	}
	else {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: Lux can not be accessed.";
		messageBox.show();
		this.updateTimer.stop();
	}

	if(this.lux != undefined) {
		return this.lux;
	}
};


QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};


/*
var wdgt = new QtN810Info();
wdgt.show();

QCoreApplication.exec();
*/