/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

function tr(s) { return s; }

function QtSimpleCalendar(parent)
{
    QWidget.call(this, parent);
    
    this.centralLayout = new QVBoxLayout(this);
    this.centralWidget = new QWidget();
    this.vBoxLayout = new QVBoxLayout();
    this.controlsLayout = new QHBoxLayout();
    this.events = [];
    this.eventDialog;
    this.topic = "";
    this.place = "";
    this.startTime = "";
    this.endTime = "";
    this.eventsJSONObject;
    this.selectedDate = new Date();

    // Get all the events sorted by date and startTime.
    var url = 'http://lively.cs.tut.fi:8010/SimpleCalendarEvent/[/eventDate, /startTime]';
    var accessManager = new QNetworkAccessManager(this);
    accessManager["finished(QNetworkReply*)"].connect(this, this.getEvents);
    accessManager.get(new QNetworkRequest(new QUrl(url)));

    var months = new Array(12);
    months[0]="January";
    months[1]="February";
    months[2]="March";
    months[3]="April";
    months[4]="May";
    months[5]="June";
    months[6]="July";
    months[7]="August";
    months[8]="September";
    months[9]="October";
    months[10]="November";
    months[11]="December";

    this.monthCombo = new QComboBox;

    for (month = 0; month <= 11; ++month)
    {
        this.monthCombo.addItem(months[month]);
    }
    this.monthCombo.setCurrentIndex(this.selectedDate.getMonth());
    this.monthCombo.maxVisibleItems = 12;

    this.yearEdit = new QDateTimeEdit;
    this.yearEdit.setDateRange(Date(1753, 1, 1), Date(8000, 1, 1));    
    this.yearEdit.setDate(this.selectedDate);
    this.yearEdit.displayFormat = "yyyy";
    
    this.table = new QTableWidget(1,7);

    var horizontalHeaders = new Array(7);
    horizontalHeaders[0]="Monday";
    horizontalHeaders[1]="Tuesday";
    horizontalHeaders[2]="Wednesday";
    horizontalHeaders[3]="Thursday";
    horizontalHeaders[4]="Friday";
    horizontalHeaders[5]="Saturday";
    horizontalHeaders[6]="Sunday";
    this.table.setHorizontalHeaderLabels(horizontalHeaders);
    var header = this.table.horizontalHeader();
    header.setResizeMode(QHeaderView.Stretch);
    header.setClickable(false);

    this.monthCombo["currentIndexChanged(int)"].connect(this, this.selectMonth);
    this.yearEdit["dateChanged(QDate)"].connect(this, this.selectYear);
    this.table["cellActivated(int, int)"].connect(this, this.openDay);

    // Create UI.  
    this.controlsLayout.addWidget(this.monthCombo, 0, 0);
    this.controlsLayout.addWidget(this.yearEdit, 0, 0);
    this.controlsLayout.addStretch(1);

    this.centralLayout.addLayout(this.controlsLayout);
    this.centralLayout.addWidget(this.table, 1, 0);
    this.layout = this.centralLayout;
     
    this.windowTitle = tr("Simple Calendar");
    this.resize(754,300);
}

QtSimpleCalendar.prototype = new QWidget();

QtSimpleCalendar.prototype.getEvents = function(reply) {
    var eventsJSONString = reply.readAll().toString();
    var eventsJSONObject = eval('(' + eventsJSONString + ')'); 
    // If there is an error in the JSON or there are no events in the db, 
    // mark events as zero.
    if(eventsJSONObject.error || eventsJSONObject.length == 0)
    {
        this.events = [];
    }
    else
    {
        // Save all the events.
        this.events = [];
        for (i = 0; i < eventsJSONObject.length; i++)
        {
            this.events[i] = [];
            this.events[i][0] = eventsJSONObject[i].id;
            this.events[i][1] = eventsJSONObject[i].eventDate;
            this.events[i][2] = eventsJSONObject[i].topic;
            this.events[i][3] = eventsJSONObject[i].place;
            this.events[i][4] = eventsJSONObject[i].startTime;
            this.events[i][5] = eventsJSONObject[i].endTime;
        }
    }

    // Refresh the calendar.
    this.insertCalendar(); 
}

QtSimpleCalendar.prototype.selectMonth = function(month) {
    this.selectedDate.setMonth(month);

    // Refresh the calendar.
    this.insertCalendar(); 
}

QtSimpleCalendar.prototype.selectYear = function(newDate) {
    this.selectedDate.setFullYear(newDate.getFullYear());
    
    // Refresh the calendar.
    this.insertCalendar(); 
}

// Functions to calculate week number. Ripped from: http://javascript.about.com/library/blstdweek.htm
Date.prototype.getMDay = function() {
    return (this.getDay() + 6) %7;
}
Date.prototype.getISOYear = function() {
    var thu = new Date(this.getFullYear(), this.getMonth(), this.getDate()+3-this.getMDay());
    return thu.getFullYear();
}
Date.prototype.getISOWeek = function() {
    var onejan = new Date(this.getISOYear(),0,1);
    var wk = Math.ceil((((this - onejan) / 86400000) + onejan.getMDay()+1)/7);
    if (onejan.getMDay() > 3) wk--;return wk;
} 

QtSimpleCalendar.prototype.insertCalendar = function() {

    var tableItem;
    var tableDay;
    var weekDayCounter;
    var header;
    var rowCount = this.table.rowCount;
    var iFlags = new Qt.ItemFlags(Qt.ItemIsEnabled);
    var brush = new QBrush(new QColor("orange"));
    var strongFont = new QFont();
    strongFont.setBold(true);
    strongFont.setUnderline(true);
    var verticalHeaders = new Array();

    var currentDate = new Date();
    var newDate = new Date();
    newDate.setFullYear(this.selectedDate.getFullYear());
    newDate.setMonth(this.selectedDate.getMonth());
    newDate.setDate(1);

    verticalHeaders.push(newDate.getISOWeek());

    // Clear table.
    this.table.clearContents();

    while (this.table.rowCount > 1)
    {
        this.table.removeRow(this.table.rowCount - 1);
    }

    // 0 == sunday, 1 == monday, ... , 6 == saturday. 
    weekDayCounter = newDate.getDay() - 1;
    if (weekDayCounter < 0)
    {
        weekDayCounter = 6;
    }

    // Disable cells before the first day of the month.
    for (i = 0; i < weekDayCounter; i++)
    {
        tableItem = new QTableWidgetItem("");
        tableItem.setFlags(0);
        
        this.table.setItem(this.table.rowCount - 1, i, tableItem);
    }

    while (newDate.getMonth() == this.selectedDate.getMonth())
    {
        tableItem = new QTableWidgetItem((newDate.getDate()).toString());
        tableItem.setFlags(iFlags);
        tableItem.setTextAlignment(4);

        // Mark the current day with bolded text.
        if (newDate.getDate() == currentDate.getDate() && newDate.getMonth() == currentDate.getMonth()
            && newDate.getFullYear() == currentDate.getFullYear())
        {
            tableItem.setFont(strongFont);
        }

        // Check if the date got an event.
        for (j = 0; j < this.events.length; j++)
        {
            if (this.events[j][1] == newDate.getFullYear() + '_' + newDate.getMonth() + '_' + newDate.getDate())
            {
                // Mark that the day got an event.
                tableItem.setBackground(brush);
                continue;
            }
        }

        this.table.setItem(this.table.rowCount - 1, weekDayCounter, tableItem);

        newDate.setDate(newDate.getDate()+1);

        if (weekDayCounter == 6 && newDate.getMonth() == this.selectedDate.getMonth())
        {
            this.table.insertRow(this.table.rowCount);
            weekDayCounter = 0;
            verticalHeaders.push(newDate.getISOWeek());
        }
        else
        {
            weekDayCounter++;
        }
    }

    // Disable cells after the last day of the month.
    for (i = weekDayCounter; i < 7; i++)
    {
        tableItem = new QTableWidgetItem("");
        tableItem.setFlags(0);
        
        this.table.setItem(this.table.rowCount - 1, i, tableItem);
    }

    // Show weeknumbers.
    this.table.setVerticalHeaderLabels(verticalHeaders);
    header = this.table.verticalHeader();
    header.setResizeMode(QHeaderView.ResizeToContents);
    header.setClickable(false);

}

QtSimpleCalendar.prototype.openDay = function(selectedRow, selectedColumn) {

    if (this.table.item(selectedRow, selectedColumn).text() == "")
    {
        return;
    }

    this.selectedDate.setDate(this.table.item(selectedRow, selectedColumn).text());

    this.eDialog = new eventDialog(this, this.selectedDate, this.events);

    // Update events.
    var url = 'http://lively.cs.tut.fi:8010/SimpleCalendarEvent/[/eventDate, /startTime]';
    var accessManager = new QNetworkAccessManager(this);
    accessManager["finished(QNetworkReply*)"].connect(this, this.getEvents);
    accessManager.get(new QNetworkRequest(new QUrl(url)));
    
}

eventDialog.prototype = new QWidget();

/* Event dialog lists all the events for the selected day. */
function eventDialog(parent, eDate, events)
{
    QWidget.call(this, parent);

    this.eDialog = new QDialog(this);
    this.eDialog.resize(320,290);
    this.eDialog.windowTitle = tr(eDate.toDateString());
    this.dialogLayout = new QGridLayout(this.eDialog);

    this.eventDate = eDate;
    this.dbDate = eDate.getFullYear() + '_' + eDate.getMonth() + '_' + eDate.getDate();
    this.dayEvents = [];
    var eventCounter = 0;

    this.eTable = new QTableWidget(0,1);
    var horizontalHeaders = new Array(1);
    horizontalHeaders[0]="Events";
    this.eTable.setHorizontalHeaderLabels(horizontalHeaders);
    var header = this.eTable.horizontalHeader();
    header.setResizeMode(QHeaderView.Stretch);
    header.setClickable(false);

    var header = this.eTable.verticalHeader();
    header.setVisible(false);

    // Find all the events for the day.
    for (j = 0; j < events.length; j++)
    {
        // Check the date of the event.
        if (events[j][1] == this.dbDate)
        {
            // Save the event.
            this.dayEvents[eventCounter] = [];
            this.dayEvents[eventCounter] = events[j];
            eventCounter++;
        }
    }

    // Update table with events for the day.
    this.updateEventsTable();

    this.closeButton = new QPushButton(tr("Close"));
    this.submitButton = new QPushButton(tr("Add Event"));

    this.eTable["cellActivated(int, int)"].connect(this, this.openEvent);
    this.eTable["currentItemChanged(QTableWidgetItem*, QTableWidgetItem* )"].connect(this, this.selectEvent);
    this.closeButton["clicked()"].connect(this, this.closeButtonClicked);
    this.submitButton["clicked()"].connect(this, this.addEventButtonClicked);

    this.dialogLayout.addWidget(this.eTable, 0, 0, 0, -1);
    this.dialogLayout.addWidget(this.closeButton, 1, 0);
    this.dialogLayout.addWidget(this.submitButton, 1, 1);

    this.eDialog.layout = this.dialogLayout;
    this.eDialog.exec();

}

eventDialog.prototype.updateEventsTable = function() {
    var tableItem;
    var iFlags = new Qt.ItemFlags(Qt.ItemIsEnabled);

    // Clear the table.
    this.eTable.clearContents();
    while (this.eTable.rowCount > 0)
    {
        this.eTable.removeRow(this.eTable.rowCount - 1);
    }

    // Go through all the events for the day.
    for (j = 0; j < this.dayEvents.length; j++)
    {
        // Add a new row for the event.
        this.eTable.insertRow(this.eTable.rowCount);

        // Add the event in the table.
        tableItem = new QTableWidgetItem(tr(this.dayEvents[j][4] + "-" + this.dayEvents[j][5] + " " + this.dayEvents[j][2]));
        tableItem.setFlags(iFlags);
        this.eTable.setItem(this.eTable.rowCount - 1, 0, tableItem);
    }

    // If there are no events, display a note.
    if (this.dayEvents.length == 0)
    {
        this.eTable.insertRow(this.eTable.rowCount);
        tableItem = new QTableWidgetItem(tr("No events..."));
        tableItem.setFlags(0);
        this.eTable.setItem(this.eTable.rowCount - 1, 0, tableItem);
    }
}

eventDialog.prototype.selectEvent = function( currentItem, previousItem) {
    var brush_orange = new QBrush(new QColor("orange"));
    var brush_default = new QBrush();

    // Paint the background of the selected item orange.
    if (currentItem != null)
    {
        currentItem.setBackground(brush_orange);
    }

    // Clear the background color from the previous item.
    if (previousItem != null)
    {
        previousItem.setBackground(brush_default);
    }
}

eventDialog.prototype.openEvent = function(selectedRow, selectedColumn) {
    
    // Display a dialog for modifying the selected event.
    this.modifyDialog(this, this.eventDate, this.dayEvents[selectedRow]);

    // Create a query to update the events for the day.
    var url = 'http://lively.cs.tut.fi:8010/SimpleCalendarEvent/[?eventDate="' + this.dbDate + '"][/startTime]';
    var accessManager = new QNetworkAccessManager(this);
    accessManager["finished(QNetworkReply*)"].connect(this, this.updateEvents);
    accessManager.get(new QNetworkRequest(new QUrl(url)));
}

eventDialog.prototype.addEventButtonClicked = function() {

    // Display a dialog for creating a new event.
    this.modifyDialog(this, this.eventDate, "");

    // Create a query to update the events for the day.
    var url = 'http://lively.cs.tut.fi:8010/SimpleCalendarEvent/[?eventDate="' + this.dbDate + '"][/startTime]';
    var accessManager = new QNetworkAccessManager(this);
    accessManager["finished(QNetworkReply*)"].connect(this, this.updateEvents);
    accessManager.get(new QNetworkRequest(new QUrl(url)));
}

eventDialog.prototype.closeButtonClicked = function() {
    this.eDialog.close();
}

eventDialog.prototype.updateEvents = function(reply) {
    var eventsJSONString = reply.readAll().toString();
    var eventsJSONObject = eval('(' + eventsJSONString + ')'); 
    // If there is an error in the JSON or there are no events in the db, 
    // mark events as zero.
    if(eventsJSONObject.error || eventsJSONObject.length == 0)
    {
        this.dayEvents = [];
    }
    else
    {
        // Save all the events for the selected day.
        this.dayEvents = [];
        for (i = 0; i < eventsJSONObject.length; i++)
        {
            this.dayEvents[i] = [];
            this.dayEvents[i][0] = eventsJSONObject[i].id;
            this.dayEvents[i][1] = eventsJSONObject[i].eventDate;
            this.dayEvents[i][2] = eventsJSONObject[i].topic;
            this.dayEvents[i][3] = eventsJSONObject[i].place;
            this.dayEvents[i][4] = eventsJSONObject[i].startTime;
            this.dayEvents[i][5] = eventsJSONObject[i].endTime;
        }
    }

    // Refresh the calendar.
    this.updateEventsTable(); 
}

/* Modify dialog allows user to either create a new event or modify an existing one. */
eventDialog.prototype.modifyDialog = function(parent, eDate, event) {

    this.id = "";

    this.dialog = new QDialog(this);
    this.dialog.resize(300,240);
    this.dialog.windowTitle = tr("Add Event");

    this.topicLabel = new QLabel(tr("Topic:"));
    this.topic = new QLineEdit;
    this.placeLabel = new QLabel(tr("Place:"));
    this.place = new QLineEdit;

    this.startTimeLabel = new QLabel(tr("Starting time:"));
    this.startTime = new QTimeEdit;
    this.startTime.setTime(new QTime(12, 0));
    this.startTime.displayFormat = "hh:mm";

    this.endTimeLabel = new QLabel(tr("Ending time:"));
    this.endTime = new QTimeEdit;
    this.endTime.setTime(new QTime(13, 0));
    this.endTime.displayFormat = "hh:mm";

    this.cancelButton = new QPushButton(tr("Cancel"));
    this.submitButton = new QPushButton(tr("Submit"));

    // Check if we are updating an event.
    if (event != "")
    {
        // Event found. Save the data.
        this.id = event[0];
        this.topic.setText(event[2]);
        this.place.setText(event[3]);
        timeObject = QTime.fromString(event[4], "hh:mm");
        this.startTime.setTime(timeObject);
        timeObject = QTime.fromString(event[5], "hh:mm");
        this.endTime.setTime(timeObject);
            
        this.dialog.windowTitle = tr("Modify Event");
    }

    this.dialogLayout = new QGridLayout(this.dialog);
    this.dialogLayout.addWidget(this.topicLabel, 0, 0);
    this.dialogLayout.addWidget(this.topic, 0, 1);
    this.dialogLayout.addWidget(this.placeLabel, 1, 0);
    this.dialogLayout.addWidget(this.place, 1, 1);
    this.dialogLayout.addWidget(this.startTimeLabel, 2, 0);
    this.dialogLayout.addWidget(this.startTime, 2, 1);
    this.dialogLayout.addWidget(this.endTimeLabel, 3, 0);
    this.dialogLayout.addWidget(this.endTime, 3, 1);
    this.dialogLayout.addWidget(this.cancelButton, 4, 0);
    this.dialogLayout.addWidget(this.submitButton, 4, 1);

    this.cancelButton["clicked()"].connect(this, this.cancelButtonClicked);
    this.submitButton["clicked()"].connect(this, this.submitButtonClicked);

    this.dialog.layout = this.dialogLayout;
    this.dialog.exec();
}

eventDialog.prototype.cancelButtonClicked = function() {
    this.dialog.close();
}

eventDialog.prototype.submitButtonClicked = function() {

    var baseStr = 'http://lively.cs.tut.fi:8010/SimpleCalendarEvent/' + this.id;
    
    var jsonStr = '{"eventDate":"' +this.dbDate + '","topic":"' +this.topic.text +'","place":"' +this.place.text +'","startTime":"' +this.startTime.text +'","endTime":"' +this.endTime.text +'"}';
    var jsonArr = new QByteArray(jsonStr);
    var req = new QNetworkRequest(new QUrl(baseStr));

    // QNetworkAccessManager for network requests
    var accessManager = new QNetworkAccessManager(this);
    accessManager["finished(QNetworkReply*)"].connect(this, this.saveEvent);
    
    if (this.id == "")
    {
        // Send the HTTP/POST request to create a new event-object.
        accessManager.post(req, jsonArr);
    }
    else
    {
        // Send the HTTP/PUT request to update the existing event-object.
        accessManager.put(req, jsonArr);
    }

    this.dialog.close();

}

eventDialog.prototype.saveEvent = function(reply) {
    var data = reply.readAll().toString();
    print(data);
}

// Returns the contents of QByteArray as a string
QByteArray.prototype.toString = function() {
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};

/* To use the application in standalone mode, remove the comment around this code
var wdgt = new QtSimpleCalendar();
wdgt.show();

QCoreApplication.exec(); */


