QtEvaluatorMorph.subclass("ChatMorph", {	
	initialize: function($super,rect) {
        $super(rect);	
		
        var inset = this.inset;
		this.addMorph(this.textEditor);
		this.pushButton.inner.text="Send";
		this.setRect(rect);

		this.textEditor.inner.plainText="txt";		
        this.textEditor.moveTo(inset, inset);
		this.pushButton.inner.pressed.connect(this, this.send);		
	
		// Init timers
		this.timer = new QTimer(this);
		this.timer.timeout.connect(this, this.pollMessages);
		this.timer.start(10000);
	},
	
	//timer calls this function for polling new messages
	pollMessages: function(event){
		this.getMessages();
	},

	//overwrites previous implementation
	send: function(foo) {
		this.postMessage(this.textEditor.inner.plainText);		
    },
	
	//overwrites previous implementation
	evaluateCode: function(foo){},
		
	//handles resizes
	setRect: function($super, rect) {
        $super(rect);

        var inset = this.inset;
        this.textEditor.setRect(qrect(0, 0, rect.width()* 0.75-inset, rect.height()  - inset*2), 0);
        this.pushButton.setRect(qrect(0, 0, rect.width()* 0.25-inset*2, rect.height() - inset*2), 0);

        this.textEditor.moveTo(inset, inset);
        this.pushButton.moveTo(inset + rect.width() * 0.75, inset);

        return this;
    },
	
	//gets messages from server
	getMessages: function (foo){
		var url = 'http://lively.cs.tut.fi:8010/Message/'; 
		var accessManager = new QNetworkAccessManager(this);
		accessManager["finished(QNetworkReply*)"].connect(this, this.showMessageReply);
		accessManager.get(new QNetworkRequest(new QUrl(url)));
	},

	//callback function for getMessages function
	showMessageReply: function(reply){
		var array=QPersevereAccessManager.parseReply(reply);

		var string = "";
		var to=this.parent.friendName;
		var from = QtLogin.userName;
		
		for (var i = 0; i < array.length; i ++){
			if ((array[i].from===from&&array[i].to===to)||(array[i].from===to&&array[i].to===from)){
				var name = array[i].from;
				var txt = array[i].text;
				string +='[' + name + ']' + txt +'\n';
			}
		}
		
		this.parent.showSpeechBubble(string);
	},
	
	//posts new message
	postMessage: function (txt){
		var url = 'http://lively.cs.tut.fi:8010/Message/';
		var accessManager = new QNetworkAccessManager(this);
		var message={};
		message.from=QtLogin.userName;
		message.to=this.parent.friendName;
		message.text=txt;
		var json = JSON.stringify(message);
		var data = new QByteArray(json);
		accessManager["finished(QNetworkReply*)"].connect(this, this.getMessages);
		accessManager.post(new QNetworkRequest(new QUrl(url)),data);
	}
});

QtAppMan.subclass("QtUserMan", {

    documentation: "a QtUserManMorph",
	chatPad: null,
	userIcon: new QIcon("images/appman/User.png"),
	userDownIcon: new QIcon("images/appman/User_down.png"),
	friendName:"",
	useDockPad: false,
	padsVisible: false,
	
	speechBubble: null,
	
	initialize: function($super, friendName) {
			$super();		 
			this.fillColor=this.homeGradient;
			//creates home button
			this.buildHome();
			QtAppMan.appMan=this;
		//sets pictures
		this.homeButton.inner.icon =this.userIcon;
		this.friendName=friendName;
		return this;
    },

	//shows speech bubble with text
	showSpeechBubble:function(txt){
		//removes old one
		if (this.speechBubble!==null){
			this.removeMorph(this.speechBubble);
			this.speechBubble=null;
			this.removeMorph(this.text);
			this.text=null;
		}
		var height = 140;
		var width=170;
		
		//creates bubble (yes, it's pushbutton, but you can not push it :)
		var button = new QPushButton(); 
		var mask = new QBitmap(new QPixmap("images/appman/speechBubbleMask.png").scaled(width,height));
		var speechBubbleIcon = new QIcon(new QPixmap("images/appman/speechBubble.png").scaled(width,height));

		button.setFixedSize(width,height);
		button.setIconSize(new QSize(width,height));
		button.icon = speechBubbleIcon;
		button.setMask(mask);

		this.speechBubble = new QtProxyMorph(button, qrect(0, 0, width, height),   0);

		this.speechBubble.moveTo(100-width/2, -height);
		
		this.speechBubble.setBorderWidth(0);
		this.speechBubble.setFillColor(QtColor.transparent);
		
		this.speechBubble.inner.borderWidth=0;
		this.speechBubble.inner.fillColor=QtColor.transparent;
	
		this.addMorph(this.speechBubble);

		//sets text
		var edit = new QPlainTextEdit(txt);
		edit.setFrameStyle(QFrame.NoFrame);
		this.text = new QtProxyMorph(edit, qrect(0, 0, width*8/10, height/2,   0));
		this.text.moveTo(100-width/2+width/10, -9*height/10);
		this.text.setBorderWidth(0);
		this.text.setFillColor(QtColor.transparent);
		this.text.inner.readOnly=true;
		//shows last line
		var maximum = 50;
		this.text.inner.verticalScrollBar().maximum=maximum;
		this.text.inner.verticalScrollBar().setValue(maximum-this.text.inner.verticalScrollBar().singleStep); 
	
		this.addMorph(this.text);
	},
	
	//builds launch pad for launching new applications and friend panels
	homePressed: function() {
		this.homeButton.inner.icon =this.userDownIcon;
	},

	//changes homebutton icon back to normal after user has released it
	homeReleased: function(foo) {
		if (this.padsVisible){
			if (this.useDockPad){this.removeMorph(this.dockPad);}
			this.removeMorph(this.chatPad);
			this.chatPad=null;
			this.dockPad=null;
			this.padsVisible = false;
			QtWorldMorph.current().removeMorph(this.ellipse);
			QtWorldMorph.current().removeMorph(this);
		}else{
			var dist1=this.homeSize*0.95+this.distFromBorder;
			var dist2=this.homeSize/2+this.distFromBorder-QtAppMan.padWidth/2;		
			//this.dockPad
			if (this.useDockPad){this.buildDockPad(dist1,dist2);}
			this.buildChatPad(dist1,dist2);
			this.padsVisible = true;
		}
		this.homeButton.inner.icon =this.userIcon;
	},
	
	//builds chat pad
	buildChatPad:function(dist1,dist2){
		if (this.chatPad===null){
			dist1=dist1-200;
			dist2=dist2+3*this.homeSize/4;
			this.chatPad = new ChatMorph(qrect(0, 0, 340, 90));
			this.chatPad.allowsManipulation(false);
			this.chatPad.setPos(qpt(dist1, dist2));
			that = this.addMorph(this.chatPad);
			this.chatPad.parent= this;
		}
	}
	
});
