/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

function tr(s) { 
	return s; 
}


/* ----- ZoomImageWidget definition ----- */

// A Widget that will show an image and zoom it
// A small map can be viewed over the zoomed in image.
function ZoomImageWidget(parent, webcam) {	
	QWidget.call(this,parent);
	
	this.styleSheet = "QWidget { background-color: rgba(50,50,50,50%); } ";
	
	this.MY_API_KEY = "ABQIAAAAjfRjTDOGscFuDUwMxJYaQRQUpi_xkJDYa0W0Jatk4awxKuvp7BQBfOyxYXrIsE8mc8pdYrHlaNjZjQ";
	this.RESIZE_FACTOR = 4;
	
	this.img = new QPixmap();
	
	this.map = new QPixmap();
	this.mapVisible = false;
	this.hideLetter = false;
	
	this.resizeTmr = new QTimer(this);
	this.resizeTmr.timeout.connect(this, this.resizeTmrTmout);
	
	this.setAttribute(Qt.WA_DeleteOnClose);
	this.cursor = new QCursor(Qt.PointingHandCursor);
	
	this.parent = parent;
	this.webcam = webcam;
	
	this.painterOpacity = 0.4;
	
	this.links = new QLabel('<style>p {color:  	#C0C0C0} a { color:  	#C0C0C0; }</style><p><a href="' + this.webcam.url + 
		'">Open in Webcams.travel</a>. Added by <a href="' + 
		this.webcam.user_url + '">' + 
		this.webcam.user + '</a></p>', this);
	this.links.linkActivated.connect(this, this.linkActivated);
	this.links.hide();
	
	this.loadImage('http://images.webcams.travel/webcam/' + webcam.webcamid + '.jpg');

}


ZoomImageWidget.prototype = new QWidget();

ZoomImageWidget.prototype.linkActivated = function(url) {
	QDesktopServices.openUrl(new QUrl(url)) 
};

ZoomImageWidget.prototype.resizeTmrTmout = function(url) {
	if(this.width > this.originalWidth) {
		this.resize(this.width-this.width/10,this.height-this.height/10);
	}
	else {
		this.resizeTmr.stop();
	}
};

ZoomImageWidget.prototype.loadMap = function(url) {
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.mapReplyFinished);
	accessManager.get(new QNetworkRequest((new QUrl(url))));
};

ZoomImageWidget.prototype.mapReplyFinished = function(reply) {
	var data = reply.readAll();
	if(this.mapVisible === true) {
		this.map.loadFromData(data, Qt.AutoColor);
	}
	this.update();
};

// Zoom the image and load a small map over the image
ZoomImageWidget.prototype.enterEvent = function(event) {
	this.raise();
	this.resize(this.originalWidth * this.RESIZE_FACTOR,this.originalHeight * this.RESIZE_FACTOR);
	if(this.map.isNull()) {
		this.loadMap('http://maps.google.com/staticmap?center=' + 
			this.webcam.latitude+','+this.webcam.longitude +
			'&zoom=13&size=100x100&maptype=mobile&'+'markers=' + 
			this.webcam.latitude+','+this.webcam.longitude +
			'&sensor=false&key=' + this.MY_API_KEY);
	}
	this.mapVisible = true;
	this.hideLetter = true;
	this.painterOpacity = 1;
	this.links.show();
};

// UnZoom the image
ZoomImageWidget.prototype.leaveEvent = function(event) {
	this.resizeTmr.start(5);
	this.map = new QPixmap();
	this.update();
	this.mapVisible = false;
	this.hideLetter = false;
	this.painterOpacity = 0.4;
	this.links.hide();
};

/* Load image frow the web */
ZoomImageWidget.prototype.loadImage = function(url) {
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.imageReplyFinished);
	accessManager.get(new QNetworkRequest((new QUrl(url))));
};

/* After the image is succesfully loaded scale the widget */
ZoomImageWidget.prototype.imageReplyFinished = function(reply) {
	var data = reply.readAll();
	this.img.loadFromData(data, Qt.AutoColor);
	
	this.resize((this.img.size().width())/this.RESIZE_FACTOR,(this.img.size().height())/this.RESIZE_FACTOR);
	this.originalWidth = this.width;
	this.originalHeight = this.height;
	
	this.parent.lineUpCameraOverlays();
};

/* When the widget is dragged with mouse, change widget position */
ZoomImageWidget.prototype.mouseMoveEvent = function(event) {
	var fGeom = this.parent.frameGeometry;
	var offsetY =  this.parent.frameGeometry.y() - this.parent.geometry.y();
	this.pos = new QPoint(event.globalX() - fGeom.x() - this.width/2, event.globalY() - fGeom.y() - this.height/2 + offsetY);
};

// This one needs to be defined to make double click event
// and mouse move event to work
ZoomImageWidget.prototype.mousePressEvent = function(event) {
};

// When the user double clicks the widget, the widget is hidden
ZoomImageWidget.prototype.mouseDoubleClickEvent = function(event) {
	this.hide();
};

// When the widget is resized paint the pictures again
// TODO: map is not always cleared if the used makes fast movements with the mouse
ZoomImageWidget.prototype.paintEvent = function (event) {
	var painter = new QPainter();
	painter.begin(this);
	
	if(!this.img.isNull()) {
		painter.setOpacity(this.painterOpacity);
		painter.drawPixmap(3,3,this.img.scaledToWidth(this.width-6,Qt.SmoothTransformation));
	}
	
	if(!this.map.isNull()) {		
		painter.setOpacity(0.5);
		painter.drawPixmap(this.width-this.map.size().width()-6,this.height-this.map.size().height()-6,this.map);	
	}

	if(this.webcam.letter !== undefined && this.hideLetter !== true) {
		painter.setOpacity(0.5);
		painter.setFont(new QFont ("Helvetica", 30, QFont.Bold));
		painter.drawText(new QPoint(this.rect.topLeft().x()+10,this.rect.topLeft().y()+35), this.webcam.letter);
	}
	
	this.links.pos = new QPoint(4, this.height-this.links.height);
	
	painter.end();
};

/* ----- FaderWidget definition ----- */

/* Fadein effect with a widget that destroys itself after a while */
function FaderWidget(parent, endAlpha) {
	QWidget.call(this, parent);
	
    this.startColor = Qt.black;
	this.currentAlpha = 255;
	if(endAlpha) {
		this.endAlpha = endAlpha;
	}
	else {
		this.endAlpha = 0;
	}
		
	// How long the effect will be
	this.duration = 500; 
	
	this.timer = new QTimer(this);
	this.timer.timeout.connect(this, this.update);
	
	// Delete widget when closed
	this.setAttribute(Qt.WA_DeleteOnClose);
	
	// Fill the full screen
	this.resize(parent.width, parent.height);
}



/* ----- FaderWidget definition ----- */

FaderWidget.prototype = new QWidget();

// Start the effect
FaderWidget.prototype.start = function() {
	this.currentAlpha = 255;
    this.timer.start(33);
    this.show();
};

// Doing the fadeout
FaderWidget.prototype.paintEvent = function(event) {
	var painter = new QPainter();
	painter.begin(this);
	var semiTransparentColor = new QColor(this.startColor);
	semiTransparentColor.setAlpha(this.currentAlpha);
	painter.fillRect(this.rect, semiTransparentColor);
	this.currentAlpha -= 255 * this.timer.interval / this.duration;
	//print(this.currentAlpha);
	painter.end();
	if (this.currentAlpha <= this.endAlpha) {
		this.timer.stop();
		this.close();
	}
};

/* Main application widget */
/* This application uses Webcams.Travel API and Google Maps API to show
	webcam images over a Google Map. */
function QtWebcamMap(parent) {
	QWidget.call(this, parent);
	
	// Application is disabled until webview is fully loaded
	this.setDisabled(true);
	
	// Google Maps API key
	// Get your own API key from Google: http://code.google.com/intl/fi-FI/apis/maps/signup.html
	this.MY_API_KEY = "ABQIAAAAjfRjTDOGscFuDUwMxJYaQRQUpi_xkJDYa0W0Jatk4awxKuvp7BQBfOyxYXrIsE8mc8pdYrHlaNjZjQ";
	
	// Webcams.traver devid
	this.WT_DEVID = "0ada714bb65ef9c8ff81f4ca329e06dc";
	this.WT_REST_REQ = "http://api.webcams.travel/rest?method=";
	
	// Array to hold the small images over the main map
	this.camArray = [];
	
	// QWebView shows a web page with full screen Google Map
	this.view = new QWebView(this);
	
	// lineEdit and buttons are in the upper row
	this.lineEdit = new QLineEdit(this);
	this.goButton = new QPushButton(tr("Go"), this);
	this.goButton.setDisabled(true);
	this.searchButton = new QPushButton(tr("Search from map"), this);
	this.lineUpButton = new QPushButton(tr("Line up"), this);
	this.lineUpButton.setDisabled(true);
	
	// Use timer to check if user has clicked a marker or not
	this.cameraTimer = new QTimer(this);
	this.cameraTimer.timeout.connect(this, this.timerTimeout);

	// upper row widget connections
	this.lineUpButton.clicked.connect(this, this.lineUpCameraOverlays);
	this.goButton.clicked.connect(this, this.searchAddress);
	this.lineEdit.returnPressed.connect(this, this.searchAddress);
	this.lineEdit["textChanged(QString)"].connect(this, function () { 
		if(this.lineEdit.text === "") {
			this.goButton.setDisabled(true);
		}
		else {
			this.goButton.setEnabled(true);
		}
	});
	this.searchButton.clicked.connect(this, this.searchFromMapCenter);

	// let's do some initialization after view has loaded its web page
	this.view["loadFinished(bool)"].connect(this, this.loadFinished);

	// Complex layout definitions
	this.layout = new QVBoxLayout(this);
	this.hboxLayout = new QHBoxLayout();
	
	// Add widgets to the layout
	this.hboxLayout.addWidget(new QLabel("Jump to:"), 0, 0);
	this.hboxLayout.addWidget(this.lineEdit, 0, 0);
	this.hboxLayout.addWidget(this.goButton, 0, 0);
	this.hboxLayout.addWidget(this.searchButton, 0, 0);
	this.hboxLayout.addWidget(this.lineUpButton, 0, 0);
		
	this.layout.addLayout(this.hboxLayout);
	this.layout.addWidget(this.view, 1, 0);
	//this.layout.addLayout(this.smallImageLayout);

	this.statusLbl = new QLabel("<h3>Use \'Search from map\' button to find cameras.</h3>", this);
	this.statusLbl.minimumSize = new QSize(400,40);
	//this.statusLbl.maximumWidth = 800;
	//this.statusLbl.width = 600;
	this.statusLbl.alignment = Qt.AlignCenter;
	this.statusLbl.openExternalLinks = true;
	
	// Try to load a web page with a Google Map
	try {
		this.view.load(new QUrl("http://www.cs.tut.fi/~salmin39/map/index.html"));
	} catch(err) {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Error: " + err;
		messageBox.show();
	}
	
	this.windowTitle = tr("QtWebcamMap");
	this.resize(800,480);
	
	// Nice effect
	this.darkFader = new FaderWidget(this,0);
	this.darkFader.start();
}

QtWebcamMap.prototype = new QWidget();

/* Set map center and zoom */
QtWebcamMap.prototype.setCenter = function(lat,lng,zoom) {
	var scriptString = 	"map.setCenter(new GLatLng(" + lat + "," + lng + ")," + zoom + ");";
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};


/* If user has clicked a marker we should show marker related weather
 * information if it is available 
 */
QtWebcamMap.prototype.timerTimeout = function() {
	// get the marker
	var marker = this.getUserLastClickedMarker();
	this.view.page().mainFrame().evaluateJavaScript("userClickedMarker = undefined;");
	if(marker !== undefined && marker.camID !== undefined) {
		try{
			var webcam = {
				latitude: marker.latitude,
				longitude: marker.longitude,
				letter: marker.letter.toString(),
				webcamid: marker.camID
			};
			
			var cam = new ZoomImageWidget(this, webcam);
		
			if(Math.random() > 0.5) {
				xPos = Math.floor((this.width/2-250)*Math.random()+10);
			}
			else {
				xPos = Math.floor((this.width/2-250)*Math.random()+this.width/2+50);
			}

			cam.pos = new QPoint(xPos,
				Math.floor((this.height-300)*Math.random()+50));	
			cam.show();
			this.camArray.push(cam);
		}
		catch(err) {
			print(err);
		}
	}
};

/* After the web page has been fully loaded we can do some
 * initialization and other stuff. If this is done too soon it will
 * not work at all */
QtWebcamMap.prototype.loadFinished = function(ok) {	
	var scriptString = 
	"var userClickedPoint = undefined;" +
	"var userClickedMarker = undefined;" +
	"var myClickListener = undefined;" +
	//"map.addControl(new GMapTypeControl());" +
	"map.setCenter( new GLatLng(0,0),1 );" +
	"var markers = new Array();" +
	//"var mm = new GMarkerManager(map, {borderPadding:1});" +
	"map.disableDoubleClickZoom();" +
	"var topRight = new GControlPosition(G_ANCHOR_TOP_RIGHT, new GSize(10,10));"+
	"map.addControl(new GLargeMapControl(), topRight);" +
	"myClickListener = GEvent.addListener(map, \"click\", function(marker, point) {" +
    	"userClickedPoint = point;" +
    	"userClickedMarker = marker;" +
	"});" +
	"GEvent.addListener(map, \"moveend\", function() {" +
  		"userClickedPoint = undefined;" +
  		"userClickedMarker = undefined;" +
	"});" +

	// Customizes lettered icons
	'var baseIcon = new GIcon(G_DEFAULT_ICON);' +
	'baseIcon.shadow = \"http://www.google.com/mapfiles/shadow50.png\";'+
	'baseIcon.iconSize = new GSize(20, 34);'+
	'baseIcon.shadowSize = new GSize(37, 34);'+
	'baseIcon.iconAnchor = new GPoint(9, 34);'+
	'baseIcon.infoWindowAnchor = new GPoint(9, 2);'+

	'function createMarker(point, index, infoHtml, camID) {'+
		'var letter = String.fromCharCode(\"A\".charCodeAt(0) + index);'+
		'var letteredIcon = new GIcon(baseIcon);'+
		'letteredIcon.image = \"http://www.google.com/mapfiles/marker\" + letter + \".png\";'+

		'markerOptions = { icon:letteredIcon, title:infoHtml.toString() };'+
		'var marker = new GMarker(point, markerOptions);'+
		'marker.letter = letter;'+
		'marker.camID = camID;' +
		'marker.latitude = marker.getPoint().lat();' +
		'marker.longitude = marker.getPoint().lng();' +
		'return marker;'+
	'}';
	
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
	
	
	// Enable whole application
	this.setEnabled(true);
};

QtWebcamMap.prototype.getMapCenter = function() {
	return this.view.page().mainFrame().evaluateJavaScript("map.getCenter()");
};

QtWebcamMap.prototype.getUserLastClickedMarker = function() {
	return this.view.page().mainFrame().evaluateJavaScript("userClickedMarker;");
};

/* Close all camera related overlays on the map */
QtWebcamMap.prototype.clearCameraOverlays = function() {
	// Close old open camera images
	while(this.camArray.length > 0) {
		this.camArray.pop().close();
	}
		
	// Clear old markers
	this.view.page().mainFrame().evaluateJavaScript("map.clearOverlays();");
};

/* Adjust camera overlay positions to form of a grid */
QtWebcamMap.prototype.lineUpCameraOverlays = function() {
	var xPos = this.view.pos.x() + 10;
	var yPos = this.view.pos.y() + 10;
	var heightiestHeight = 0;
	
	for(var i = 0; i < this.camArray.length; i = i+1) {
		this.camArray[i].pos = (new QPoint(xPos,yPos));
		xPos += this.camArray[i].width + 5;
		if(this.camArray[i].height > heightiestHeight) {
			heightiestHeight = this.camArray[i].height;
		}
		if(i + 1 < this.camArray.length){
			if((xPos + this.camArray[i+1].width + 5) > this.view.width){
				yPos += heightiestHeight + 5;
				heightiestHeight = 0;
				xPos = this.view.pos.x() + 10;
			}
		}
	}
};

/* Fetch camera information around map center */
QtWebcamMap.prototype.getCams = function() {
	var mapCenter = this.getMapCenter();
	
	// Get camera data from Webcams Travel (http://fi.webcams.travel/)
	requestStr = this.WT_REST_REQ + "wct.webcams.list_nearby&devid=" + 
		this.WT_DEVID + 
		"&lat=" + mapCenter.y.toString() + 
		"&lng=" + mapCenter.x.toString() + 
		"&radius=10&unit=km&per_page=50&format=json";
		
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, this.getCamsCallback);
	accessManager.get(new QNetworkRequest(new QUrl(requestStr)));
};

/* Process information from wct.webcams */
QtWebcamMap.prototype.getCamsCallback = function(reply) {
	var replyString = reply.readAll().toString();
	var replyJSON = undefined;
	replyJSON = eval('(' + replyString + ')');
	if(replyJSON.status !== "ok") {
		var messageBox = new QMessageBox(this);
		messageBox.text = "Webcams.travel error!\n" + 
			"Status: " + replyJSO.status + "\n" + 
			"Description: " + replyJSON.error.description;
		messageBox.show();
		return;
	}		
	if (replyJSON.webcams.count === 0) {
		this.clearCameraOverlays();
		this.statusLbl.text = '<h3>No webcams</h3>';
		this.lineUpButton.setDisabled(true);
	}
	else if (replyJSON.webcams.count > 0) {
		this.clearCameraOverlays();
		
		// Only 15 cameras at a time
		if(replyJSON.webcams.count > 20) {
			replyJSON.webcams.count = 20;
		}
		
		this.statusLbl.text = replyJSON.webcams.count.toString() + ' webcams found<br><b><a href="http://webcams.travel">Webcams provided by webcams.travel</a></b>';	
		
		// For each camera: insert marker to the main map
		// and show webcam image on separate widget over the main map
		for(var i = 0; i < replyJSON.webcams.count; i=i+1) {
			this.insertMarker(replyJSON.webcams.webcam[i].latitude,
				replyJSON.webcams.webcam[i].longitude, 
				replyJSON.webcams.webcam[i].title,
				replyJSON.webcams.webcam[i].webcamid, i);
			
			replyJSON.webcams.webcam[i].letter = String.fromCharCode("A".charCodeAt(0) + i);
				
			var cam = new ZoomImageWidget(this, replyJSON.webcams.webcam[i]);
			
			// Position the widget on the main map randomly 
			// like in this clarifying picture here: [X| |X]
			// Right or left side of the map
			if(Math.random() > 0.5) {
				xPos = Math.floor((this.width/2-250)*Math.random()+10);
			}
			else {
				xPos = Math.floor((this.width/2-250)*Math.random()+this.width/2+50);
			}
	
			cam.pos = new QPoint(xPos,
				Math.floor((this.height-300)*Math.random()+50));	
			
			cam.show();
			this.camArray.push(cam);
		}
		
		// Start a timer to check if the user has clicked a marker
		this.cameraTimer.start(500);
		this.lineUpButton.setEnabled(true);
	}
	
};

/* Geocode a location string */
QtWebcamMap.prototype.geoCode = function(locationString, callback) {
	requestStr = "http://maps.google.com/maps/geo?q=" +
		locationString + "&output=csv&key=" + this.MY_API_KEY;
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, callback);
	accessManager.get(new QNetworkRequest(new QUrl(requestStr)));
};

/* Insert new marker to the map */
QtWebcamMap.prototype.insertMarker = function(lat,lng,infoHtml,camID,i) {
	var scriptString =
	'var newMarker = createMarker(new GLatLng(' + lat + ',' + lng + '),' + i + ',\"' + infoHtml + '\",'+ camID +');'+
	'markers.push(newMarker);' +
	'map.addOverlay(markers[markers.length-1]);';
	this.view.page().mainFrame().evaluateJavaScript(scriptString);
};

/* Geocoding an address */
QtWebcamMap.prototype.searchAddress = function() {
	if(this.lineEdit.text !== "") {
		this.geoCode(this.lineEdit.text, this.searchAddressCallback);
	}
	else {
		this.getCams();
	}
};

QtWebcamMap.prototype.searchAddressCallback = function(reply) {
	var replyString = reply.readAll().toString();
	var geoCodeArray = [];
	geoCodeArray = replyString.split(",");
	var htmlResponse = geoCodeArray[0];
	var accuracy = geoCodeArray[1];
	var lat = geoCodeArray[2];
	var lng = geoCodeArray[3];
	if(htmlResponse === "200") {
		this.setCenter(lat, lng, parseInt(accuracy,10) + 7);
		this.getCams();
		this.lineEdit.text = "";
	}
};

QtWebcamMap.prototype.searchFromMapCenter = function() {
	this.getCams();
};

QtWebcamMap.prototype.paintEvent = function() {
	this.statusLbl.pos = new QPoint((this.width/2)-this.statusLbl.width/2, this.height-this.statusLbl.height-10);
	/*var painter = new QPainter();
	painter.begin(this);
	painter.setWindow(this.rect);
	painter.setViewport(this.rect);
	painter.drawLine(new QPoint(0,0), new QPoint(this.width, this.height));
	painter.end();*/
};

QtWebcamMap.prototype.resizeEvent = function() {
	this.lineUpCameraOverlays();
};


/* After image is succesfully loaded, scale it and show it */
QtWebcamMap.prototype.addSmallImageReplyFinished = function(reply) {
	var data = reply.readAll();
	var imagePixmap = new QPixmap();
	var newLabel = new QLabel(this);
	// mouseDoubleClickEvent
	imagePixmap.loadFromData(data, 0, Qt.AutoColor);
	if(!imagePixmap.isNull()) {
		imagePixmap = imagePixmap.scaledToWidth(40,Qt.SmoothTransformation);
		newLabel.setPixmap(imagePixmap);
		//~imagePixmap;
	}
	else {
		/* imagePixmap was null, image data was not available (bad url)*/
		newLabel.text = '<font color="white"><center>No image available</center></font>';
	}
	this.smallImageLayout.addWidget(newLabel, 0, Qt.AlignCenter);
};



QByteArray.prototype.toString = function()
{
   ts = new QTextStream( this, QIODevice.ReadOnly );
   return ts.readAll();
};

/* To use the application in standalone mode, remove the comment around this code 

var map = new QtWebcamMap();
map.show();

QCoreApplication.exec();
*/
