/*==============================================================================
 * Genesis
 *============================================================================*/

// Create the main window with its view and scene
GlobalWindow = new MainWindow();

// Create a gradient color for the world background
var wgrad = new QLinearGradient(0, 0, 0, QtWorldMorph.defaultExtent.y);
wgrad.setColorAt(0, new QColor("white"));
wgrad.setColorAt(1, new QColor(0x00, 0x99, 0xFF));
wgrad.setSpread(QGradient.ReflectSpread);

// Create the world and display it on the scene
var world = new QtWorldMorph(null, wgrad);
world.setCurrent();

/*==============================================================================
 * Sample Objects and Applications
 *============================================================================*/

// Create sample QtMorphs
var rect0 = new QtRectangleMorph(qrect(0, 0, 100, 30));
rect0.setPos(qpt(200, 170));
world.addMorph(rect0);

// IMPORTANT: Set the initial value of the 'that' pointer!
that = rect0;

var rect1 = new QtRectangleMorph(qrect(0, 0, 100, 30));
rect1.setPos(qpt(230, 190));
world.addMorph(rect1);

var rect2 = new QtRectangleMorph(qrect(0, 0, 100, 30));
rect2.setPos(qpt(50, 180));
world.addMorph(rect2);

var rect3 = new QtRectangleMorph(qrect(-50, -15, 100, 30), QtColor.green, 2, QtColor.black);
rect3.setPos(qpt(-25, -10)); // WAS: 400, 220
rect3.setOpacity(0.75);
// world.addMorph(rect3);

var rect4 = rect2.copy();
rect2.addMorph(rect4);
rect4.setPos(qpt(15, 15)).rotate(5).setFillColor(new QColor("indigo"));
// rect2.setFillColor(QtColor.darkCyan);

var rect5 = rect4.copy();
rect4.addMorph(rect5);
rect5.setPos(qpt(15, 15));

// Gradient color for the rectangle above
var gradient4 = new QLinearGradient(0, 0, 100, 0);
gradient4.setColorAt(0, new QColor(Qt.blue));
gradient4.setColorAt(1, new QColor(Qt.red));
gradient4.setSpread(QGradient.ReflectSpread);
rect5.setFillColor(gradient4);

var ellipse1 = new QtEllipseMorph(qrect(-50, -20, 100, 40), QtColor.transparent, 3, QtColor.blue);
ellipse1.setPos(qpt(800, 50));
world.addMorph(ellipse1);

var ellipse2 = new QtEllipseMorph(qrect(0, 0, 40, 40), QtColor.red, 2, QtColor.blue);
ellipse2.setOpacity(0.9);
ellipse2.setPos(qpt(0, 0));
ellipse1.addMorph(ellipse2);
ellipse1.addMorph(rect3);

var clock = new QtClockMorph();
clock.moveTo(-20, -55);
ellipse1.addMorph(clock);

var calendar = new QtProxyMorph(new QCalendarWidget(), qrect(0, 0, 250, 200), 6);
calendar.moveTo(360, 290);
calendar.shear(0.2, 0);
world.addMorph(calendar);

var slider = new QtProxyMorph(new QSlider(/*Qt.Horizontal*/), qrect(0, 0, 25+10, 100+10), 5);
slider.setPos(qpt(400, 170));
world.addMorph(slider);

var spinbox = new QtProxyMorph(new QSpinBox(), qrect(0, 0, 60+10, 30+10), 5);
spinbox.setPos(qpt(700, 230));
world.addMorph(spinbox);

var dial = new QtProxyMorph(new QDial(), qrect(0, 0, 100+10, 100+10), 5);
dial.inner.notchesVisible = true;
dial.setPos(qpt(550, 170));
world.addMorph(dial);

var progressbar = new QtProxyMorph(new QProgressBar(), qrect(0, 0, 200+10, 25+10), 5);
progressbar.setPos(qpt(700, 170));
world.addMorph(progressbar);

var scrollbar = new QtProxyMorph(new QScrollBar(/*Qt.Horizontal*/), qrect(0, 0, 25+10, 100+10), 5);
scrollbar.setPos(qpt(475, 170));
world.addMorph(scrollbar);

// Connect the widget value changes to each other
slider.inner.valueChanged.connect(this, spinbox.inner.setValue);
spinbox.inner['valueChanged(int)'].connect(this, dial.inner.setValue); // Qt 4.5 QSpinBox has two different 'valueChanged' methods!
dial.inner.valueChanged.connect(this, progressbar.inner.setValue);
progressbar.inner.valueChanged.connect(this, scrollbar.inner.setValue);
scrollbar.inner.valueChanged.connect(this, slider.inner.setValue);

// Set initial value for the widgets above
spinbox.inner.setValue(10);

var dateEdit = new QtProxyMorph(new QDateEdit(), qrect(0, 0, 100+10, 30+10), 5);
dateEdit.setPos(qpt(800, 230));
world.addMorph(dateEdit);

var evaluator = new QtEvaluatorMorph(qrect(0, 0, 250, 145));
evaluator.moveTo(10, 250);
world.addMorph(evaluator);

var qWebBrowser = new QWebView();
qWebBrowser.resize(600, 400);
var webBrowser = new QtProxyMorph(qWebBrowser, null, 8);
webBrowser.moveTo(220, 430);
webBrowser.inner.load(new QUrl("http://www.cnn.com/"));
world.addMorph(webBrowser);

// Create a button that opens up all the mashup demos
var mashButton = new QtProxyMorph(new QPushButton("Open Mashup Demos"), qrect(0, 0, 170+10, 30+10), 5);
mashButton.moveTo(10, 425);
mashButton.setFillColor(QtColor.darkRed);
mashButton.inner.pressed.connect(this, QtEvents.openMashupDemos);
world.addMorph(mashButton);

// Define a subworld
var subworld = world.addSubworld(qpt(10, 475), qrect(0, 0, 160, 40), "Enter Subworld 1", "Back to Main World");

// Create a gradient color for the subworld background
var subgrad = new QLinearGradient(0, 0, 0, QtWorldMorph.defaultExtent.y);
subgrad.setColorAt(0, new QColor("white"));
subgrad.setColorAt(1, new QColor("cyan"));
subgrad.setSpread(QGradient.ReflectSpread);
subworld.setFillColor(subgrad);

// Define another subworld
var subworld2 = world.addSubworld(qpt(10, 525), qrect(0, 0, 160, 40), "Enter Subworld 2", "Back to Main World");
subgrad.setColorAt(1, new QColor("salmon")); // Modify gradient
subworld2.setFillColor(subgrad);

// Open an inspector also
/* Commented out because of a bug in inspector with Qt 4.6 that slows system down */
/*
try {
    if (QtInspector) {
        var inspector = new QtProxyMorph(new QtInspector());
        inspector.moveTo(700, 220);
        inspector.setRect(qrect(0, 0, 600, 500));
        world.addMorph(inspector);
    }
} catch(e) {
    // Avoid errors if the inspector isn't available
}
*/

// Open an application loader widget also
try {
    if (QtAppLoader) {
        var appLoaderWdgt = new QtProxyMorph(new QtAppLoader());
        appLoaderWdgt.moveTo(150, 120);
        appLoaderWdgt.setRect(qrect(0, 0, 600, 40));
        world.addMorph(appLoaderWdgt);
    }
} catch(e) {
    // Avoid errors if the inspector isn't available
}

// Open a few demos if they have been loaded
try {
    if (QtCubeCrashWithScores) {
        var gameWidget = new QtProxyMorph(new QtCubeCrashWithScores(), qrect(0, 0, 250, 340), 5);
        gameWidget.moveTo(900, 400);
        world.addMorph(gameWidget);
    }
} catch(e) {}; // Squelch errors!

try {
    if (QtWeather) {
        var weatherWidget = new QtProxyMorph(new QtWeather(), qrect(0, 0, 300, 420));
        weatherWidget.moveTo(950, 40);
		weatherWidget.rotate(5);
        world.addMorph(weatherWidget);
    }
} catch(e) {}; // Squelch errors!

try {
    if (QtProjectCalendar) {
        var projectCal = new QtProxyMorph(new QtProjectCalendar(), qrect(0, 0, 700, 520));
        projectCal.moveTo(150, 200);
		projectCal.rotate(-10);
        world.addMorph(projectCal);
    }
} catch(e) {}; // Squelch errors!

/* // not in use
try {
    if (QtLogin) {
        var qtlogin = new QtProxyMorph(new QtLogin(), qrect(0, 0, 340, 150));
        qtlogin.moveTo(10, 575);
        world.addMorph(qtlogin);
    }
} catch(e) {}; // Squelch errors!
*/

try {
    if (QtAppMan) {
		var appman = new QtAppMan();
        world.addMorph(appman);
    }
} catch(e) {}; // Squelch errors!

// Default timer behavior
function doStepping() {
    ellipse1.rotate(-6);
    rect3.rotate(10);
}

//QtAppMan.minimize(inspector);
QtAppMan.minimize(gameWidget);
//QtAppMan.minimize(weatherWidget);
QtAppMan.minimize(webBrowser);
QtAppMan.minimize(projectCal);

// Start timers automatically
QtEvents.toggleTimers();

/*==============================================================================
 * Qt startup
 *============================================================================*/

QCoreApplication.exec();


